/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 *
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by the Free Software Foundation; either version 3
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
 *  the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  if not, see <http://www.gnu.org/licenses/>.
 *
 *  ************************************************************************************************
 *
 *  SVN Status Information:
 *
 *      $Rev:: 275                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-05 12:47:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 *
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file initailize_core_module.c
 *   @brief This is a template of a core module initialization
 *
 *   It contains only the orion_Init as a template<br/>
 *   to use for your modules (copy & paste)
 */
/*  ************************************************************************************************  */


/* Initialize a module for commands like /ns "set display" NEWDISPLAY (Command has TWO keywords) */

/**
 * @brief Initialize the module in the orion DB
 *
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"set keyword[1]","set keyword[1]");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addLeastModuleLevels(thread,&Module,"set",is_services_user,is_services_user); /* just to add a displaylevel for /msg ns help
															  this should always be the lowest used Module level! */

	addCommand(thread,&Module,"set keyword1",ns_set_keyword1,is_services_user,is_services_user,4,5);
	addCommand(thread,&Module,"set keyword2",ns_set_keyword2,is_services_user,is_services_user,4,5);

	addCommand(thread,&Module,"help",ns_set_help_listing,is_services_user,is_services_user,3,3);
	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/* Initialize a module for commands like /ns "register" .... (Command has ONE keyword) */

/**
 * @brief Initialize the module in the orion DB
 *
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"keyword","keyword");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	/* :shock PRIVMSG NickServ :keyword param[4-..] */
	addCommand(thread,&Module,"keyword",ns_keyword,is_services_user,is_services_user,4,5);
	addCommand(thread,&Module,"help keyword",ns_keyword_help,is_services_user,is_services_user,4,5);
	addCommand(thread,&Module,"help",ns_keyword_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */
