/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 272                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-16 19:39:52 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file datatypes.h
 *   @brief This is the datatypes header.
 *
 *   It contains all datatypes (used all over the program).
 */
/*  ************************************************************************************************  */

#ifndef WEBFORM_H
#define WEBFORM_H

#include "datatypes.h"

Socket *addWebSocket(orion_socket_t Sock);
void    addWebSockets(int fdmax);
void    delWebSocket(Socket *s);
void    delWebSockets(void);
Socket *FindWebSocket(orion_socket_t Sock);

void WebSocketClose(Threads *thread, Socket *s);
bool WebCheckIp(Socket *s);

void *WebSocket (void *in_thread);

/* Translate signales to IRC and Web WRC ( Web Relay Chat ) */
PROTOCOL_BOOL orion_WRCtranslate_cmd_initialize(Threads *thread);
PROTOCOL_BOOL orion_WRCtranslate_cmd_add_account(Threads *thread, char *uid, char *authnick);
PROTOCOL_BOOL orion_WRCtranslate_cmd_del_account(Threads *thread, char *uid);

PROTOCOL_BOOL orion_WRCtranslate_cmd_topic(Threads *thread, char *whosets, char *chan, char *whosetit, char *topic, time_t when);

PROTOCOL_BOOL orion_WRCtranslate_cmd_gline(Threads *thread, char *user, char *host, char *who, time_t when, time_t expires, char *reason);
PROTOCOL_BOOL orion_WRCtranslate_cmd_ungline(Threads *thread, char *unsetter, char *host, char *user, time_t time_in);

PROTOCOL_BOOL orion_WRCtranslate_cmd_svskill(Threads *thread, char *source, char *user, char *buf);
PROTOCOL_BOOL orion_WRCtranslate_cmd_kill(Threads *thread, char *source, char *user, char *buf);
PROTOCOL_BOOL orion_WRCtranslate_cmd_rawsvsmode(Threads *thread, char *source, char *nick, char *modes);
PROTOCOL_BOOL orion_WRCtranslate_cmd_svsmode(Threads *thread, char *nick, char *target, char *modes);
PROTOCOL_BOOL orion_WRCtranslate_cmd_svs2mode(Threads *thread, char *nick, char *target, char *modes);
PROTOCOL_BOOL orion_WRCtranslate_cmd_nick(Threads *thread, char *nick, char *ident, char *host, char *name, char *modes);
PROTOCOL_BOOL orion_WRCtranslate_cmd_mode(Threads *thread, char *source, char *dest, time_t time_in, char *buf);
PROTOCOL_BOOL orion_WRCtranslate_cmd_rawmode(Threads *thread, char *source, char *dest, time_t time_in, char *buf);

PROTOCOL_BOOL orion_WRCtranslate_cmd_kick(Threads *thread, char *source, char *chan, char *user, char *buf);
PROTOCOL_BOOL orion_WRCtranslate_cmd_notice(Threads *thread, char *source, char *dest, char *buf);
PROTOCOL_BOOL orion_WRCtranslate_cmd_privmsg(Threads *thread, char *source, char *dest, char *buf);
PROTOCOL_BOOL orion_WRCtranslate_cmd_serv_notice(Threads *thread, char *source, char *dest, char *msg);
PROTOCOL_BOOL orion_WRCtranslate_cmd_serv_privmsg(Threads *thread, char *source, char *dest, char *msg);

PROTOCOL_BOOL orion_WRCtranslate_cmd_quit(Threads *thread, char *source, char *buf);
PROTOCOL_BOOL orion_WRCtranslate_cmd_protoctl(Threads *thread);
PROTOCOL_BOOL orion_WRCtranslate_cmd_pass(Threads *thread, char *pass);
PROTOCOL_BOOL orion_WRCtranslate_cmd_server(Threads *thread, char *servname, int hop, char *descript);
PROTOCOL_BOOL orion_WRCtranslate_cmd_pong(Threads *thread, char *servname, char *who);
PROTOCOL_BOOL orion_WRCtranslate_cmd_ping(Threads *thread, char *servname, char *who);
PROTOCOL_BOOL orion_WRCtranslate_cmd_join(Threads *thread, char *user, char *channel, time_t chantime);
PROTOCOL_BOOL orion_WRCtranslate_cmd_services_join(Threads *thread, char *user, char *channel, time_t chantime, int fake);

PROTOCOL_BOOL orion_WRCtranslate_cmd_chghost(Threads *thread, char *source, char *nick, char *vhost);
PROTOCOL_BOOL orion_WRCtranslate_cmd_chgident(Threads *thread, char *source, char *nick, char *vIdent);
PROTOCOL_BOOL orion_WRCtranslate_cmd_invite(Threads *thread, char *source, char *chan, char *nick);
PROTOCOL_BOOL orion_WRCtranslate_cmd_part(Threads *thread, char *nick, char *chan, char *buf);

PROTOCOL_BOOL orion_WRCtranslate_cmd_wallops(Threads *thread, char *source, char *fmt);
PROTOCOL_BOOL orion_WRCtranslate_cmd_global(Threads *thread, char *source, char *buf);

PROTOCOL_BOOL orion_WRCtranslate_cmd_sqline(Threads *thread, char *sender, char *mask, time_t time_in, time_t expire, char *reason);
PROTOCOL_BOOL orion_WRCtranslate_cmd_unsqline(Threads *thread, char *servername, char *nick, char *snick);

PROTOCOL_BOOL orion_WRCtranslate_cmd_squit(Threads *thread, char *servname, char *message);
PROTOCOL_BOOL orion_WRCtranslate_cmd_svso(Threads *thread, char *source, char *nick, char *flag);
PROTOCOL_BOOL orion_WRCtranslate_cmd_chg_nick(Threads *thread, char *oldnick, char *newnick);
PROTOCOL_BOOL orion_WRCtranslate_cmd_svsnick(Threads *thread, char *source, char *oldnick, char *newnick, time_t when);
PROTOCOL_BOOL orion_WRCtranslate_cmd_connect(Threads *thread, bool how);
PROTOCOL_BOOL orion_WRCtranslate_cmd_svshold(Threads *thread, char *nick);
PROTOCOL_BOOL orion_WRCtranslate_cmd_release_svshold(Threads *thread, char *nick);

PROTOCOL_BOOL orion_WRCtranslate_cmd_szline(Threads *thread, char *servername, char *mask, char *snick, time_t when, time_t expire, char *reason);
PROTOCOL_BOOL orion_WRCtranslate_cmd_unszline(Threads *thread, char *servername, char *mask, char *snick, time_t in_time);

PROTOCOL_BOOL orion_WRCtranslate_cmd_unban(Threads *thread, char *name, char *nick);
PROTOCOL_BOOL orion_WRCtranslate_cmd_svsmode_chan(Threads *thread, char *name, char *mode, char *nick);
PROTOCOL_BOOL orion_WRCtranslate_cmd_svsjoin(Threads *thread, char *source, char *nick, char *chan, time_t date);
PROTOCOL_BOOL orion_WRCtranslate_cmd_svspart(Threads *thread, char *source, char *nick, char *chan);
PROTOCOL_BOOL orion_WRCtranslate_cmd_sapart(Threads *thread, char *source, char *nick, char *chan);
PROTOCOL_BOOL orion_WRCtranslate_cmd_sajoin(Threads *thread, char *source, char *nick, char *chan, time_t date);

PROTOCOL_BOOL orion_WRCtranslate_cmd_swhois(Threads *thread, char *source, char *who, char *mask);
PROTOCOL_BOOL orion_WRCtranslate_cmd_eob(Threads *thread);
PROTOCOL_BOOL orion_WRCtranslate_cmd_jupe(Threads *thread, char *jserver, char *who, char *reason);
PROTOCOL_BOOL orion_WRCtranslate_cmd_global_legacy(Threads *thread, char *source, char *fmt);

PROTOCOL_BOOL orion_WRCtranslate_cmd_raw(Threads *thread, char *param[]);
bool orion_WRCtranslate_valid_chan(Threads *thread, char *chan);
bool orion_WRCtranslate_valid_nick(Threads *thread, char *nick);
bool orion_WRCtranslate_flood_mode_check(Threads *thread, char *value);
#endif
