/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file bs_info.c
 *   @brief This is the Module file for BotServ INFO Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the BotServ INFO command.
 *
 *   The BotServ INFO Command allows the user to dump<br/>
 *   some Information about a #channel or Bot.
 *
 *   Syntax is: /msg BotServ INFO { channel | botnick }
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "bs"
#define MODUL "info"
#define MODUL_FILE "bs_info"
#define MODUL_VERSION "1.0"

MOD_BOOL bs_info(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_info_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_info_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"info","info");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"info",bs_info,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help info",bs_info_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",bs_info_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the BotServ INFO Command.
 * 
 * The BotServ INFO Command allows the user to dump<br/>
 *  some Information about a #channel or Bot.
 *  
 * Syntax is: /msg BotServ INFO { channel | botnick }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_info(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	deepSQL MySQL;
	int i=0;
	
	/* Initialise Variables to (NULL) */
	bzero(&MySQL,sizeof(MySQL));
	
	/*
	 * Info Command
	 */
 	//:shock PRIVMSG BotServ :info #deepirc
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"info")){
        /*
        (14:58) -BotServ- Information for channel #DeepIRC:
        (14:58) -BotServ- Bot nick : DeepIRC
        (14:58) -BotServ- Bad words kicker : enabled
        (14:58) -BotServ- Bolds kicker : disabled
        (14:58) -BotServ- Caps kicker : enabled (minimum 25/75%)
        (14:58) -BotServ- Colors kicker : disabled
        (14:58) -BotServ- Flood kicker : enabled (8 lines in 1s)
        (14:58) -BotServ- Repeat kicker : enabled (3 times)
        (14:58) -BotServ- Reverses kicker : disabled
        (14:58) -BotServ- Underlines kicker : disabled
        (14:58) -BotServ- Options : Fantasy, Greet, Symbiosis*/
		if(param[4][0]=='#'){
			/* Updates the Threads Channel Management */
			UpdateChannelInformation(thread,param[4],NULL);
			
			if(thread->c.chan.serv[0]!='\0'){
    			if(thread->c.chan.bot[0]!='\0'){
                    
					send_lang(thread,language,howtosend,snick,nick,"BS_INFO_INFO",thread->c.chan.name);
					send_lang(thread,language,howtosend,snick,nick,"BS_INFO_BOTNICK",thread->c.chan.bot);
					
					if(thread->c.chan.botserv.set&BS_KICK_BADWORDS_BIT) send_lang(thread,language,howtosend,snick,nick,"BS_INFO_BADWORDSKICKER","enabled");
					else send_lang(thread,language,howtosend,snick,nick,"BS_INFO_BADWORDSKICKER","disabled");
					
					if(thread->c.chan.botserv.set&BS_KICK_BOLDS_BIT) send_lang(thread,language,howtosend,snick,nick,"BS_INFO_BOLDSKICKER","enabled");
					else send_lang(thread,language,howtosend,snick,nick,"BS_INFO_BOLDSKICKER","disabled");
					
					if(thread->c.chan.botserv.set&BS_KICK_CAPS_BIT && thread->c.chan.botserv.caps_min>0 && thread->c.chan.botserv.caps_per>0) send_lang(thread,language,howtosend,snick,nick,"BS_INFO_CAPSKICKER_ENABLED",thread->c.chan.botserv.caps_min,thread->c.chan.botserv.caps_per);
					else send_lang(thread,language,howtosend,snick,nick,"BS_INFO_CAPSKICKER_DISABLED");
					
					if(thread->c.chan.botserv.set&BS_KICK_COLORS_BIT) send_lang(thread,language,howtosend,snick,nick,"BS_INFO_COLORSKICKER","enabled");
					else send_lang(thread,language,howtosend,snick,nick,"BS_INFO_COLORSKICKER","disabled");
					
					if(thread->c.chan.botserv.set&BS_KICK_FLOOD_BIT && thread->c.chan.botserv.flood_ln>=0 && thread->c.chan.botserv.flood_secs>=0) send_lang(thread,language,howtosend,snick,nick,"BS_INFO_FLOODKICKER_ENABLED",thread->c.chan.botserv.flood_ln,thread->c.chan.botserv.flood_secs);
					else send_lang(thread,language,howtosend,snick,nick,"BS_INFO_FLOODKICKER_DISABLED");
					
					if(thread->c.chan.botserv.set&BS_KICK_REPEAT_BIT) send_lang(thread,language,howtosend,snick,nick,"BS_INFO_REPEATKICKER","enabled");
					else send_lang(thread,language,howtosend,snick,nick,"BS_INFO_REPEATKICKER","disabled");
					
					if(thread->c.chan.botserv.set&BS_KICK_REVERSES_BIT) send_lang(thread,language,howtosend,snick,nick,"BS_INFO_REVERSESKICKER","enabled");
					else send_lang(thread,language,howtosend,snick,nick,"BS_INFO_REVERSESKICKER","disabled");
					
					if(thread->c.chan.botserv.set&BS_KICK_UNDERLINES_BIT) send_lang(thread,language,howtosend,snick,nick,"BS_INFO_UNDERLINESKICKER","enabled");
					else send_lang(thread,language,howtosend,snick,nick,"BS_INFO_UNDERLINESKICKER","disabled");

            		char options[150];
            		bzero(&options,sizeof(options));
            		if(thread->c.chan.botserv.set&BS_SET_DONTKICKOPS_BIT) straddns(options,"Ops protection, ",sizeof(options));
            		if(thread->c.chan.botserv.set&BS_SET_DONTKICKVOICES_BIT) straddns(options,"Voices protection, ",sizeof(options));
            		if(thread->c.chan.botserv.set&BS_SET_GREET_BIT) straddns(options,"Greet, ",sizeof(options));
            		if(thread->c.chan.botserv.set&BS_SET_FANTASY_BIT) straddns(options,"Fantasy, ",sizeof(options));
            		if(thread->c.chan.botserv.set&BS_SET_SYMBIOSIS_BIT) straddns(options,"Symbiosis, ",sizeof(options));
            		if(stoi(mysql_sel_var(thread,"SELECT `private` FROM `orion_bs_core` WHERE `botnick`='%s'",mysql_quote(thread,thread->c.chan.bot)))) straddns(options,"Private, ",sizeof(options));
        		    if(options[0]!='\0'){
                        options[strlen(options)-2]='\0';
                        send_lang(thread,language,howtosend,snick,nick,"BS_INFO_OPTIONS",options);
                    }
				}else{
            		char options[150];
            		bzero(&options,sizeof(options));
            		if(thread->c.chan.botserv.set&BS_SET_DONTKICKOPS_BIT) straddns(options,"Ops protection, ",sizeof(options));
            		if(thread->c.chan.botserv.set&BS_SET_DONTKICKVOICES_BIT) straddns(options,"Voices protection, ",sizeof(options));
            		if(thread->c.chan.botserv.set&BS_SET_GREET_BIT) straddns(options,"Greet, ",sizeof(options));
            		if(thread->c.chan.botserv.set&BS_SET_FANTASY_BIT) straddns(options,"Fantasy, ",sizeof(options));
            		if(thread->c.chan.botserv.set&BS_SET_SYMBIOSIS_BIT) straddns(options,"Symbiosis, ",sizeof(options));
            		if(stoi(mysql_sel_var(thread,"SELECT `private` FROM `orion_bs_core` WHERE `botnick`='%s'",mysql_quote(thread,thread->c.chan.bot)))) straddns(options,"Private, ",sizeof(options));
        		    if(options[0]!='\0'){
                        options[strlen(options)-2]='\0';
                        send_lang(thread,language,howtosend,snick,nick,"BS_INFO_OPTIONS",options);
                    }
					send_lang(thread,language,howtosend,snick,nick,"BS_NO_BOT_ASSIGNED_FOR_CHANNEL_X",thread->c.chan.name);
				}
			}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
		}else if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_bs_core` WHERE `botnick`='%s'",mysql_quote(thread,param[4])))){
			char channelstr[MAX_CHANS*MAX_CHANNELNAME_LEN+MAX_CHANS-1];
			bzero(&channelstr,sizeof(channelstr));
			
			send_lang(thread,language,howtosend,snick,nick,"BS_INFO_INFO_BOT",param[4]);
			/*(16:00) -BotServ- Information for bot [Admin]:
            (16:00) -BotServ- Mask : Servicebot@DeepIRC.net
            (16:00) -BotServ- Real name : [Admin]
            (16:00) -BotServ- Created : Jan 28 10:37:52 2003 CET
            (16:00) -BotServ- Options : Private
            (16:00) -BotServ- Used on : 4 channel(s)
            (16:00) -BotServ- #ircops #services #support-team #technical-team*/
            int counted=stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_cs_core` WHERE `botnick`='%s'",mysql_quote(thread,param[4])));
            
			MySQL.res    = doSelectMySQL(thread,"SELECT `botident`,`bothost`,`botrealname`,`private`,`time` FROM `orion_bs_core` WHERE `botnick`='%s'",mysql_quote(thread,param[4]));
            MySQL.fields = mysql_num_rows(MySQL.res);
			MySQL.row    = mysql_fetch_row(MySQL.res);
            
            if(MySQL.fields){
				send_lang(thread,language,howtosend,snick,nick,"BS_INFO_MASK",MySQL.row[0],MySQL.row[1]);
	            send_lang(thread,language,howtosend,snick,nick,"BS_INFO_REALNAME",MySQL.row[2]);
	            send_lang(thread,language,howtosend,snick,nick,"BS_INFO_CREATED",get_localtime((time_t)stol(MySQL.row[4])));
	            if(stoi(MySQL.row[3]))
					send_lang(thread,language,howtosend,snick,nick,"BS_INFO_OPTIONS_PRIVATE");
			}
			mysql_free_result_myr(MySQL.res);
			bzero(&MySQL,sizeof(MySQL));
			
            send_lang(thread,language,howtosend,snick,nick,"BS_INFO_USED_ON_X_CHANNELS",counted);
            MySQL.res    = doSelectMySQL(thread,"SELECT `channel` FROM `orion_cs_core` WHERE `botnick`='%s'",mysql_quote(thread,param[4]));
            MySQL.fields = mysql_num_rows(MySQL.res);
            
			for(i=0;i<MySQL.fields;i++){
				MySQL.row = mysql_fetch_row(MySQL.res);
				
                straddns(channelstr,MySQL.row[0],sizeof(channelstr));
                straddnc(channelstr,' ',sizeof(channelstr));
            }
            msend(thread,howtosend,snick,nick,channelstr);
            
			mysql_free_result_myr(MySQL.res);
		}else{
			//lol is not a valid bot or registered channel.
			send_lang(thread,language,howtosend,snick,nick,"BS_INFO_NOT_A_VALID_BOT_OR_CHANNEL",param[4]);
		}
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the BotServ INFO Command (/msg BotServ HELP INFO)
 * 
 * When the user tries to get help for this command, using /msg BotServ HELP INFO,<br/>
 * this function is executed.
 * 
 * Syntax: /msg BotServ HELP INFO
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_info_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Info Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help info"))
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_INFO",snick);
	else
	    send_lang(thread,language,howtosend,snick,nick,"BS_NO_HELP_AVAIL",snick);
		
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief BotServ Help List Command - INFO part (/msg BotServ help)
 * 
 * When the user tries to get general BotServ help, using /msg BotServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the INFO command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_info_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_BS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Info (Help Listing)
	 */
	if(Users_BS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_CMD_INFO");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
