/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file bs_kick.c
 *   @brief This is the Module file for BotServ KICK Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the BotServ KICK command.
 *
 *   The BotServ KICK Command allows the user to set various KICK-Options<br/>
 *   in target Channel. (Controls kicks/kickbans for i.e. BOLDS/UNDERLINES/ ...)
 *
 *   Syntax is: /msg BotServ KICK #channel OPTION { ON | OFF } [Parameters]
 *
 *<table border="0">
 *    <tr><td> BOLDS		</td><td> Syntax is: /msg BotServ KICK #channel BOLDS		</td><td> { ON | OFF }	</td><td> [ttb]					</td></tr>
 *    <tr><td> BADWORDS		</td><td> Syntax is: /msg BotServ KICK #channel BADWORDS	</td><td> { ON | OFF }	</td><td> [ttb]					</td></tr>
 *    <tr><td> CAPS			</td><td> Syntax is: /msg BotServ KICK #channel CAPS		</td><td> { ON | OFF }	</td><td> [ttb [min [percent]]]	</td></tr>
 *    <tr><td> COLORS		</td><td> Syntax is: /msg BotServ KICK #channel COLORS		</td><td> { ON | OFF }	</td><td> [ttb]					</td></tr>
 *    <tr><td> FLOOD		</td><td> Syntax is: /msg BotServ KICK #channel FLOOD		</td><td> { ON | OFF }	</td><td> [ttb [ln [secs]]]		</td></tr>
 *    <tr><td> REPEAT		</td><td> Syntax is: /msg BotServ KICK #channel REPEAT		</td><td> { ON | OFF }	</td><td> [ttb [num]]			</td></tr>
 *    <tr><td> REVERSES		</td><td> Syntax is: /msg BotServ KICK #channel REVERSES	</td><td> { ON | OFF }	</td><td> [ttb]					</td></tr>
 *    <tr><td> UNDERLINES	</td><td> Syntax is: /msg BotServ KICK #channel UNDERLINES	</td><td> { ON | OFF }	</td><td> [ttb]					</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "bs"
#define MODUL "kick"
#define MODUL_FILE "bs_kick"
#define MODUL_VERSION "1.0"

MOD_BOOL bs_kick(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL bs_kick_bolds(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_kick_badwords(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_kick_caps(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_kick_colors(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_kick_flood(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_kick_repeat(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_kick_reverses(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_kick_underlines(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL bs_kick_bolds_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_kick_badwords_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_kick_caps_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_kick_colors_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_kick_flood_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_kick_repeat_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_kick_reverses_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_kick_underlines_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL bs_kick_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_kick_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"kick * option[1]","kick helpoption[1]");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addLeastModuleLevels(thread,&Module,"kick",is_services_oper,is_services_user); /* just to add a displaylevel for /msg bs help
															  this should always be the lowest used Module level! */

	addCommand(thread,&Module,"kick bolds",bs_kick_bolds,is_services_oper,is_services_user,6,7);
	addCommand(thread,&Module,"kick badwords",bs_kick_badwords,is_services_oper,is_services_user,6,7);
	addCommand(thread,&Module,"kick caps",bs_kick_caps,is_services_oper,is_services_user,6,7);
	addCommand(thread,&Module,"kick colors",bs_kick_colors,is_services_oper,is_services_user,6,7);
	addCommand(thread,&Module,"kick flood",bs_kick_flood,is_services_oper,is_services_user,6,9);
	addCommand(thread,&Module,"kick repeat",bs_kick_repeat,is_services_oper,is_services_user,6,8);
	addCommand(thread,&Module,"kick reverses",bs_kick_reverses,is_services_oper,is_services_user,6,7);
	addCommand(thread,&Module,"kick underlines",bs_kick_underlines,is_services_oper,is_services_user,6,7);

	addCommand(thread,&Module,"help kick",bs_kick_help,is_services_oper,is_services_user,4,4);
	addCommand(thread,&Module,"help kick bolds",bs_kick_bolds_help,is_services_oper,is_services_user,5,5);
	addCommand(thread,&Module,"help kick badwords",bs_kick_badwords_help,is_services_oper,is_services_user,5,5);
	addCommand(thread,&Module,"help kick caps",bs_kick_caps_help,is_services_oper,is_services_user,5,5);
	addCommand(thread,&Module,"help kick colors",bs_kick_colors_help,is_services_oper,is_services_user,5,5);
	addCommand(thread,&Module,"help kick flood",bs_kick_flood_help,is_services_oper,is_services_user,5,5);
	addCommand(thread,&Module,"help kick repeat",bs_kick_repeat_help,is_services_oper,is_services_user,5,5);
	addCommand(thread,&Module,"help kick reverses",bs_kick_reverses_help,is_services_oper,is_services_user,5,5);
	addCommand(thread,&Module,"help kick underlines",bs_kick_underlines_help,is_services_oper,is_services_user,5,5);
	
	addCommand(thread,&Module,"help",bs_kick_help_listing,is_services_oper,is_services_user,3,3);
	
	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}
//:

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the BotServ KICK Command (/msg BotServ HELP KICK)
 * 
 * When the user tries to get help for this command, using /msg BotServ HELP KICK,<br/>
 * this function is executed.
 * 
 * Syntax: /msg BotServ HELP KICK
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_kick_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
   	/*
   	 * Kick Help
   	 */
   	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help kick")){
   		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_KICK",snick);
   	}else send_lang(thread,language,howtosend,snick,nick,"BS_NO_HELP_AVAIL",snick);
   	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS KICK Bolds function for the BotServ KICK Command.
 * 
 * The BotServ KICK Bolds Command allows the user to set Bold kicking for<br/>
 *   his Channel on or off.<br/>
 *   (BotServ will then calculate a percantage of Bold using and kick if necessary)
 *  
 * Syntax is: /msg BotServ KICK #channel BOLDS { ON | OFF } [ttb]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_kick_bolds(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	
	/*
	 * Bolds Command
	 */
 	//:shock PRIVMSG BotServ :set #channel dontkickops ON/OFF
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"kick bolds")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		//c = sql_chan_db(thread,param[4]); //Channel c - STRUCT
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv") || streqnc(thread->c.chan.serv,"q")){
            if(thread->c.chan.bot[0]!='\0'){
    			if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || strcor(thread->c.chan.access.ownlevel,"mn") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"kick bolds")){
    				if(streqnc(param[6],"on")){
                        if(!param[7]){
							thread->c.chan.botserv.set |= BS_KICK_BOLDS_BIT;
							thread->c.chan.botserv.bolds = -1;
							
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_bolds`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.bolds,mysql_quote(thread,thread->c.chan.name));
							send_lang(thread,language,howtosend,snick,nick,"BS_KICK_BOLDS"); /* No Lang */
                        }else if(digitcol(param[7]) && stoi(param[7])>=0){
							thread->c.chan.botserv.set |= BS_KICK_BOLDS_BIT;
							thread->c.chan.botserv.bolds = stoi(param[7]);
							
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_bolds`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.bolds,mysql_quote(thread,thread->c.chan.name));
							send_lang(thread,language,howtosend,snick,nick,"BS_KICK_BOLDS_BAN",stoi(param[7]));
						}else send_lang(thread,language,howtosend,snick,nick,"INVALID_PARAMETER",param[7]);
    				}else if(streqnc(param[6],"off")){
						thread->c.chan.botserv.set &= ~BS_KICK_BOLDS_BIT;
						thread->c.chan.botserv.bolds = -1;
							
                        domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_bolds`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.bolds,mysql_quote(thread,thread->c.chan.name));
                        send_lang(thread,language,howtosend,snick,nick,"BS_KICK_BOLDS_OFF");
    				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
    			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
    		}else send_lang(thread,language,howtosend,snick,nick,"BS_NO_BOT_ASSIGNED_FOR_CHANNEL_X",param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS KICK Badwords function for the BotServ KICK Command.
 * 
 * The BotServ KICK Badwords Command allows the user to set Badword kicking for<br/>
 *   his Channel on or off.<br/>
 *   (BotServ will check for Badwords in Channelmessages and kick of necessary)
 *  
* Syntax is: /msg BotServ KICK #channel BADWORDS { ON | OFF } [ttb]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_kick_badwords(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	
	/*
	 * Badwords Command
	 */
 	//:shock PRIVMSG BotServ :set #channel dontkickvoices ON/OFF
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"kick badwords")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		//c = sql_chan_db(thread,param[4]); //Channel c - STRUCT
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv") || streqnc(thread->c.chan.serv,"q")){
            if(thread->c.chan.bot[0]!='\0'){
    			if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || strcor(thread->c.chan.access.ownlevel,"mn") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"kick badwords")){
    				if(streqnc(param[6],"on")){
                        if(!param[7]){
							thread->c.chan.botserv.set |= BS_KICK_BADWORDS_BIT;
							thread->c.chan.botserv.badwords = -1;
							
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_badwords`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.badwords,mysql_quote(thread,thread->c.chan.name));
							send_lang(thread,language,howtosend,snick,nick,"BS_KICK_BADWORDS");
                        }else if(digitcol(param[7]) && stoi(param[7])>=0){
							thread->c.chan.botserv.set |= BS_KICK_BADWORDS_BIT;
							thread->c.chan.botserv.badwords = stoi(param[7]);
							
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_badwords`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.badwords,mysql_quote(thread,thread->c.chan.name));
							send_lang(thread,language,howtosend,snick,nick,"BS_KICK_BADWORDS_BAN",stoi(param[7]));
						}else send_lang(thread,language,howtosend,snick,nick,"INVALID_PARAMETER",param[7]);
    				}else if(streqnc(param[6],"off")){
						thread->c.chan.botserv.set &= ~BS_KICK_BADWORDS_BIT;
						thread->c.chan.botserv.badwords = -1;
						
                        domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_bolds`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.badwords,mysql_quote(thread,thread->c.chan.name));
                        send_lang(thread,language,howtosend,snick,nick,"BS_KICK_BADWORDS_OFF");
    				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
    			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
    		}else send_lang(thread,language,howtosend,snick,nick,"BS_NO_BOT_ASSIGNED_FOR_CHANNEL_X",param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS KICK Caps function for the BotServ KICK Command.
 * 
 * The BotServ KICK Caps Command allows the user to set Caps kicking for<br/>
 *   his Channel on or off.<br/>
 *   (BotServ will check for a Caps percentage in Channelmessages and kick of necessary)
 *  
 * Syntax is: /msg BotServ KICK #channel CAPS { ON | OFF } [ttb [min [percent]]]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_kick_caps(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	
	/*
	 * Caps Command
	 */
 	//:shock PRIVMSG BotServ :kick #channel caps ON 3 1 75
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"kick caps")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		//c = sql_chan_db(thread,param[4]); //Channel c - STRUCT
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv") || streqnc(thread->c.chan.serv,"q")){
            if(thread->c.chan.bot[0]!='\0'){
    			if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || strcor(thread->c.chan.access.ownlevel,"mn") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"kick caps")){
    				if(streqnc(param[6],"on")){
                        if(digitcol(param[9]) && digitcol(param[8]) && digitcol(param[7]) && stoi(param[7])>=0){
							thread->c.chan.botserv.set |= BS_KICK_CAPS_BIT;
							thread->c.chan.botserv.caps_ttb = stoi(param[7]);
							thread->c.chan.botserv.caps_min = stoi(param[8]);
							thread->c.chan.botserv.caps_per = stoi(param[9]);
							
                        	domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_caps_ttb`='%d',`bs_caps_min`='%d',`bs_caps_per`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.caps_ttb,thread->c.chan.botserv.caps_min,thread->c.chan.botserv.caps_per,mysql_quote(thread,thread->c.chan.name));
							send_lang(thread,language,howtosend,snick,nick,"BS_KICK_CAPS_BAN",thread->c.chan.botserv.caps_min,thread->c.chan.botserv.caps_per,thread->c.chan.botserv.caps_ttb);
                        }else if(digitcol(param[8]) && digitcol(param[7]) && stoi(param[7])>=0){
							thread->c.chan.botserv.set |= BS_KICK_CAPS_BIT;
							thread->c.chan.botserv.caps_ttb = stoi(param[7]);
							thread->c.chan.botserv.caps_min = stoi(param[8]);
							thread->c.chan.botserv.caps_per = 25;
							
                        	domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_caps_ttb`='%d',`bs_caps_min`='%d',`bs_caps_per`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.caps_ttb,thread->c.chan.botserv.caps_min,thread->c.chan.botserv.caps_per,mysql_quote(thread,thread->c.chan.name));
                        	send_lang(thread,language,howtosend,snick,nick,"BS_KICK_CAPS_BAN",thread->c.chan.botserv.caps_min,thread->c.chan.botserv.caps_per,thread->c.chan.botserv.caps_ttb);
                        }else if(digitcol(param[7]) && stoi(param[7])>=0){
							thread->c.chan.botserv.set |= BS_KICK_CAPS_BIT;
							thread->c.chan.botserv.caps_ttb = stoi(param[7]);
							thread->c.chan.botserv.caps_min = 10;
							thread->c.chan.botserv.caps_per = 25;
							
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_caps_ttb`='%d',`bs_caps_min`='%d',`bs_caps_per`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.caps_ttb,thread->c.chan.botserv.caps_min,thread->c.chan.botserv.caps_per,mysql_quote(thread,thread->c.chan.name));
							send_lang(thread,language,howtosend,snick,nick,"BS_KICK_CAPS_BAN",thread->c.chan.botserv.caps_min,thread->c.chan.botserv.caps_per,thread->c.chan.botserv.caps_ttb);
						}else if(!param[7]){
							thread->c.chan.botserv.set |= BS_KICK_CAPS_BIT;
							thread->c.chan.botserv.caps_ttb = -1;
							thread->c.chan.botserv.caps_min = 10;
							thread->c.chan.botserv.caps_per = 25;
							
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_caps_ttb`='%d',`bs_caps_min`='%d',`bs_caps_per`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.caps_ttb,thread->c.chan.botserv.caps_min,thread->c.chan.botserv.caps_per,mysql_quote(thread,thread->c.chan.name));
							send_lang(thread,language,howtosend,snick,nick,"BS_KICK_CAPS",thread->c.chan.botserv.caps_min,thread->c.chan.botserv.caps_per);
						}else send_lang(thread,language,howtosend,snick,nick,"INVALID_PARAMETER",param[7]);
    				}else if(streqnc(param[6],"off")){
						thread->c.chan.botserv.set &= ~BS_KICK_CAPS_BIT;
						thread->c.chan.botserv.caps_ttb = -1;
						thread->c.chan.botserv.caps_min = -1;
						thread->c.chan.botserv.caps_per = -1;
							
                        domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_caps_ttb`='%d',`bs_caps_min`='%d',`bs_caps_per`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.caps_ttb,thread->c.chan.botserv.caps_min,thread->c.chan.botserv.caps_per,mysql_quote(thread,thread->c.chan.name));
                        send_lang(thread,language,howtosend,snick,nick,"BS_KICK_CAPS_OFF");
    				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
    			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
    		}else send_lang(thread,language,howtosend,snick,nick,"BS_NO_BOT_ASSIGNED_FOR_CHANNEL_X",param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS KICK Colors function for the BotServ KICK Command.
 * 
 * The BotServ KICK Colors Command allows the user to set Colors kicking for<br/>
 *   his Channel on or off.<br/>
 *   (BotServ will check for a Color percentage in Channelmessages and kick of necessary)
 *  
 * Syntax is: /msg BotServ KICK #channel COLORS { ON | OFF } [ttb]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_kick_colors(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	
	/*
	 * Colors Command
	 */
 	//:shock PRIVMSG BotServ :set #channel fantasy ON/OFF
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"kick colors")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		//c = sql_chan_db(thread,param[4]); //Channel c - STRUCT
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv") || streqnc(thread->c.chan.serv,"q")){
            if(thread->c.chan.bot[0]!='\0'){
    			if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || strcor(thread->c.chan.access.ownlevel,"mn") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"kick colors")){
    				if(streqnc(param[6],"on")){
                        if(!param[7]){
							thread->c.chan.botserv.set |= BS_KICK_COLORS_BIT;
							thread->c.chan.botserv.colors = -1;
							
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_colors`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.colors,mysql_quote(thread,thread->c.chan.name));
							send_lang(thread,language,howtosend,snick,nick,"BS_KICK_COLORS"); /* No Lang */
                        }else if(digitcol(param[7]) && stoi(param[7])>=0){
							thread->c.chan.botserv.set |= BS_KICK_COLORS_BIT;
							thread->c.chan.botserv.colors = stoi(param[7]);
							
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_colors`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.colors,mysql_quote(thread,thread->c.chan.name));
							send_lang(thread,language,howtosend,snick,nick,"BS_KICK_COLORS_BAN",thread->c.chan.botserv.colors);
						}else send_lang(thread,language,howtosend,snick,nick,"INVALID_PARAMETER",param[7]);
    				}else if(streqnc(param[6],"off")){
						thread->c.chan.botserv.set &= ~BS_KICK_COLORS_BIT;
						thread->c.chan.botserv.colors = -1;
							
                        domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_colors`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.colors,mysql_quote(thread,thread->c.chan.name));
                        send_lang(thread,language,howtosend,snick,nick,"BS_KICK_COLORS_OFF");
    				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
    			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
    		}else send_lang(thread,language,howtosend,snick,nick,"BS_NO_BOT_ASSIGNED_FOR_CHANNEL_X",param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS KICK Flood function for the BotServ KICK Command.
 * 
 * The BotServ KICK Flood Command allows the user to set Flood kicking for<br/>
 *   his Channel on or off.<br/>
 *   (BotServ will check for a Flood in target Channel and kick of necessary)
 *  
 * Syntax is: /msg BotServ KICK #channel FLOOD { ON | OFF } [ttb [ln [secs]]]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_kick_flood(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	/*
	 * Flood Command
	 */
 	//:shock PRIVMSG BotServ :kick #channel flood ON/OFF 6 10
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"kick flood")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		//c = sql_chan_db(thread,param[4]); //Channel c - STRUCT
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv") || streqnc(thread->c.chan.serv,"q")){
            if(thread->c.chan.bot[0]!='\0'){
    			if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || strcor(thread->c.chan.access.ownlevel,"mn") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"kick flood")){
    				if(streqnc(param[6],"on")){
                        if(digitcol(param[9]) && digitcol(param[8]) && digitcol(param[7]) && stoi(param[7])>=0){
							thread->c.chan.botserv.set |= BS_KICK_FLOOD_BIT;
							thread->c.chan.botserv.flood_ttb  = stoi(param[7]);
							thread->c.chan.botserv.flood_ln   = stoi(param[8]);
							thread->c.chan.botserv.flood_secs = stoi(param[9]);
							
                        	domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_flood_ttb`='%d',`bs_flood_ln`='%d',`bs_flood_secs`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.flood_ttb,thread->c.chan.botserv.flood_ln,thread->c.chan.botserv.flood_secs,mysql_quote(thread,thread->c.chan.name));
							send_lang(thread,language,howtosend,snick,nick,"BS_KICK_FLOOD_BAN",thread->c.chan.botserv.flood_ln,thread->c.chan.botserv.flood_secs,thread->c.chan.botserv.flood_ttb);
                        }else if(digitcol(param[8]) && digitcol(param[7]) && stoi(param[7])>=0){
							thread->c.chan.botserv.set |= BS_KICK_FLOOD_BIT;
							thread->c.chan.botserv.flood_ttb  = stoi(param[7]);
							thread->c.chan.botserv.flood_ln   = stoi(param[8]);
							thread->c.chan.botserv.flood_secs = 10;
							
                        	domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_flood_ttb`='%d',`bs_flood_ln`='%d',`bs_flood_secs`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.flood_ttb,thread->c.chan.botserv.flood_ln,thread->c.chan.botserv.flood_secs,mysql_quote(thread,thread->c.chan.name));
                        	send_lang(thread,language,howtosend,snick,nick,"BS_KICK_FLOOD_BAN",thread->c.chan.botserv.flood_ln,thread->c.chan.botserv.flood_secs,thread->c.chan.botserv.flood_ttb);
                        }else if(digitcol(param[7]) && stoi(param[7])>=0){
							thread->c.chan.botserv.set |= BS_KICK_FLOOD_BIT;
							thread->c.chan.botserv.flood_ttb  = stoi(param[7]);
							thread->c.chan.botserv.flood_ln   = 6;
							thread->c.chan.botserv.flood_secs = 10;
							
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_flood_ttb`='%d',`bs_flood_ln`='%d',`bs_flood_secs`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.flood_ttb,thread->c.chan.botserv.flood_ln,thread->c.chan.botserv.flood_secs,mysql_quote(thread,thread->c.chan.name));
							send_lang(thread,language,howtosend,snick,nick,"BS_KICK_FLOOD_BAN",thread->c.chan.botserv.flood_ln,thread->c.chan.botserv.flood_secs,thread->c.chan.botserv.flood_ttb);
						}else if(!param[7]){
							thread->c.chan.botserv.set |= BS_KICK_FLOOD_BIT;
							thread->c.chan.botserv.flood_ttb  = -1;
							thread->c.chan.botserv.flood_ln   = 6;
							thread->c.chan.botserv.flood_secs = 10;
							
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_flood_ttb`='%d',`bs_flood_ln`='%d',`bs_flood_secs`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.flood_ttb,thread->c.chan.botserv.flood_ln,thread->c.chan.botserv.flood_secs,mysql_quote(thread,thread->c.chan.name));
							send_lang(thread,language,howtosend,snick,nick,"BS_KICK_FLOOD",thread->c.chan.botserv.flood_ln,thread->c.chan.botserv.flood_secs); /* No Lang */
						}
    				}else if(streqnc(param[6],"off")){
						thread->c.chan.botserv.set &= ~BS_KICK_FLOOD_BIT;
						thread->c.chan.botserv.flood_ttb  = -1;
						thread->c.chan.botserv.flood_ln   = -1;
						thread->c.chan.botserv.flood_secs = -1;
							
                        domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_flood_ttb`='%d',`bs_flood_ln`='%d',`bs_flood_secs`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.flood_ttb,thread->c.chan.botserv.flood_ln,thread->c.chan.botserv.flood_secs,mysql_quote(thread,thread->c.chan.name));
                        send_lang(thread,language,howtosend,snick,nick,"BS_KICK_FLOOD_OFF");
    				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
    			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
    		}else send_lang(thread,language,howtosend,snick,nick,"BS_NO_BOT_ASSIGNED_FOR_CHANNEL_X",param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS KICK Repeat function for the BotServ KICK Command.
 * 
 * The BotServ KICK Repeat Command allows the user to set Repeat kicking for<br/>
 *   his Channel on or off.<br/>
 *   (BotServ will check for Repeatings in target Channel and kick of necessary)
 *  
 * Syntax is: /msg BotServ KICK #channel REPEAT { ON | OFF } [ttb [num]]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_kick_repeat(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	
	/*
	 * Repeat Command
	 */
 	//:shock PRIVMSG BotServ :kick #channel repeat ON/OFF 3
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"kick repeat")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		//c = sql_chan_db(thread,param[4]); //Channel c - STRUCT
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv") || streqnc(thread->c.chan.serv,"q")){
            if(thread->c.chan.bot[0]!='\0'){
    			if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || strcor(thread->c.chan.access.ownlevel,"mn") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"kick repeat")){
    				if(streqnc(param[6],"on")){
						if(digitcol(param[8]) && digitcol(param[7]) && stoi(param[7])>=0){
							thread->c.chan.botserv.set |= BS_KICK_REPEAT_BIT;
							thread->c.chan.botserv.repeat_ttb = stoi(param[7]);
							thread->c.chan.botserv.repeat     = stoi(param[8]);
							
                        	domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_repeat_ttb`='%d',`bs_repeat`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.repeat_ttb,thread->c.chan.botserv.repeat,mysql_quote(thread,thread->c.chan.name));
                        	send_lang(thread,language,howtosend,snick,nick,"BS_KICK_REPEAT_BAN",thread->c.chan.botserv.repeat,thread->c.chan.botserv.repeat_ttb);
                        }else if(digitcol(param[7]) && stoi(param[7])>=0){
							thread->c.chan.botserv.set |= BS_KICK_REPEAT_BIT;
							thread->c.chan.botserv.repeat_ttb = stoi(param[7]);
							thread->c.chan.botserv.repeat     = 3;
							
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_repeat_ttb`='%d',`bs_repeat`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.repeat_ttb,thread->c.chan.botserv.repeat,mysql_quote(thread,thread->c.chan.name));
							send_lang(thread,language,howtosend,snick,nick,"BS_KICK_REPEAT_BAN",thread->c.chan.botserv.repeat,thread->c.chan.botserv.repeat_ttb);
						}else if(!param[7]){
							thread->c.chan.botserv.set |= BS_KICK_REPEAT_BIT;
							thread->c.chan.botserv.repeat_ttb = -1;
							thread->c.chan.botserv.repeat     = 3;
							
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_repeat_ttb`='%d',`bs_repeat`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.repeat_ttb,thread->c.chan.botserv.repeat,mysql_quote(thread,thread->c.chan.name));
							send_lang(thread,language,howtosend,snick,nick,"BS_KICK_REPEAT",thread->c.chan.botserv.repeat); /* No Lang */
						}
    				}else if(streqnc(param[6],"off")){
						thread->c.chan.botserv.set &= ~BS_KICK_REPEAT_BIT;
						thread->c.chan.botserv.repeat_ttb = -1;
						thread->c.chan.botserv.repeat     = -1;
							
                        domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_repeat_ttb`='%d',`bs_repeat`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.repeat_ttb,thread->c.chan.botserv.repeat,mysql_quote(thread,thread->c.chan.name));
                        send_lang(thread,language,howtosend,snick,nick,"BS_KICK_REPEAT_OFF");
    				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
    			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
    		}else send_lang(thread,language,howtosend,snick,nick,"BS_NO_BOT_ASSIGNED_FOR_CHANNEL_X",param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS KICK Reverses function for the BotServ KICK Command.
 * 
 * The BotServ KICK Reverses Command allows the user to set Reverses kicking for<br/>
 *   his Channel on or off.<br/>
 *   (BotServ will check for Reverses in target Channel and kick of necessary)
 *  
 * Syntax is: /msg BotServ KICK #channel REVERSES { ON | OFF } [ttb]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_kick_reverses(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	
	/*
	 * Reverses Command
	 */
 	//:shock PRIVMSG BotServ :set #channel fantasy ON/OFF
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"kick reverses")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		//c = sql_chan_db(thread,param[4]); //Channel c - STRUCT
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv") || streqnc(thread->c.chan.serv,"q")){
            if(thread->c.chan.bot[0]!='\0'){
    			if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || strcor(thread->c.chan.access.ownlevel,"mn") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"kick reverses")){
    				if(streqnc(param[6],"on")){
                        if(!param[7]){
							thread->c.chan.botserv.set |= BS_KICK_REVERSES_BIT;
							thread->c.chan.botserv.reverses = -1;
							
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_reverses`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.reverses,mysql_quote(thread,thread->c.chan.name));
							send_lang(thread,language,howtosend,snick,nick,"BS_KICK_REVERSES"); /* No Lang */
                        }else if(digitcol(param[7]) && stoi(param[7])>=0){
							thread->c.chan.botserv.set |= BS_KICK_REVERSES_BIT;
							thread->c.chan.botserv.reverses = stoi(param[7]);
							
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_reverses`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.reverses,mysql_quote(thread,thread->c.chan.name));
							send_lang(thread,language,howtosend,snick,nick,"BS_KICK_REVERSES_BAN",thread->c.chan.botserv.reverses);
						}else send_lang(thread,language,howtosend,snick,nick,"INVALID_PARAMETER",param[7]);
    				}else if(streqnc(param[6],"off")){
						thread->c.chan.botserv.set &= ~BS_KICK_REVERSES_BIT;
						thread->c.chan.botserv.reverses = -1;
							
                        domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_reverses`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.reverses,mysql_quote(thread,thread->c.chan.name));
                        send_lang(thread,language,howtosend,snick,nick,"BS_KICK_REVERSES_OFF");
    				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
    			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
    		}else send_lang(thread,language,howtosend,snick,nick,"BS_NO_BOT_ASSIGNED_FOR_CHANNEL_X",param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS KICK UnderLines function for the BotServ KICK Command.
 * 
 * The BotServ KICK UnderLines Command allows the user to set UnderLine kicking for<br/>
 *   his Channel on or off.<br/>
 *   (BotServ will check for a UnderLine percentage in target ChannelMessage and kick of necessary)
 *  
 * Syntax is: /msg BotServ KICK #channel UNDERLINES { ON | OFF } [ttb]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_kick_underlines(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	
	/*
	 * UnderLines Command
	 */
 	//:shock PRIVMSG BotServ :set #channel fantasy ON/OFF
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"kick underlines")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		//c = sql_chan_db(thread,param[4]); //Channel c - STRUCT
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv") || streqnc(thread->c.chan.serv,"q")){
            if(thread->c.chan.bot[0]!='\0'){
    			if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || strcor(thread->c.chan.access.ownlevel,"mn") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"kick underlines")){
    				if(streqnc(param[6],"on")){
                        if(!param[7]){
							thread->c.chan.botserv.set |= BS_KICK_UNDERLINES_BIT;
							thread->c.chan.botserv.underlines = -1;
							
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_underlines`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.underlines,mysql_quote(thread,thread->c.chan.name));
							send_lang(thread,language,howtosend,snick,nick,"BS_KICK_UNDERLINES"); /* No Lang */
                        }else if(digitcol(param[7]) && stoi(param[7])>=0){
							thread->c.chan.botserv.set |= BS_KICK_UNDERLINES_BIT;
							thread->c.chan.botserv.underlines = stoi(param[7]);
							
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_underlines`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.underlines,mysql_quote(thread,thread->c.chan.name));
							send_lang(thread,language,howtosend,snick,nick,"BS_KICK_UNDERLINES_BAN",thread->c.chan.botserv.underlines);
						}else send_lang(thread,language,howtosend,snick,nick,"INVALID_PARAMETER",param[7]);
    				}else if(streqnc(param[6],"off")){
						thread->c.chan.botserv.set &= ~BS_KICK_UNDERLINES_BIT;
						thread->c.chan.botserv.underlines = -1;
							
                        domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d',`bs_underlines`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,thread->c.chan.botserv.underlines,mysql_quote(thread,thread->c.chan.name));
                        send_lang(thread,language,howtosend,snick,nick,"BS_KICK_UNDERLINES_OFF");
    				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
    			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
    		}else send_lang(thread,language,howtosend,snick,nick,"BS_NO_BOT_ASSIGNED_FOR_CHANNEL_X",param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS KICK Bolds help function for the BotServ KICK Command (/msg BotServ HELP KICK BOLDS)
 * 
 * When the user tries to get help for this command, using /msg BotServ HELP KICK BOLDS,<br/>
 * this function is executed.
 * 
 * Syntax: /msg BotServ HELP KICK BOLDS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_kick_bolds_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Bolds Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help kick bolds"))
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_KICK_BOLDS");
	else
	    send_lang(thread,language,howtosend,snick,nick,"BS_NO_HELP_AVAIL",snick);
		
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS KICK Badwords help function for the BotServ KICK Command (/msg BotServ HELP KICK BADWORDS)
 * 
 * When the user tries to get help for this command, using /msg BotServ HELP KICK BADWORDS,<br/>
 * this function is executed.
 * 
 * Syntax: /msg BotServ HELP KICK BADWORDS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_kick_badwords_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Badwords Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help kick badwords"))
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_KICK_BADWORDS",snick);
	else
	    send_lang(thread,language,howtosend,snick,nick,"BS_NO_HELP_AVAIL",snick);
	
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS KICK Caps help function for the BotServ KICK Command (/msg BotServ HELP KICK CAPS)
 * 
 * When the user tries to get help for this command, using /msg BotServ HELP KICK CAPS,<br/>
 * this function is executed.
 * 
 * Syntax: /msg BotServ HELP KICK CAPS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_kick_caps_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Caps Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help kick caps"))
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_KICK_CAPS");
	else
	    send_lang(thread,language,howtosend,snick,nick,"BS_NO_HELP_AVAIL",snick);
	    
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS KICK Colors help function for the BotServ KICK Command (/msg BotServ HELP KICK COLORS)
 * 
 * When the user tries to get help for this command, using /msg BotServ HELP KICK COLORS,<br/>
 * this function is executed.
 * 
 * Syntax: /msg BotServ HELP KICK COLORS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_kick_colors_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Colors Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help kick colors"))
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_KICK_COLORS");
	else
	    send_lang(thread,language,howtosend,snick,nick,"BS_NO_HELP_AVAIL",snick);
	
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS KICK Flood help function for the BotServ KICK Command (/msg BotServ HELP KICK FLOOD)
 * 
 * When the user tries to get help for this command, using /msg BotServ HELP KICK FLOOD,<br/>
 * this function is executed.
 * 
 * Syntax: /msg BotServ HELP KICK FLOOD
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_kick_flood_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Flood Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help kick flood"))
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_KICK_FLOOD");
	else
	    send_lang(thread,language,howtosend,snick,nick,"BS_NO_HELP_AVAIL",snick);
	
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS KICK Repeat help function for the BotServ KICK Command (/msg BotServ HELP KICK REPEAT)
 * 
 * When the user tries to get help for this command, using /msg BotServ HELP KICK REPEAT,<br/>
 * this function is executed.
 * 
 * Syntax: /msg BotServ HELP KICK REPEAT
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_kick_repeat_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Repeat Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help kick repeat"))
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_KICK_REPEAT");
	else
	    send_lang(thread,language,howtosend,snick,nick,"BS_NO_HELP_AVAIL",snick);
	
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS KICK Reverses help function for the BotServ KICK Command (/msg BotServ HELP KICK REVERSES)
 * 
 * When the user tries to get help for this command, using /msg BotServ HELP KICK REVERSES,<br/>
 * this function is executed.
 * 
 * Syntax: /msg BotServ HELP KICK REVERSES
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_kick_reverses_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Reverses Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help kick reverses"))
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_KICK_REVERSES");
	else
	    send_lang(thread,language,howtosend,snick,nick,"BS_NO_HELP_AVAIL",snick);
	
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS KICK UnderLines help function for the BotServ KICK Command (/msg BotServ HELP KICK UNDERLINES)
 * 
 * When the user tries to get help for this command, using /msg BotServ HELP KICK UNDERLINES,<br/>
 * this function is executed.
 * 
 * Syntax: /msg BotServ HELP KICK UNDERLINES
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_kick_underlines_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * UnderLines Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help kick underlines"))
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_KICK_UNDERLINES");
	else
	    send_lang(thread,language,howtosend,snick,nick,"BS_NO_HELP_AVAIL",snick);
	
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BotServ Help List Command - KICK part (/msg BotServ help)
 * 
 * When the user tries to get general BotServ help, using /msg BotServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the KICK command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_kick_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_BS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Kick (Help listing)
	 */
	if(Users_BS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_CMD_KICK");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

