/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Chrisostomos Adamos (alias: Ally), Email: Ally@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 203                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 23:10:45 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file hs_off.c
 *   @brief This is the Module file for HostServ OFF Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/> 
 *   the HostServ OFF command.
 *
 *   The HostServ OFF Command allows the user to deactivate the vHost<br/>
 *   currently assigned to the nick in use.
 *
 *   Syntax is: /msg HostServ OFF
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "hs"
#define MODUL "off"
#define MODUL_FILE "hs_off"
#define MODUL_VERSION "1.1"

MOD_BOOL hs_off(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL hs_off_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL hs_off_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module;
	
	if(!IRC.VHost){
		Module.ModErr = MOD_ERR_CANCELLED;
		Module.module = NULL;
		return Module;
	}
	
	Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"off","off");

	addAuthor (thread,&Module,"Ally, cLiEbiG, (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"off",hs_off,is_services_user,is_services_user,3,3);
	addCommand(thread,&Module,"help off",hs_off_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",hs_off_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the HostServ OFF Command.
 * 
 * The HostServ OFF Command allows the user to deactivate the vHost<br/>
 *   currently assigned to the nick in use.
 *
 * Syntax is: /msg HostServ OFF
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_off(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL checkvhost;
    
	/* Initialise Variables to (NULL) */
	bzero(&checkvhost,sizeof(checkvhost));

	/*
	 * Off Command
	 */
   	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"off")){

		checkvhost.res    = doSelectMySQL(thread,"SELECT `nick`, `active`, `vhost` FROM `orion_hs_core` WHERE `nick`='%s' LIMIT 0,1",mysql_quote(thread,nick));
		checkvhost.fields = mysql_num_rows(checkvhost.res);

		if(checkvhost.fields){
			checkvhost.row = mysql_fetch_row(checkvhost.res);
			
			if(stoi(checkvhost.row[1]) == 1){
                                       
				domysql(thread,"UPDATE `orion_hs_core` SET `active` = '0' WHERE `nick`='%s'",mysql_quote(thread,nick));
				do_svsmode(thread,snick,nick,_dynadd("-%c",IRC.VhostMode));
				send_lang(thread,language,howtosend,snick,nick,"HS_OFF_SUCCESS",snick);
				
			}else send_lang(thread,language,howtosend,snick,nick,"HS_OFF_IS_NOT_ACTIVE",snick);
		}else send_lang(thread,language,howtosend,snick,nick,"HS_DEL_NO_VHOST",nick);
		
		/* mysql_free_result_myr(); - checkvhost */
		mysql_free_result_myr(checkvhost.res);
    }else{
	/*
	 * Error Messages
	 */
 		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
    }
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the HostServ OFF Command (/msg HostServ HELP OFF)
 * 
 * When the user tries to get help for this command, using /msg HostServ HELP OFF,<br/>
 * this function is executed.
 * 
 * Syntax: /msg HostServ HELP OFF
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_off_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Off Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help off"))
		send_lang(thread,language,howtosend,snick,nick,"HS_HELP_OFF",snick);
	else
	    send_lang(thread,language,howtosend,snick,nick,"HS_NO_HELP_AVAIL",snick);
		
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief HostServ Help List Command - OFF part (/msg HostServ help)
 * 
 * When the user tries to get general HostServ help, using /msg HostServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the OFF command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_off_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_HS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Off (Help Listing)
	 */
	if(Users_HS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"HS_HELP_CMD_OFF");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
