/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ms_cancel.c
 *   @brief This is the Module file for MemoServ CANCEL Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the MemoServ CANCEL command.
 *
 *   The MemoServ CANCEL Command allows the user to cancel the last memo he sent<br/>
 *   to a nick or channel, provided it has not been read at the time the command was used.
 *
 *   Syntax is: /msg MemoServ CANCEL <nick / #channel>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "ms"
#define MODUL "cancel"
#define MODUL_FILE "ms_cancel"
#define MODUL_VERSION "1.0"

MOD_BOOL ms_cancel(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ms_cancel_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ms_cancel_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"cancel","cancel");
	
	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"cancel",ms_cancel,is_services_admin,is_services_user,4,4);
	addCommand(thread,&Module,"help cancel",ms_cancel_help,is_services_admin,is_services_user,4,4);
	addCommand(thread,&Module,"help",ms_cancel_help_listing,is_services_admin,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

//:shock PRIVMSG MemoServ :cancel shock
/*  ************************************************************************************************  */

/**
 * @brief Main function for the MemoServ CANCEL Command.
 * 
 * The MemoServ CANCEL Command allows the user to cancel the last memo he sent<br/>
 *   to a nick or channel, provided it has not been read at the time the command was used.
 *
 * Syntax is: /msg MemoServ CANCEL <nick / #channel>
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_cancel(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL memos;
	Aliases aliases5;
	Channel c;
	
	/* Initialise Variables to (NULL) */
	bzero(&memos,sizeof(memos));
	bzero(&aliases5,sizeof(aliases5));
	bzero(&c,sizeof(c));
	
	/*
	 * Cancel Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"cancel")){
        if(param[4][0]=='#'){ //Channel
            c            = sql_chan(thread,param[4]);
            memos.res    = doSelectMySQL(thread,"SELECT `memo`,`time`,`mode` FROM `orion_ms_core` WHERE `receiver`='%s' AND `sender`='%s' ORDER BY `time` DESC LIMIT 1",mysql_quote(thread,c.name),mysql_quote(thread,thread->sql.aliases.authnick));
            memos.fields = mysql_num_rows(memos.res);
			if(memos.fields)
				memos.row    = mysql_fetch_row(memos.res);
  
            if(c.serv[0]=='\0')		send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
            else if(!memos.fields)	send_lang(thread,language,howtosend,snick,nick,"MEMO_NEVER_SENT_TO_RECEIVER",param[4]);
           
			else if(memos.fields && c.serv[0]!='\0' && stoi(memos.row[2])==0){ //memo unread
                domysql(thread,"DELETE FROM `orion_ms_core` WHERE `receiver`='%s' AND `time`='%d'",mysql_quote(thread,c.name),stoi(memos.row[1]));
                send_lang(thread,language,howtosend,snick,nick,"MS_CANCEL_CANCELLED_MEMO_FOR_X_MEMO_X",c.name,memos.row[0]);
            }
            else send_lang(thread,language,howtosend,snick,nick,"MEMO_ALREADY_READ");
            
    		/* mysql_free_result_myr(); - memos */
    		mysql_free_result_myr(memos.res);
        }else{
            aliases5     = sql_aliases(thread,param[4]);
            memos.res    = doSelectMySQL(thread,"SELECT `memo`,`time`,`mode` FROM `orion_ms_core` WHERE `receiver`='%s' AND `sender`='%s' ORDER BY `time` DESC LIMIT 1",mysql_quote(thread,aliases5.authnick),mysql_quote(thread,thread->sql.aliases.authnick));
            memos.fields = mysql_num_rows(memos.res);
			if(memos.fields)
				memos.row    = mysql_fetch_row(memos.res);
  
            if(aliases5.authnick[0]=='\0')	send_lang(thread,language,howtosend,snick,nick,"NO_AUTHNICK");
            else if(!memos.fields)			send_lang(thread,language,howtosend,snick,nick,"MEMO_NEVER_SENT_TO_RECEIVER",param[4]);

            else if(memos.fields && aliases5.authnick[0]!='\0' && stoi(memos.row[2])==0){ //memo unread
                domysql(thread,"DELETE FROM `orion_ms_core` WHERE `receiver`='%s' AND `time`='%d'",mysql_quote(thread,aliases5.authnick),stoi(memos.row[1]));
                send_lang(thread,language,howtosend,snick,nick,"MS_CANCEL_CANCELLED_MEMO_FOR_X_MEMO_X",aliases5.authnick,memos.row[0]);
            }
            else send_lang(thread,language,howtosend,snick,nick,"MEMO_ALREADY_READ");
            
    		/* mysql_free_result_myr(); - memos */
    		mysql_free_result_myr(memos.res);
        }
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the MemoServ CANCEL Command (/msg MemoServ HELP CANCEL)
 * 
 * When the user tries to get help for this command, using /msg MemoServ HELP CANCEL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg MemoServ HELP CANCEL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_cancel_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Cancel Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help cancel"))
		send_lang(thread,language,howtosend,snick,nick,"MS_HELP_CANCEL");

	else send_lang(thread,language,howtosend,snick,nick,"MS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief MemoServ Help List Command - CANCEL part (/msg MemoServ help)
 * 
 * When the user tries to get general MemoServ help, using /msg MemoServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the CANCEL command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_cancel_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_MS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Cancel (Help listing)
	 */
	if(Users_MS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"MS_HELP_CMD_CANCEL");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
