/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 207                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-05 12:32:05 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ms_events.c
 *   @brief This is the Module file for MemoServ EVENTS Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the MemoServ EVENTS command.
 *
 *   Main functions - EVENTS ONLY
 *
 *   Syntax is: /msg MemoServ EVENTS
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "ms"
#define MODUL "events"
#define MODUL_FILE "ms_events"
#define MODUL_VERSION "1.0"

/*  ******************************************* Event List *****************************************  */

MOD_BOOL ms_on_message(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ms_on_identify(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL ms_primary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ms_secondary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ms_squit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

/*  **************************************** End of Event List **************************************  */

MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.<br/>
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,EVENT_PROTOTYPE,EVENT_PROTOTYPE);

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addAccess (thread,&Module,NOONE);
	addMODType(thread,&Module,EVENT);

	addEventHook(thread,&Module,"on_message"	,ms_on_message);
	addEventHook(thread,&Module,"on_identify"	,ms_on_identify);

	addEventHook(thread,&Module,"on_connect_primary"	,ms_primary);
	addEventHook(thread,&Module,"on_connect_secondary"	,ms_secondary);
	addEventHook(thread,&Module,"on_squit"				,ms_squit);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	
/* Things we want to process "on load" */
	ms->loaded = 1;
	
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the MemoServ PRIMARY Command.
 * 
 * The MemoServ PRIMARY Command allows the user....
 *  
 * Syntax is: /msg MemoServ PRIMARY
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_primary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    do_kill(thread,m->servername,ms->nick,"nick used by orion");
	do_sqline(thread,m->servername,ms->nick,0,time(NULL),"nick used by orion");
	createuser(thread,2,ms->nick,ms->ident,ms->host,IRC.MSMode,ms->realname);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the MemoServ SECONDARY Command.
 * 
 * The MemoServ SECONDARY Command allows the user....
 *  
 * Syntax is: /msg MemoServ SECONDARY
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_secondary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	if(m->logchan[0] != '\0'){
		do_services_join(thread,ms->nick,m->logchan,time(NULL),2);
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the MemoServ SQUIT Command.
 * 
 * The MemoServ SQUIT Command allows the user....
 *  
 * Syntax is: /msg MemoServ SQUIT
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_squit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * ms_on_identify: MemoServ Events: On Identify Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL ms_on_identify(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    // :shock PRIVMSG NICKSERV :identify 34154424
    Aliases aliases;
    deepSQL memosforme;

	if(GetUser(nick)==NULL)
		return MOD_CONT;

    aliases           = sql_aliases(thread,nick);
    memosforme.res    = doSelectMySQL(thread,"SELECT `time` FROM `orion_ms_core` WHERE `receiver`='%s' AND `mode`='0' ORDER BY `time` ASC",mysql_quote(thread,aliases.authnick));
    memosforme.fields = mysql_num_rows(memosforme.res);
    
    if(memosforme.fields){
        if(memosforme.fields>1)
            send_lang(thread,language,howtosend,ms->nick,nick,"MS_ON_IDENTIFY_1",memosforme.fields,ms->nick);
        else if(memosforme.fields==1)
            send_lang(thread,language,howtosend,ms->nick,nick,"MS_ON_IDENTIFY_2",memosforme.fields,ms->nick);
    }
    
	/* mysql_free_result_myr(); - memosforme */
	mysql_free_result_myr(memosforme.res);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * ms_on_join: MemoServ Events: On Join Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL ms_on_join(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int memos=0;
	
	if(GetUser(nick)==NULL)
		return MOD_CONT;
	
	/* Updates the Threads Channel Management */
	UpdateChannelInformation2(thread,param[2],nick);
	
	if(streqnc(thread->c.chan.serv,"chanserv")){
		if(!thread->c.aliases.active)
			unknown_access(thread->c.chan.access.targetlevel,sizeof(thread->c.chan.access.targetlevel));
	}

	/*
	 * If it's a registered channel
	 */
	if(thread->c.chan.fields && thread->c.chan.serv[0]!='\0'){
		if(!streqnc(nick,cs->nick) || !streqnc(nick,Q->nick)/* || !streqnc(nick,L->nick)*/){
			if(thread->c.aliases.active && thread->c.chan.access.targetlevel[0]!='\0'){
                memos = stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_ms_core` WHERE `receiver`='%s' AND `mode`='0'",mysql_quote(thread,thread->c.chan.name)));
				if(streqnc(thread->c.chan.serv,"chanserv") && stoi(thread->c.chan.access.targetlevel)>=thread->c.chan.levels.memo){
                    send_lang(thread,language,howtosend,ms->nick,nick,"MS_ON_JOIN",memos,thread->c.chan.name);
				}else if((streqnc(thread->c.chan.serv,"q") || streqnc(thread->c.chan.serv,"l")) && strcor(thread->c.chan.access.targetlevel,"o")){
                    send_lang(thread,language,howtosend,ms->nick,nick,"MS_ON_JOIN",memos,thread->c.chan.name);
				}
			}
		}
	}
	bzero(&thread->c,sizeof(thread->c));
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * ms_on_message: MemoServ Events: On Message Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL ms_on_message(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* (language == NULL, howtosend = 0, snick == NULL) */

	if((m->UseStrictPrivMsg && streqnc(param[2],_dynadd("%s@%s",ms->ident,ms->host))) || (!m->UseStrictPrivMsg && (streqnc(param[2],ms->nick) || streqnc(param[2],_dynadd("%s@%s",ms->ident,ms->host))))){
		if(GetUser(nick)==NULL)
	    	return MOD_CONT;

		ModCheck modcheck;
    	ModCo *modul = NULL;
    	char *cmd    = &param[3][1];
    	
    	bzero(&modcheck,sizeof(modcheck));
    	
    	bzero(&thread->sql,sizeof(SQL));
		sql_mainAAC(thread,nick);
		
    	/* Since language == NULL, howtosend = 0, snick == NULL - we wake them up now ;) */
    	snick		= ms->nick;
    	howtosend	= thread->sql.access.settings&NS_SET_MSG ? 1 : 0;
    	language	= languages[thread->sql.access.language];
    	/*********************************************************************************/
    	
	/* MS Commands */
	
		/* If help for a specific Command was requested */
    	if(streqnc(cmd,"help") && param[4]){
			
			/* Help with wildmasks */
    		if(strcor(param[4],"*"))
				call_wildview(thread, snick, nick, SERVICE, "help", param);
    		
			/* Normal help for a specific command (no wildmasks) */
			else{
    			modul 		= findModule(thread, SERVICE, param[4]);
  				modcheck 	= call_helpcommand(thread, modul, cmd, snick, nick, param);
  				
				if(!ModReply(thread, &modcheck, snick, nick, param))
					send_lang(thread, language, howtosend, snick, nick, "MS_UNKNOWN_COMMAND", snick);
    		}
    	}
		
		/* If Command was only "help" */
		else if(streqnc(cmd,"help") && !param[4]){
    		modul 		= findModule(thread, SERVICE, cmd);
    		modcheck 	= call_helpcommand(thread, modul, cmd, snick, nick, param);
    		
			if(!ModReply(thread, &modcheck, snick, nick, param))
    			send_lang(thread, language, howtosend, snick, nick, "MS_UNKNOWN_COMMAND", snick);
    	}
    	
		/* All other commands which aren't "help" */
		else if(!streqnc(cmd,"help")){
    		modul 		= findModule(thread, SERVICE, cmd);
  			modcheck 	= call_command(thread, modul, cmd, snick, nick, param);
			  	
			if(!ModReply(thread, &modcheck, snick, nick, param))
    			send_lang(thread, language, howtosend, snick, nick, "MS_UNKNOWN_COMMAND", snick);
    	}
		else send_lang(thread, language, howtosend, snick, nick, "MS_UNKNOWN_COMMAND", snick);
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return MOD_BOOL
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
    if(Action == DO_UNLOAD){
		ms->loaded = 0;
		do_quit(thread,ms->nick,"module unloaded");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */
