/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 203                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 23:10:45 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ns_events.c
 *   @brief This is the Module file for NickServ EVENTS Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the NickServ EVENTS command.
 *
 *   Main functions - EVENTS ONLY
 *
 *   Syntax is: /msg NickServ EVENTS
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "ns"
#define MODUL "events"
#define MODUL_FILE "ns_events"
#define MODUL_VERSION "1.0"

/*  ******************************************* Event List *****************************************  */

MOD_BOOL ns_on_kill(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_on_kick(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_on_link(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_on_linkdone(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_on_netinfo(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_on_mode(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_on_nick(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL ns_on_part(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_on_quit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_on_protoctl(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_on_sethost(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_on_setident(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_on_chghost(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_on_chgident(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_on_version(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_on_topic(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_on_join(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_on_message(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL ns_primary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_secondary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_squit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

/*  **************************************** End of Event List **************************************  */

MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,EVENT_PROTOTYPE,EVENT_PROTOTYPE);

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addAccess (thread,&Module,NOONE);
	addMODType(thread,&Module,EVENT);
	
	addEventHook(thread,&Module,"on_join"		,ns_on_join);
	addEventHook(thread,&Module,"on_part"		,ns_on_part);

	addEventHook(thread,&Module,"on_kill"		,ns_on_kill);
	addEventHook(thread,&Module,"on_kick"		,ns_on_kick);
	addEventHook(thread,&Module,"on_link"		,ns_on_link);
	addEventHook(thread,&Module,"on_linkdone"	,ns_on_linkdone);
	addEventHook(thread,&Module,"on_netinfo"	,ns_on_netinfo);

	addEventHook(thread,&Module,"on_nick"		,ns_on_nick);
	addEventHook(thread,&Module,"on_quit"		,ns_on_quit);
	addEventHook(thread,&Module,"on_protoctl"	,ns_on_protoctl);
	addEventHook(thread,&Module,"on_sethost"	,ns_on_sethost);
	addEventHook(thread,&Module,"on_setident"	,ns_on_setident);
	addEventHook(thread,&Module,"on_chghost"	,ns_on_chghost);
	addEventHook(thread,&Module,"on_chgident"	,ns_on_chgident);
	addEventHook(thread,&Module,"on_version"	,ns_on_version);
	addEventHook(thread,&Module,"on_topic"		,ns_on_topic);
	
	addEventHook(thread,&Module,"on_message"	,ns_on_message);

	addEventHook(thread,&Module,"on_connect_primary"	,ns_primary);
	addEventHook(thread,&Module,"on_connect_secondary"	,ns_secondary);
	addEventHook(thread,&Module,"on_squit"				,ns_squit);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);

/* Things we want to process "on load" */
	ns->loaded = 1;

	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the NickServ PRIMARY Command.
 * 
 * The NickServ PRIMARY Command allows the user....
 *  
 * Syntax is: /msg NickServ PRIMARY
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_primary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    do_kill(thread,m->servername,ns->nick,"nick used by orion");
	do_sqline(thread,m->servername,ns->nick,0,time(NULL),"nick used by orion");
	createuser(thread,2,ns->nick,ns->ident,ns->host,IRC.NSMode,ns->realname);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the NickServ SECONDARY Command.
 * 
 * The NickServ SECONDARY Command allows the user....
 *  
 * Syntax is: /msg NickServ SECONDARY
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_secondary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	if(m->logchan[0] != '\0'){
		do_services_join(thread,ns->nick,m->logchan,time(NULL),2);
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the NickServ SQUIT Command.
 * 
 * The NickServ SQUIT Command allows the user....
 *  
 * Syntax is: /msg NickServ SQUIT
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_squit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * ns_on_kill: NickServ Events: On Kill Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL ns_on_kill(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * ns_on_kick: NickServ Events: On Kick Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL ns_on_kick(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * ns_on_link: NickServ Events: On Link Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL ns_on_link(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * ns_on_linkdone: NickServ Events: On Linkdone Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL ns_on_linkdone(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * ns_on_netinfo: NickServ Events: On Netinfo Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL ns_on_netinfo(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	domysql(thread,"UPDATE `orion_core_condata` LEFT JOIN `orion_ns_aliases` ON (`orion_ns_aliases`.`nick` = `orion_core_condata`.`nick`) SET `ns_warntime`='%d' WHERE `orion_core_condata`.`faketype`='0' AND `orion_ns_aliases`.`nick` IS NOT NULL AND `orion_ns_aliases`.`active`='0'",time(NULL));
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * ns_on_nick: NickServ Events: On Nick Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL ns_on_nick(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_ns_aliases` WHERE `nick`='%s' AND `orion_ns_aliases`.`active`='0'",mysql_quote(thread,nick)))==1){
		send_lang(thread,languages[m->language],0,ns->nick,nick,"NS_WARN_NICK_REGISTERED_BY_ANOTHER_USER");//Keine Rechte - Warnen mit NickServ dieser nick ist nicht in der selben gruppe - geh�t wem anderen (nswarntime setzen)
		//domysql(thread,"UPDATE `orion_core_condata` SET `ns_warntime`='%d' WHERE `nick`='%s'",time(NULL),mysql_quote(thread,param[1]));
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * ns_on_part: NickServ Events: On Part Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL ns_on_part(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * ns_on_quit: NickServ Events: On Quit Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL ns_on_quit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * ns_on_protoctl: NickServ Events: On Protoctl Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL ns_on_protoctl(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * ns_on_sethost: NickServ Events: On Sethost Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL ns_on_sethost(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * ns_on_setident: NickServ Events: On Setident Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL ns_on_setident(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * ns_on_chghost: NickServ Events: On Chghost Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL ns_on_chghost(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * ns_on_chgident: NickServ Events: On Chgident Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL ns_on_chgident(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * ns_on_version: NickServ Events: On Version Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL ns_on_version(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * ns_on_topic: NickServ Events: On Topic Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL ns_on_topic(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * ns_on_join: NickServ Events: On Join Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL ns_on_join(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * ns_on_message: NickServ Events: On Message Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL ns_on_message(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* (language == NULL, howtosend = 0, snick == NULL) */

	if((m->UseStrictPrivMsg && streqnc(param[2],_dynadd("%s@%s",ns->ident,ns->host))) || (!m->UseStrictPrivMsg && (streqnc(param[2],ns->nick) || streqnc(param[2],_dynadd("%s@%s",ns->ident,ns->host))))){
		ModCheck modcheck;
    	ModCo *modul = NULL;
    	char *cmd    = &param[3][1];
    	
    	bzero(&modcheck,sizeof(modcheck));
    	
    	bzero(&thread->sql,sizeof(SQL));
		sql_mainAAC(thread,nick);
		
    	/* Since language == NULL, howtosend = 0, snick == NULL - we wake them up now ;) */
    	snick		= ns->nick;
    	howtosend	= thread->sql.access.settings&NS_SET_MSG ? 1 : 0;
    	language	= languages[thread->sql.access.language];
    	/*********************************************************************************/
    	
	/* NS Commands */
	
		/* If help for a specific Command was requested */
    	if(streqnc(cmd,"help") && param[4]){
			
			/* Help with wildmasks */
    		if(strcor(param[4],"*"))
				call_wildview(thread, snick, nick, SERVICE, "help", param);
    		
			/* Normal help for a specific command (no wildmasks) */
			else{
    			modul 		= findModule(thread, SERVICE, param[4]);
  				modcheck 	= call_helpcommand(thread, modul, cmd, snick, nick, param);
  				
				if(!ModReply(thread, &modcheck, snick, nick, param))
					send_lang(thread, language, howtosend, snick, nick, "NS_UNKNOWN_COMMAND", snick);
    		}
    	}
		
		/* If Command was only "help" */
		else if(streqnc(cmd,"help") && !param[4]){
    		modul 		= findModule(thread, SERVICE, cmd);
    		modcheck 	= call_helpcommand(thread, modul, cmd, snick, nick, param);
    		
			if(!ModReply(thread, &modcheck, snick, nick, param))
    			send_lang(thread, language, howtosend, snick, nick, "NS_UNKNOWN_COMMAND", snick);
    	}
    	
		/* All other commands which aren't "help" */
		else if(!streqnc(cmd,"help")){
    		modul 		= findModule(thread, SERVICE, cmd);
  			modcheck 	= call_command(thread, modul, cmd, snick, nick, param);
			  	
			if(!ModReply(thread, &modcheck, snick, nick, param))
    			send_lang(thread, language, howtosend, snick, nick, "NS_UNKNOWN_COMMAND", snick);
    	}
		else send_lang(thread, language, howtosend, snick, nick, "NS_UNKNOWN_COMMAND", snick);
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return MOD_BOOL
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
    if(Action == DO_UNLOAD){
		ns->loaded = 0;
		do_quit(thread,ns->nick,"module unloaded");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */
