/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ns_forbid.c
 *   @brief This is the Module file for NickServ FORBID Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the NickServ FORBID command.
 *
 *   The NickServ FORBID Command allows an admin to disallow a nickname<br/>
 *   from being registered or used by anyone. May be cancelled by dropping the nick.
 *
 *   Syntax is: /msg NickServ FORBID < nick > < reason >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "ns"
#define MODUL "forbid"
#define MODUL_FILE "ns_forbid"
#define MODUL_VERSION "1.0"

MOD_BOOL ns_forbid(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_forbid_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_forbid_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"forbid","forbid");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"forbid",ns_forbid,is_services_admin,INFINIT,4,0);
	addCommand(thread,&Module,"help forbid",ns_forbid_help,is_services_admin,INFINIT,4,4);
	addCommand(thread,&Module,"help",ns_forbid_help_listing,is_services_admin,INFINIT,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

//:shock PRIVMSG NickServ :forbid rofl so
/*  ************************************************************************************************  */

/**
 * @brief Main function for the NickServ FORBID Command.
 * 
 * The NickServ FORBID Command allows an admin to disallow a nickname<br/>
 *   from being registered or used by anyone. May be cancelled by dropping the nick.
 *
 * Syntax is: /msg NickServ FORBID < nick > < reason >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_forbid(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	Aliases aliases5;

	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	
	/*
	 * Forbid Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"forbid")){
		aliases5 = sql_aliases(thread,param[4]);
		
        if(aliases5.nick[0]=='\0'){
        	if(!aliases5.forbid){
        	
				if(m->WallForbid)
					do_global(thread,snick,"%s used forbid for nick %s",nick,param[4]);
				
				if(m->ForceForbidReason && param[5]){
	        		domysql(thread,"INSERT INTO `orion_ns_aliases` (`nick`,`display`,`last_usermask`,`last_realname`,"
	        										"`last_quit`,`time_linked`,`last_login`,`last_used`,"
	        										"`expire`,`linked`,`active`,`forbid`)"
	        										" VALUES ('%s',NULL,'','','%s','','','','','','','1')",mysql_quote(thread,param[4]),mysql_quote(thread,_dstrjoin(' ',param,5)));
	        		send_lang(thread,language,howtosend,snick,nick,"NS_FORBID_NICK_DONE");
				}else if(!m->ForceForbidReason){
					if(param[5]){
		        		domysql(thread,"INSERT INTO `orion_ns_aliases` (`nick`,`display`,`last_usermask`,`last_realname`,"
		        										"`last_quit`,`time_linked`,`last_login`,`last_used`,"
		        										"`expire`,`linked`,`active`,`forbid`)"
		        										" VALUES ('%s',NULL,'','','%s','','','','','','','1')",mysql_quote(thread,param[4]),mysql_quote(thread,_dstrjoin(' ',param,5)));
		        		send_lang(thread,language,howtosend,snick,nick,"NS_FORBID_NICK_DONE");
					}else{
		        		domysql(thread,"INSERT INTO `orion_ns_aliases` (`nick`,`display`,`last_usermask`,`last_realname`,"
		        										"`last_quit`,`time_linked`,`last_login`,`last_used`,"
		        										"`expire`,`linked`,`active`,`forbid`)"
		        										" VALUES ('%s',NULL,'','','Forbidden without a reason','','','','','','','1')",mysql_quote(thread,param[4]));
		        		send_lang(thread,language,howtosend,snick,nick,"NS_FORBID_NICK_DONE");
					}
				}else send_lang(thread,language,howtosend,snick,nick,"NO_REASON");
        	}else send_lang(thread,language,howtosend,snick,nick,"NS_FORBID_NICK_ALREADY_FORBIDDEN");
        }else{
        	if(!aliases5.forbid){
        	
				if(m->WallForbid)
					do_global(thread,snick,"%s used forbid for nick %s",nick,param[4]);
				
				if(m->ForceForbidReason && param[5]){
	        		domysql(thread,"UPDATE `orion_ns_aliases` SET `last_quit`='%s',`forbid`='1' WHERE `nick`='%s'",mysql_quote(thread,_dstrjoin(' ',param,5)),mysql_quote(thread,param[4]));
	        		send_lang(thread,language,howtosend,snick,nick,"NS_FORBID_NICK_DONE");
				}else if(!m->ForceForbidReason){
					if(param[5]){
		        		domysql(thread,"UPDATE `orion_ns_aliases` SET `last_quit`='%s',`forbid`='1' WHERE `nick`='%s'",mysql_quote(thread,_dstrjoin(' ',param,5)),mysql_quote(thread,param[4]));
		        		send_lang(thread,language,howtosend,snick,nick,"NS_FORBID_NICK_DONE");
					}else{
		        		domysql(thread,"UPDATE `orion_ns_aliases` SET `last_quit`='Forbidden without a reason',`forbid`='1' WHERE `nick`='%s'",mysql_quote(thread,_dstrjoin(' ',param,5)),mysql_quote(thread,param[4]));
		        		send_lang(thread,language,howtosend,snick,nick,"NS_FORBID_NICK_DONE");
					}
				}else send_lang(thread,language,howtosend,snick,nick,"NO_REASON");
        	}else send_lang(thread,language,howtosend,snick,nick,"NS_FORBID_NICK_ALREADY_FORBIDDEN");
		}
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the NickServ FORBID Command (/msg NickServ HELP FORBID)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP FORBID,<br/> 
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP FORBID
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_forbid_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Forbid Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help forbid"))
		send_lang(thread,language,howtosend,snick,nick,"NS_SERVADMIN_HELP_FORBID");

	else send_lang(thread,language,howtosend,snick,nick,"NS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief NickServ Help List Command - FORBID part (/msg NickServ help)
 * 
 * When the user tries to get general NickServ help, using /msg NickServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the FORBID command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_forbid_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_NS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Forbid (Help listing)
	 */
	if(Users_NS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_CMD_FORBID");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
