/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ns_resend.c
 *   @brief This is the Module file for NickServ RESEND Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the NickServ RESEND command.
 *
 *   The NickServ RESEND Command allows the user to re-request the passcode via email.<br/>
 *   This Command can only be used after a sepcified time (services.conf setting) - so be patient.
 *
 *   Syntax is: /msg NickServ RESEND
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "ns"
#define MODUL "resend"
#define MODUL_FILE "ns_resend"
#define MODUL_VERSION "1.0"

MOD_BOOL ns_resend(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_resend_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_resend_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module;

	if(!m->NSEmailReg){
		Module.ModErr = MOD_ERR_CANCELLED;
		return Module;
	}
	
	Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"resend","resend");
	
	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,USER);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"resend",ns_resend,is_user,is_user,3,5);
	addCommand(thread,&Module,"help resend",ns_resend_help,is_user,is_user,4,4);
	addCommand(thread,&Module,"help",ns_resend_help_listing,is_user,is_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the NickServ RESEND Command.
 * 
 * The NickServ RESEND Command allows the user to re-request the passcode via email.<br/>
 *   This Command can only be used after a sepcified time (services.conf setting) - so be patient.
 *
 * Syntax is: /msg NickServ RESEND
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_resend(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
    deepSQL request;
	char From[80];
	char Subject[80];
	
	/* Initialise Variables to (NULL) */
	bzero(&request,sizeof(request));
	bzero(&From,sizeof(From));
	bzero(&Subject,sizeof(Subject));

	/*
	 * Resend Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"resend")){ //cmd resend
		request.res    = doSelectMySQL(thread,"SELECT `display`,`passcode`,`password`,`email`,`reqtime`,`reqsent`,`emailchange` FROM `orion_ns_request` WHERE `display`='%s'",mysql_quote(thread,nick));
		request.fields = mysql_num_rows(request.res);
		
		if(request.fields){
			request.row = mysql_fetch_row(request.res);
			
			if(stoi(request.row[5])+m->NSResendDelay<time(NULL)){
				
				if(stoi(request.row[6])==0){
					if(request.row[3][0]!='\0' && request.row[0][0]!='\0' && request.row[2][0]!='\0' && m->UseMail){
						domysql(thread,"UPDATE `orion_ns_request` SET `reqsent`='%ld' WHERE `display`='%s'",time(NULL),mysql_quote(thread,request.row[0]));
						
						dynncpy(From,_dynadd("%s Network (%s) <%s>",m->network,ns->nick,m->SendFrom),sizeof(From));
						dynncpy(Subject,_dynadd("Your %s account",snick),sizeof(Subject));
					
						MailInfo *mail = o_email(thread,From,nick,request.row[3],Subject);
						//Email					
						s_email(thread,language,mail->pipe,"NS_REGISTER_REQUEST_NICK_EMAIL",nick,request.row[2],request.row[1],ns->nick,0xA0,ns->nick,ns->host,request.row[1],ns->nick,0xA0,ns->nick,ns->host,request.row[2],0xA0,ns->nick,nick,nick,Q->nick,0xA0,ns->nick,m->NSExpire,m->homepage);
						c_email(thread,mail);
						send_lang(thread,language,howtosend,snick,nick,"NS_RESEND_RESENT_PASSCODE_TO_EMAIL_X",request.row[3]);
					}else dlog(thread,0,"DataBase ERROR (somebody modified it) - Resend for (%s - email: %s) password=%s FAILED (nick registration)",request.row[0],request.row[3],request.row[2]);
				}else if(stoi(request.row[6])==1){
					if(request.row[3][0]!='\0' && request.row[0][0]!='\0' && m->UseMail){
						domysql(thread,"UPDATE `orion_ns_request` SET `reqsent`='%ld' WHERE `display`='%s'",time(NULL),mysql_quote(thread,request.row[0]));
						
						dynncpy(From,_dynadd("%s Network (%s) <%s>",m->network,ns->nick,m->SendFrom),sizeof(From));
						dynncpy(Subject,_dynadd("Your %s account (Your requested email change)",snick),sizeof(Subject));
					
						MailInfo *mail = o_email(thread,From,nick,request.row[3],Subject);
						//Email					
						s_email(thread,language,mail->pipe,"NS_CHANGE_EMAIL_REQUEST",thread->sql.access.authnick,request.row[1],ns->nick,0xA0,ns->nick,ns->host,request.row[1],m->NSExpire,m->homepage);
						c_email(thread,mail);
						send_lang(thread,language,howtosend,snick,nick,"NS_RESEND_RESENT_PASSCODE_TO_EMAIL_X",request.row[3]);
					}else dlog(thread,0,"DataBase ERROR (somebody modified it) - Resend for (%s - email: %s) FAILED (email change)",request.row[0],request.row[3]);
				}
				
			}else send_lang(thread,language,howtosend,snick,nick,"NS_RESEND_DELAY_WAIT_MINS",((stoi(request.row[5])+m->NSResendDelay-time(NULL)) / 60) % 60);
		}else send_lang(thread,language,howtosend,snick,nick,"NICK_WAS_NOT_REQUESTED");

		/* MySQL Free Result */
		mysql_free_result_myr(request.res);
	}else{
	/*
	 *  Error Messages
	 */
	 	send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the NickServ RESEND Command (/msg NickServ HELP RESEND)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP RESEND,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP RESEND
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_resend_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Resend Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help resend"))
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_RESEND",snick,m->NSResendDelay  / 60);

	else send_lang(thread,language,howtosend,snick,nick,"NS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief NickServ Help List Command - RESEND part (/msg NickServ help)
 * 
 * When the user tries to get general NickServ help, using /msg NickServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the RESEND command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_resend_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_NS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Resend (Help listing)
	 */
	if(Users_NS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_CMD_RESEND");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
