/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 206                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-04 17:15:02 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_chan.c
 *   @brief This is the Module file for O CHAN Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the O CHAN command.
 *
 *   The O CHAN Command allows an admin to manage the channel in various<br/>
 *   ways. (Controls users and the channel - CLEAR / MOVE channel)
 *
 *   Syntax is: /msg O CHAN { CLEAR | RENAME | DEOPALL | FORCE | KICK | MOVE | OP } #channel [ KILL | GLINE | KICK | BAN ]
 *
 *<table border="0">
 *    <tr><td> CLEAR	</td><td> Syntax is: /msg O CHAN CLEAR		</td><td> #channel	</td><td> { KICK | BAN | KILL | GLINE }	</td><td> (kicks/bans/kills/glines ALL users in #channel)	</td></tr>
 *    <tr><td> MOVE		</td><td> Syntax is: /msg O CHAN MOVE		</td><td> #channel	</td><td>								</td><td> (Moves the Channels Data Q<->ChanServ)			</td></tr>
 *    <tr><td> RENAME	</td><td> Syntax is: /msg O CHAN RENAME		</td><td> #channel	</td><td> #newchannel					</td><td> (Renames a Channel #channel -> #newchannel)		</td></tr>
 *    <tr><td> FORCE	</td><td> Syntax is: /msg O CHAN FORCE		</td><td> #channel	</td><td>								</td><td> (Forces all homeless Users to join that channel)	</td></tr>
 *    <tr><td> GLOBMODE	</td><td> Syntax is: /msg O CHAN GLOBMODE	</td><td> modes		</td><td>								</td><td> (Sets these mode string to all channels)			</td></tr>
 *    <tr><td> OP		</td><td> Syntax is: /msg O CHAN OP			</td><td> #channel	</td><td> [nick]						</td><td> (Ops me or target user in given channel)			</td></tr>
 *    <tr><td> KICK		</td><td> Syntax is: /msg O CHAN KICK		</td><td> #channel	</td><td> nick							</td><td> (Kicks the given user from target channel)		</td></tr>
 *    <tr><td> DEOPALL	</td><td> Syntax is: /msg O CHAN DEOPALL	</td><td> #channel	</td><td>								</td><td> (Deops all Ops in target channel)					</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "O"
#define MODUL "chan"
#define MODUL_FILE "o_chan"
#define MODUL_VERSION "1.0"

MOD_BOOL O_chan_clear(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_chan_move(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_chan_rename(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_chan_force(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_chan_globmode(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_chan_op(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_chan_kick(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_chan_deopall(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_chan_clear_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_chan_move_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_chan_rename_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_chan_force_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_chan_globmode_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_chan_op_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_chan_kick_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_chan_deopall_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_chan_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_chan_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"chan option[1]","chan helpoption[1]");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);
	
	addLeastModuleLevels(thread,&Module,"chan",500,INFINIT); /* just to add a displaylevel for /msg o help
															  this should always be the lowest used Module level! */
	
	addCommand(thread,&Module,"chan clear",O_chan_clear,950,INFINIT,6,7);
	addCommand(thread,&Module,"chan move",O_chan_move,850,INFINIT,5,5);
	addCommand(thread,&Module,"chan rename",O_chan_rename,920,INFINIT,6,6);
	addCommand(thread,&Module,"chan force",O_chan_force,1000,INFINIT,5,5);
	addCommand(thread,&Module,"chan globmode",O_chan_globmode,900,INFINIT,5,5);
	addCommand(thread,&Module,"chan op",O_chan_op,800,INFINIT,5,6);
	addCommand(thread,&Module,"chan kick",O_chan_kick,500,INFINIT,6,0);
	addCommand(thread,&Module,"chan deopall",O_chan_deopall,850,INFINIT,5,5);

	addCommand(thread,&Module,"help chan",O_chan_help,950,INFINIT,4,4);
	addCommand(thread,&Module,"help chan clear",O_chan_clear_help,950,INFINIT,5,5);
	addCommand(thread,&Module,"help chan move",O_chan_move_help,850,INFINIT,5,5);
	addCommand(thread,&Module,"help chan rename",O_chan_rename_help,920,INFINIT,5,5);
	addCommand(thread,&Module,"help chan force",O_chan_force_help,1000,INFINIT,5,5);
	addCommand(thread,&Module,"help chan globmode",O_chan_globmode_help,900,INFINIT,5,5);
	addCommand(thread,&Module,"help chan op",O_chan_op_help,800,INFINIT,5,5);
	addCommand(thread,&Module,"help chan kick",O_chan_kick_help,500,INFINIT,5,5);
	addCommand(thread,&Module,"help chan deopall",O_chan_deopall_help,850,INFINIT,5,5);

	addCommand(thread,&Module,"help",O_chan_help_listing,950,INFINIT,3,3); //100 - each Admin is able to see it.
	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}
 
/*  ************************************************************************************************  */

/**
 * @brief O CHAN Clear function for the O CHAN Command.
 * 
 * The O CHAN Clear Command allows an admin to clear a Channel by using { KICK | BAN | KILL | GLINE }
 *  
 * Syntax is: /msg O CHAN CLEAR #channel { KICK | BAN | KILL | GLINE }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_chan_clear(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL nicks;
	Condata Cdata;
	char reason[300];
	int i = 0;
	time_t secs = 0;

	/* Initialise Variables to (NULL) */
	bzero(&nicks,sizeof(nicks));
	bzero(&Cdata,sizeof(Cdata));

	/*
	 * Chan Clear Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chan clear")){
		nicks.res		= doSelectMySQL(thread,"SELECT `orion_chan_users`.`nick`,`orion_core_condata`.`modes`,`orion_core_condata`.`vhost`,CONCAT('*@',`orion_core_condata`.`rhost`) AS `realhost`,`orion_core_condata`.`ident` FROM `orion_chan_users` LEFT JOIN `orion_core_condata` ON `orion_core_condata`.`nick`=`orion_chan_users`.`nick` WHERE `orion_chan_users`.`faketype`=0 AND `channel`='%s'",mysql_quote(thread,param[5]));
		nicks.fields	= mysql_num_rows(nicks.res);

		if(nicks.fields){
			if(m->WallOSClearchan && (streqnc(param[6],"ban") || streqnc(param[6],"kick") || streqnc(param[6],"kill") || streqnc(param[6],"gline")))
				do_global(thread,O->nick,"Clearchan used by %s, for channel %s - with option %s, matching ~%d user(s)",thread->sql.aliases.nick,param[5],param[6],nicks.fields);
			
			if(streqnc(param[6],"ban")){
				Channel c=sql_chan(thread,param[5]);
				
				for(i=0;i<nicks.fields;i++){
					nicks.row = mysql_fetch_row(nicks.res);

					if(strcor(nicks.row[1],"o"))
						continue;

					dynncpy(Cdata.nick,nicks.row[0],sizeof(Cdata.nick));
					dynncpy(Cdata.ident,nicks.row[4],sizeof(Cdata.ident));
					dynncpy(Cdata.vhost,nicks.row[2],sizeof(Cdata.vhost));

					do_mode(thread,snick,param[5],c.datecreate,"+b %s",BuildBanMask(thread,&Cdata,CSAnalyseBantype(c.settings)));
					do_kick(thread,snick,param[5],nicks.row[2],"resolved clearchan (ban).");
				}
			}else if(streqnc(param[6],"kick")){
				for(i=0;i<nicks.fields;i++){
					nicks.row = mysql_fetch_row(nicks.res);

					if(strcor(nicks.row[1],"o"))
						continue;

					do_kick(thread,snick,param[5],nicks.row[0],"resolved clearchan (kick).");
				}
			}else if(streqnc(param[6],"kill")){
				for(i=0;i<nicks.fields;i++){
					nicks.row = mysql_fetch_row(nicks.res);

					if(strcor(nicks.row[1],"o"))
						continue;

					do_kill(thread,snick,nicks.row[0],"resolved clearchan for %s (kill).",param[5]);
				}
			}else if(streqnc(param[6],"gline")){
				if(param[7]){
					if(!check_expire(param[7])){
						send_lang(thread,language,howtosend,snick,nick,"INVALID_EXPIRESTRING",param[7]);
						return MOD_CONT;
					}
					secs = expire(param[7])+time(NULL);
				}
				else
					secs = expire(m->glineonclearchan)+time(NULL);
				
				for(i=0;i<nicks.fields;i++){
					nicks.row = mysql_fetch_row(nicks.res);

					if(strcor(nicks.row[1],"o"))
						continue;
					
					if(param[7]){
						snprintf(reason,sizeof(reason),"resolved clearchan for %s (gline %s)",param[5],param[7]);
						do_gline(thread,m->servername,nicks.row[3],_dynadd("%s!%s@%s",O->nick,O->ident,O->host),secs,time(NULL),reason);
					}else{
						snprintf(reason,sizeof(reason),"resolved clearchan for %s (gline %s)",param[5],m->glineonclearchan);
						do_gline(thread,m->servername,nicks.row[3],_dynadd("%s!%s@%s",O->nick,O->ident,O->host),secs,time(NULL),reason);
					}
				}
			}
			send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_EXISTING");

		/* mysql_free_result_myr(); - nicks */
		mysql_free_result_myr(nicks.res);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O CHAN Move function for the O CHAN Command.
 * 
 * The O CHAN Move Command allows an admin to move the Database for taget Channel<br/>
 *    from Q to ChanServ or from ChanServ to Q!
 *  
 * Syntax is: /msg O CHAN MOVE #channel
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_chan_move(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	Channel c;
	ChanLevels cLev;
	deepSQL chanaccess;
	char level      [BUFSIZE];
	char mlock      [MAX_CHAN_MODES];
	char mlock_modes[BUFSIZE];
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&c,sizeof(c));
	bzero(&cLev,sizeof(cLev));
	bzero(&chanaccess,sizeof(chanaccess));
	bzero(&level,sizeof(level));
	bzero(&mlock,sizeof(mlock));
	bzero(&mlock_modes,sizeof(mlock_modes));

	/*
	 * Chan Move Command
	 */
	c = sql_chan(thread,param[5]);
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chan move")){
		if(streqnc(c.serv,"q") && c.fields)	{
			do_part(thread,Q->nick,param[5],"processing movechan ...");
			
			/* Q -> ChanServ */
			c.settings = 0;
			
			//bcfkloptvw
			if(strcor("b",c.flags)) if(~c.settings&CS_SECUREOPS_BIT) c.settings |= CS_SECUREOPS_BIT;
			//if(strcor("c",c.flags)) if(~c.settings&CS_SECUREOPS_BIT) c.settings += (1<<CS_SECUREOPS); // - No Autolimit for ChanServ
			if(strcor("f",c.flags)) if(~c.settings&CS_TOPICLOCK_BIT) c.settings |= CS_TOPICLOCK_BIT;
			if(strcor("k",c.flags)){
				straddns(mlock,_dynadd("+%c",IRC.KeyCMode),sizeof(mlock));
				straddns(mlock_modes,c.key,sizeof(mlock_modes));
				straddnc(mlock_modes,' ',sizeof(mlock_modes));
			}
			if(strcor("l",c.flags)){
				straddns(mlock,_dynadd("+%c",IRC.LimitCMode),sizeof(mlock));
				straddns(mlock_modes,_dynadd("%ld",c.limit),sizeof(mlock_modes));
				straddnc(mlock_modes,' ',sizeof(mlock_modes));
			}
			//if(strcor("o",c.flags)) if(~c.settings&CS_SECUREOPS_BIT) c.settings += (1<<CS_SECUREOPS); // No such level
			if(strcor("p",c.flags)) if(~c.settings&CS_SECUREOPS_BIT) c.settings |= CS_SECUREOPS_BIT;
			if(strcor("t",c.flags)) if(~c.settings&CS_KEEPTOPIC_BIT) c.settings |= CS_KEEPTOPIC_BIT;
			//if(strcor("v",c.flags)) if(~c.settings&CS_SECUREOPS_BIT) c.settings += (1<<CS_SECUREOPS); // No such level
			//if(strcor("w",c.flags)) if(~c.settings&CS_SECUREOPS_BIT) c.settings += (1<<CS_SECUREOPS); // No such level
			if(~c.settings&CS_XOP_BIT) c.settings |= CS_XOP_BIT;
					
			strclean(mlock_modes);
						
			domysql(thread,
				"UPDATE `orion_cs_core` SET "
							
				"`serv`='ChanServ',`last_used`='%ld',`settings`='%d',"
				"`cs_set`='%d',`cs_autodeop`='%d',`cs_autohalfop`='%d',`cs_autoop`='%d',`cs_autoprotect`='%d',`cs_autovoice`='%d',"
				"`cs_nojoin`='%d',`cs_signkick`='%d',`cs_acclist`='%d',`cs_accchange`='%d',`cs_akick`='%d',`cs_ban`='%d',`cs_banme`='%d',"
				"`cs_cclear`='%d',`cs_getkey`='%d',`cs_halfop`='%d',`cs_halfopme`='%d',`cs_info`='%d',`cs_kick`='%d',`cs_kickme`='%d',`cs_invite`='%d',"
				"`cs_opdeop`='%d',`cs_opdeopme`='%d',`cs_protect`='%d',`cs_protectme`='%d',`cs_topic`='%d',`cs_unban`='%d',`cs_voice`='%d',"
				"`cs_voiceme`='%d',`cs_memo`='%d',`cs_botassign`='%d',`cs_badwords`='%d',`cs_fantasia`='%d',`cs_greet`='%d',`cs_nokick`='%d',"
				"`cs_say`='%d',"
				"`bs_set`='-1',"
				"`bs_caps_min`='-1',`bs_caps_per`='-1',`bs_flood_ln`='-1',`bs_flood_secs`='-1',`bs_repeat`='-1',`bs_colors`='-1',"
				"`bs_underlines`='-1',`bs_reverses`='-1',`bs_badwords`='-1',`bs_bolds`='-1'"
		
				" WHERE `channel`='%s'",
								
				time(NULL),c.settings,
				CS_SET,CS_AUTODEOP,CS_AUTOHALFOP,CS_AUTOOP,CS_AUTOPROTECT,CS_AUTOVOICE,
				CS_NOJOIN,CS_SIGNKICK,CS_ACCESS_LIST,CS_ACCESS_CHANGE,CS_AKICK,CS_BAN,CS_BANME,
				CS_CLEAR,CS_GETKEY,CS_HALFOP,CS_HALFOPME,CS_INFO,CS_KICK,CS_KICKME,CS_INVITE,
				CS_OPDEOP,CS_OPDEOPME,CS_PROTECT,CS_PROTECTME,CS_TOPIC,CS_UNBAN,CS_VOICE,
				CS_VOICEME,CS_MEMO,CS_ASSIGN,CS_BADWORDS,CS_FANTASIA,CS_GREET,CS_NOKICK,
				CS_SAY,
				mysql_quote(thread,c.name)
			);
			dynncpy(c.serv,"ChanServ",sizeof(c.serv));
			
			
			domysql(thread,"UPDATE `orion_cs_access` SET `level`=%d WHERE `channel`='%s' AND `level` RLIKE '(n)'",ACCESS_FOUNDER,mysql_quote(thread,c.name));
			domysql(thread,"UPDATE `orion_cs_access` SET `level`=%d WHERE `channel`='%s' AND `level` RLIKE '(m)'",ACCESS_SUCCESSOR,mysql_quote(thread,c.name));
					
			domysql(thread,"UPDATE `orion_cs_access` SET `level`=%d WHERE `channel`='%s' AND `level` RLIKE '(a*p)'",CS_AUTOPROTECT,mysql_quote(thread,c.name));
			domysql(thread,"UPDATE `orion_cs_access` SET `level`=%d WHERE `channel`='%s' AND `level` RLIKE '(a*o)'",CS_AUTOOP,mysql_quote(thread,c.name));
			domysql(thread,"UPDATE `orion_cs_access` SET `level`=%d WHERE `channel`='%s' AND `level` RLIKE '(a*h)'",CS_AUTOHALFOP,mysql_quote(thread,c.name));
			domysql(thread,"UPDATE `orion_cs_access` SET `level`=%d WHERE `channel`='%s' AND `level` RLIKE '(a*v)'",CS_AUTOVOICE,mysql_quote(thread,c.name));
					
			domysql(thread,"UPDATE `orion_cs_access` SET `level`=%d WHERE `channel`='%s' AND `level` RLIKE '(p)'",CS_PROTECT,mysql_quote(thread,c.name));
			domysql(thread,"UPDATE `orion_cs_access` SET `level`=%d WHERE `channel`='%s' AND `level` RLIKE '(o)'",CS_OPDEOP,mysql_quote(thread,c.name));
			domysql(thread,"UPDATE `orion_cs_access` SET `level`=%d WHERE `channel`='%s' AND `level` RLIKE '(h)'",CS_HALFOP,mysql_quote(thread,c.name));
			domysql(thread,"UPDATE `orion_cs_access` SET `level`=%d WHERE `channel`='%s' AND `level` RLIKE '(v)'",CS_VOICE,mysql_quote(thread,c.name));

			domysql(thread,"UPDATE `orion_cs_access` SET `serv`='ChanServ' WHERE `channel`='%s'",mysql_quote(thread,c.name));

			//do_join(thread,1,cs->nick,param[5],c.datecreate == 0 ? time(NULL) : c.datecreate);
			//do_mode(thread,cs->nick,param[5],c.datecreate == 0 ? time(NULL) : c.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
			bzero(&thread->c,sizeof(thread->c));
			do_mlock(thread,cs->nick,c.name,c.datecreate == 0 ? time(NULL) : c.datecreate,mlock,mlock_modes);
			send_lang(thread,language,howtosend,snick,nick,"O_CHAN_MOVE_MOVED_CHANNEL_X_FROM_Y_TO_Z",c.name,"Q",c.serv);
		}else if(streqnc(c.serv,"chanserv") && c.fields){
			do_part(thread,cs->nick,param[5],"processing movechan ...");
			
			/* ChanServ -> Q */
			bzero(&c.flags,sizeof(c.flags));
			//bcfkloptvw
			if(c.settings&CS_SECUREOPS_BIT) straddnc(c.flags,'b',sizeof(c.flags));
			if(c.settings&CS_TOPICLOCK_BIT) straddnc(c.flags,'f',sizeof(c.flags));
			if(strcin(IRC.KeyCMode,c.mlock_positive)){
				straddnc(c.flags,'k',sizeof(c.flags));
				//bzero(&c.key,sizeof(c.key));
				dynncpy(c.key,c.mlock_key,sizeof(c.key));
			}
			if(strcin(IRC.LimitCMode,c.mlock_positive)){
				straddnc(c.flags,'l',sizeof(c.flags));
				c.limit = c.mlock_limit;
			}
			//if(strcor("o",c.flags)) if(~c.settings&CS_SECUREOPS_BIT) c.settings += (1<<CS_SECUREOPS); // No such level
			if(c.settings&CS_SECUREOPS_BIT) straddnc(c.flags,'p',sizeof(c.flags));
			if(c.settings&CS_KEEPTOPIC_BIT) straddnc(c.flags,'t',sizeof(c.flags));
			//if(strcor("v",c.flags)) if(~c.settings&CS_SECUREOPS_BIT) c.settings += (1<<CS_SECUREOPS); // No such level
			//if(strcor("w",c.flags)) if(~c.settings&CS_SECUREOPS_BIT) c.settings += (1<<CS_SECUREOPS); // No such level
			sort(c.flags);
						
			domysql(thread,
				"UPDATE `orion_cs_core` SET `serv`='Q',`last_used`='%ld',`flags`='%s' WHERE `channel`='%s'",time(NULL),mysql_quote(thread,c.flags),mysql_quote(thread,c.name)
			);
			dynncpy(c.serv,"Q",sizeof(c.serv));
			
			cLev 			  = sql_chanlevels(thread,c.name);
			chanaccess.res    = doSelectMySQL(thread,"SELECT `level`,`authnick` FROM `orion_cs_access` WHERE `channel`='%s'",mysql_quote(thread,c.name));
			chanaccess.fields = mysql_num_rows(chanaccess.res);
			
			for(i=0;i<chanaccess.fields;i++){
				chanaccess.row = mysql_fetch_row(chanaccess.res);
				bzero(&level,sizeof(level));
				
				if(stoi(chanaccess.row[0]) >= ACCESS_FOUNDER)		straddsni(level,"n");
				if(stoi(chanaccess.row[0]) >= ACCESS_SUCCESSOR)		straddsni(level,"m");
				
				if(stoi(chanaccess.row[0]) >= cLev.accchange)		straddsni(level,"m");
				if(stoi(chanaccess.row[0]) >= cLev.acclist)			straddsni(level,"t");
				//(stoi(chanaccess.row[0]) >= cLev.akick)			straddsni(level,"o");
				//(stoi(chanaccess.row[0]) >= cLev.autodeop)		straddsni(level,"o");
				if(stoi(chanaccess.row[0]) >= cLev.autohalfop)		straddsni(level,"ah");
				if(stoi(chanaccess.row[0]) >= cLev.autoop)			straddsni(level,"ao");
				if(stoi(chanaccess.row[0]) >= cLev.autoprotect)		straddsni(level,"ap");
				if(stoi(chanaccess.row[0]) >= cLev.autovoice)		straddsni(level,"av");
				//stoi(chanaccess.row[0]) >= cLev.badwords)			straddsni(level,"o")
				if(stoi(chanaccess.row[0]) >= cLev.ban)				straddsni(level,"o");
				//(stoi(chanaccess.row[0]) >= cLev.banme)			straddsni(level,"n");
				if(stoi(chanaccess.row[0]) >= cLev.assign)			straddsni(level,"o");
				if(stoi(chanaccess.row[0]) >= cLev.clear)			straddsni(level,"m");
				//(stoi(chanaccess.row[0]) >= cLev.fantasia)		straddsni(level,"n");
				if(stoi(chanaccess.row[0]) >= cLev.getkey)			straddsni(level,"o");
				if(stoi(chanaccess.row[0]) >= cLev.greet)			straddsni(level,"m");
				if(stoi(chanaccess.row[0]) >= cLev.halfop)			straddsni(level,"h");
				//(stoi(chanaccess.row[0]) >= cLev.halfopme)		straddsni(level,"h);
				if(stoi(chanaccess.row[0]) >= cLev.info)			straddsni(level,"t");
				if(stoi(chanaccess.row[0]) >= cLev.invite)			straddsni(level,"o");
				if(stoi(chanaccess.row[0]) >= cLev.kick)			straddsni(level,"h");
				//(stoi(chanaccess.row[0]) >= cLev.kickme)			straddsni(level,"h");
				if(stoi(chanaccess.row[0]) >= cLev.memo)			straddsni(level,"o");
				//(stoi(chanaccess.row[0]) >= cLev.nojoin)
				//(stoi(chanaccess.row[0]) >= cLev.nokick)
				if(stoi(chanaccess.row[0]) >= cLev.opdeop)			straddsni(level,"o");
				//(stoi(chanaccess.row[0]) >= cLev.opdeopme)		straddsni(level,"o");
				if(stoi(chanaccess.row[0]) >= cLev.protect)			straddsni(level,"p");
				//(stoi(chanaccess.row[0]) >= cLev.protectme)		straddsni(level,"p");
				if(stoi(chanaccess.row[0]) >= cLev.say)				straddsni(level,"o");
				if(stoi(chanaccess.row[0]) >= cLev.set)				straddsni(level,"m");
				//(stoi(chanaccess.row[0]) >= cLev.signkick)		straddsni(level,"o");
				if(stoi(chanaccess.row[0]) >= cLev.topic)			straddsni(level,"t");
				if(stoi(chanaccess.row[0]) >= cLev.unban)			straddsni(level,"o");
				if(stoi(chanaccess.row[0]) >= cLev.voice)			straddsni(level,"v");
				//(stoi(chanaccess.row[0]) >= cLev.voiceme)			straddsni(level,"v");
				
				sort(level);
				domysql(thread,"UPDATE `orion_cs_access` SET `level`='%s' WHERE `channel`='%s' AND `authnick`='%s'",mysql_quote(thread,level),mysql_quote(thread,c.name),mysql_quote(thread,chanaccess.row[1]));
			}

			domysql(thread,"UPDATE `orion_cs_access` SET `serv`='Q' WHERE `channel`='%s'",mysql_quote(thread,c.name));

			bzero(&thread->c,sizeof(thread->c));
			do_join(thread,1,Q->nick,param[5],c.datecreate == 0 ? time(NULL) : c.datecreate);
			do_mode(thread,Q->nick,param[5],c.datecreate == 0 ? time(NULL) : c.datecreate,"+oaq %s %s %s",Q->nick,Q->nick,Q->nick);
			
			if(c.key[0]!='\0' && c.limit!=0)
				do_mode(thread,Q->nick,c.name,c.datecreate == 0 ? time(NULL) : c.datecreate,"+kl %s %ld",c.key,c.limit);
			else if(c.key[0]=='\0' && c.limit!=0)
				do_mode(thread,Q->nick,c.name,c.datecreate == 0 ? time(NULL) : c.datecreate,"+l %ld",c.limit);
			else if(c.key[0]!='\0' && c.limit==0)
				do_mode(thread,Q->nick,c.name,c.datecreate == 0 ? time(NULL) : c.datecreate,"+k %s",c.key);
				
			send_lang(thread,language,howtosend,snick,nick,"O_CHAN_MOVE_MOVED_CHANNEL_X_FROM_Y_TO_Z",c.name,"ChanServ",c.serv);

			/* MySQL Free Result */
			mysql_free_result_myr(chanaccess.res);
		}else send_lang(thread,language,howtosend,snick,nick,"O_CHAN_MOVE_CANNOT_MOVE",c.name);

	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O CHAN Rename function for the O CHAN Command.
 * 
 * The O CHAN Rename Command allows an admin to move the Database for taget Channel<br/>
 *    and the Users for target Channel to a NewChannel (Renames the Channel)
 *  
 * Syntax is: /msg O CHAN RENAME #channel #newchannel
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_chan_rename(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	ChannelDB c;
	Channel   c5;
	deepSQL users;
	bool done = 0;

	/* Initialise Variables to (NULL) */
	bzero(&c,sizeof(c));
	bzero(&c5,sizeof(c5));
	bzero(&users,sizeof(users));
	/*
	 * Chan Rename Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chan rename")){
		c = sql_chan_db(thread,param[5]);	
		if(c.fields){
			if((c.settings & CS_FORBID_BIT))		send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",c.serv,c.reason);
			else if((c.settings & CS_SUSPEND_BIT))	send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",c.serv,c.reason);
			else{
				c5 = sql_chan(thread,param[6]);
				if(c5.name[0]=='\0'){
					if(c5.serv[0]=='\0' && (~c.settings & CS_FORBID_BIT)){
						if(streqnc(c.serv,"q")){
							//domysql(thread,"UPDATE `orion_cs_core` SET `channel`='%s' WHERE `channel`='%s' AND `serv`='Q'",mysql_quote(thread,param[5]),mysql_quote(thread,param[6]));
							domysql(thread,"UPDATE `orion_cs_core`,`orion_ms_core` LEFT OUTER JOIN `orion_ms_core` ON (`orion_ms_core`.`receiver`=`orion_cs_core`.`channel`) SET `orion_cs_core`.`channel`='%s',`orion_ms_core`.`receiver`='%s' WHERE `orion_cs_core`.`channel`='%s' AND `orion_cs_core`.`serv`='Q'",mysql_quote(thread,param[6]),mysql_quote(thread,param[6]),mysql_quote(thread,param[5]));
							/* 
							InnoDB: using ON UPDATE/DELETE CASCADE -> Update/Delete not neccessary anymore
							domysql(thread,"UPDATE `orion_cs_access` SET `channel`='%s' WHERE `channel`='%s' AND `serv`='Q'",mysql_quote(thread,param[6]),mysql_quote(thread,param[5]));
							*/
							send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
							do_part(thread,Q->nick,param[5],_dynadd("[%s] ordered a move to %s",nick,param[6]));
							do_join(thread,2,Q->nick,param[6],c5.datecreate == 0 ? time(NULL) : c5.datecreate);
							do_mode(thread,Q->nick,param[6],c5.datecreate == 0 ? time(NULL) : c5.datecreate,"+oaq %s %s %s",Q->nick,Q->nick,Q->nick);
							
							if(c.bot[0]!='\0'){
								do_part(thread,c.bot,param[5],_dynadd("[%s] ordered a move to %s",nick,param[6]));
								do_join(thread,1,c.bot,param[6],c5.datecreate == 0 ? time(NULL) : c5.datecreate);
								do_mode(thread,c.bot,param[6],c5.datecreate == 0 ? time(NULL) : c5.datecreate,"+oaq %s %s %s",c.bot,c.bot,c.bot);
							}
							done=1;
						}/*else if(streqnc(c.serv,"l")){
							//domysql(thread,"UPDATE `orion_cs_core` SET `channel`='%s' WHERE `channel`='%s' AND `serv`='L'",mysql_quote(thread,param[5]),mysql_quote(thread,param[6]));
							domysql(thread,"UPDATE `orion_cs_core` LEFT OUTER JOIN `orion_ms_core` ON (`orion_ms_core`.`receiver`=`orion_cs_core`.`channel`) SET `orion_cs_core`.`channel`='%s',`orion_ms_core`.`receiver`='%s' WHERE `orion_cs_core`.`channel`='%s' AND `orion_cs_core`.`serv`='L'",mysql_quote(thread,param[6]),mysql_quote(thread,param[6]),mysql_quote(thread,param[5]));
							
							//InnoDB: using ON UPDATE/DELETE CASCADE -> Update/Delete not neccessary anymore
							//domysql(thread,"UPDATE `orion_cs_access` SET `channel`='%s' WHERE `channel`='%s' AND `serv`='L'",mysql_quote(thread,param[6]),mysql_quote(thread,param[5]));

							send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
							do_part(thread,L->nick,param[5],_dynadd("[%s] ordered a move to %s",nick,param[6]));
							do_join(thread,2,L->nick,param[6],c5.datecreate == 0 ? time(NULL) : c5.datecreate);
							do_mode(thread,L->nick,param[6],c5.datecreate == 0 ? time(NULL) : c5.datecreate,"+oaq %s %s %s",L->nick,L->nick,L->nick);
							done=1;
						}*/else if(c.fields && streqnc(c.serv,"chanserv")){
							//domysql(thread,"UPDATE `orion_cs_core` SET `channel`='%s' WHERE `channel`='%s' AND `serv`='ChanServ'",mysql_quote(thread,param[5]),mysql_quote(thread,param[6]));
							domysql(thread,"UPDATE `orion_cs_core` LEFT OUTER JOIN `orion_ms_core` ON (`orion_ms_core`.`receiver`=`orion_cs_core`.`channel`) SET `orion_cs_core`.`channel`='%s',`orion_ms_core`.`receiver`='%s' WHERE `orion_cs_core`.`channel`='%s' AND `orion_cs_core`.`serv`='ChanServ'",mysql_quote(thread,param[6]),mysql_quote(thread,param[6]),mysql_quote(thread,param[5]));
							/* 
							InnoDB: using ON UPDATE/DELETE CASCADE -> Update/Delete not neccessary anymore
							domysql(thread,"UPDATE `orion_cs_access` SET `channel`='%s' WHERE `channel`='%s' AND `serv`='ChanServ'",mysql_quote(thread,param[6]),mysql_quote(thread,param[5]));
							*/
							send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
							
							if(c.bot[0]!='\0'){
								do_part(thread,c.bot,param[5],_dynadd("[%s] ordered a move to %s",nick,param[6]));
								do_join(thread,1,c.bot,param[6],c5.datecreate == 0 ? time(NULL) : c5.datecreate);
								do_mode(thread,c.bot,param[6],c5.datecreate == 0 ? time(NULL) : c5.datecreate,"+oaq %s %s %s",c.bot,c.bot,c.bot);
							}
							done=1;
						}
					}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_ALREADY_REGISTERED",c5.serv);
	
					if(done==1){
						users.res = doSelectMySQL(thread,"SELECT `nick`,`modes` FROM `orion_chan_users` WHERE `channel`='%s' AND `faketype`='0'",mysql_quote(thread,param[5])); //prepare Query
						while((users.row = mysql_fetch_row(users.res))){ //work out result
							do_svspart(thread,Q->nick,users.row[0],param[5]);
							do_svsjoin(thread,0,Q->nick,users.row[0],param[6],c5.datecreate == 0 ? time(NULL) : c5.datecreate);
							
							//if(users.row[1][0]!='\0'){
							//	printf("modes = %s\n",users.row[1]);
							//	do_modecompress(thread,Q->nick,param[6],_dynadd("+%s",users.row[1]),users.row[0],c5.datecreate == 0 ? time(NULL) : c5.datecreate);
							//}
							
							/* It was planned to readd all UserChannelModes (+qohv) but, this may lead to desync's with other ircd's
							   cauzed by SVSJOIN / SVSPART ... */
						}
						//send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
						mysql_free_result_myr(users.res); //free MySQL result
					}
				}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_ALREADY_EXISTING");
			}
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_EXISTING");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O CHAN Force function for the O CHAN Command.
 * 
 * The O CHAN Force Command allows an admin to force all Users who arn't<br/>
 *    in any Channel on the Network to you the given Channel
 *  
 * Syntax is: /msg O CHAN FORCE #channel
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_chan_force(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL condatalist;
	Channel c;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&condatalist,sizeof(condatalist));
	bzero(&c,sizeof(c));

	/*
	 * Chan Force Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chan force")){
		if(!IRC.SupportSVSJoin){
			send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
			return MOD_CONT;
		}
		
		c = sql_chan(thread,param[5]);
		do_join(thread,2,snick,param[5],c.datecreate == 0 ? time(NULL) : c.datecreate);
		
		if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_chan_users` WHERE `channel`='%s' AND `nick`='%s'",mysql_quote(thread,c.name),mysql_quote(thread,nick)))==0)
			do_svsjoin(thread,0,snick,nick,param[5],c.datecreate == 0 ? time(NULL) : c.datecreate);

		condatalist.res    = doSelectMySQL(thread,"SELECT `orion_core_condata`.`nick` FROM `orion_core_condata` LEFT JOIN `orion_chan_users` ON `orion_core_condata`.`nick` = `orion_chan_users`.`nick` WHERE (`orion_core_condata`.`faketype`=0 AND `orion_chan_users`.`nick` IS NULL)");
		condatalist.fields = mysql_num_rows(condatalist.res);
		
		c.datecreate = (time_t)stol(mysql_sel_var(thread,"SELECT `datecreate` FROM `orion_cs_core` WHERE `channel`='%s'",mysql_quote(thread,c.name)));
		
		for(i=0;i<condatalist.fields;i++){
			condatalist.row = mysql_fetch_row(condatalist.res);
			
			do_svsjoin(thread,0,snick,condatalist.row[0],param[5],c.datecreate);
		}
		do_part(thread,snick,param[5],"You all were forced to join this channel");
		send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");

		/* mysql_free_result_myr(); - condatalist */
		mysql_free_result_myr(condatalist.res);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O CHAN GlobMode function for the O CHAN Command.
 * 
 * The O CHAN GlobMode Command allows an admin to correct ChannelModes in All Channel<br/>
 *    over the whole Network (use with care)
 *  
 * Syntax is: /msg O CHAN GLOBMODE < mode >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_chan_globmode(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL modes;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&modes,sizeof(modes));

	/*
	 * Chan GlobMode Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chan globmode")){
		
		modes.res    = doSelectMySQL(thread,"SELECT `orion_cs_core`.`channel`,`orion_cs_core`.`datecreate` FROM `orion_cs_core`");
		modes.fields = mysql_num_rows(modes.res);
		for(i=0;i<modes.fields;i++){
			modes.row = mysql_fetch_row(modes.res);
			
			do_mode(thread,snick,modes.row[0],(time_t)stol(modes.row[1]),_dstrjoin(' ',param,5));
		}
		send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");

		/* mysql_free_result_myr(); - condatalist */
		mysql_free_result_myr(modes.res);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O CHAN Op function for the O CHAN Command.
 * 
 * The O CHAN Op Command allows an admin to Op himself on the given Channel<br/>
 *    or any other User
 *  
 * Syntax is: /msg O CHAN OP #channel [nick]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_chan_op(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	Channel c;

	/* Initialise Variables to (NULL) */
	bzero(&c,sizeof(c));

	/*
	 * Chan Op Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chan op")){
		c = sql_chan(thread,param[5]);

		if(c.name[0]!='\0'){
			if(!param[6]){
				do_mode(thread,snick,param[5],c.datecreate,"+o %s",nick);
				send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
			}else if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_chan_users` WHERE `nick`='%s' AND `channel`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,param[5])))){
				do_mode(thread,snick,param[5],c.datecreate,"+o %s",param[6]);
				send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_KNOWN_USER",param[6]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_EXISTING");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O CHAN Kick function for the O CHAN Command.
 * 
 * The O CHAN Kick Command allows an admin to Kick any user out of<br/>
 *    given Channel
 *  
 * Syntax is: /msg O CHAN KICK #channel nick
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_chan_kick(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL chanuser;

	/* Initialise Variables to (NULL) */
	bzero(&chanuser,sizeof(chanuser));

	/*
	 * Chan Kick Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chan kick")){
		chanuser.res	= doSelectMySQL(thread,"SELECT `nick`,`faketype` FROM `orion_chan_users` WHERE `channel`='%s' AND `nick`='%s'",param[5],param[6]);
		chanuser.fields = mysql_num_rows(chanuser.res);
		
		if(chanuser.fields){
			chanuser.row = mysql_fetch_row(chanuser.res);
			if(stoi(chanuser.row[1])==2)
				do_kill(thread,m->servername,nick,"services abuse");
			else{
				if(m->WallOSKick)
					do_global(thread,O->nick,"Kick used for %s on channel %s (reason: %s)",chanuser.row[0],param[5],_dstrjoin(' ',param,7));
			
				do_kick(thread,snick,param[5],chanuser.row[0],_dstrjoin(' ',param,7));
				send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
			}
		}else send_lang(thread,language,howtosend,snick,nick,"NOT_KNOWN_USER",param[6]);

		/* MySQL Free Result */
		mysql_free_result_myr(chanuser.res);
	}else{
	/*
	 *  Error Messages
	 */
		if(!param[7])	send_lang(thread,language,howtosend,snick,nick,"NO_REASON");
		else			send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O CHAN Deopall function for the O CHAN Command.
 * 
 * The O CHAN Deopall Command allows an admin to deop all users (Including Q)<br/>
 *    in the given Channel
 *  
 * Syntax is: /msg O CHAN DEOPALL #channel
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_chan_deopall(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Chan Deopall Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chan deopall")){
		if(deopall(thread,snick,param[5])){
			send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_EXISTING");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the O CHAN Command (/msg O HELP CHAN)
 * 
 * When the user tries to get help for this command, using /msg O HELP CHAN,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP CHAN
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_chan_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Chan Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help chan")) {
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_CHAN");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O CHAN Clear Help function for the O CHAN Command (/msg O HELP CHAN CLEAR)
 * 
 * When the user tries to get help for this command, using /msg O HELP CHAN CLEAR,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP CHAN CLEAR
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_chan_clear_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Chan Clear Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help chan clear")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chan clear"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_CHAN_CLEAR");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O CHAN Move Help function for the O CHAN Command (/msg O HELP CHAN MOVE)
 * 
 * When the user tries to get help for this command, using /msg O HELP CHAN MOVE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP CHAN MOVE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_chan_move_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Chan Move Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help chan move")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chan move"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_CHAN_MOVE");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O CHAN Rename Help function for the O CHAN Command (/msg O HELP CHAN RENAME)
 * 
 * When the user tries to get help for this command, using /msg O HELP CHAN RENAME,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP CHAN RENAME
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_chan_rename_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Chan Rename Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help chan rename")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chan rename"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_CHAN_RENAME");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O CHAN Force Help function for the O CHAN Command (/msg O HELP CHAN FORCE)
 * 
 * When the user tries to get help for this command, using /msg O HELP CHAN FORCE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP CHAN FORCE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_chan_force_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Chan Force Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help chan force")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chan force"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_CHAN_FORCE");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O CHAN GlobMode Help function for the O CHAN Command (/msg O HELP CHAN GLOBMODE)
 * 
 * When the user tries to get help for this command, using /msg O HELP CHAN GLOBMODE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP CHAN GLOBMODE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_chan_globmode_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Chan GlobMode Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help chan globmode")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chan globmode"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_CHAN_GLOBMODE");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O CHAN Op Help function for the O CHAN Command (/msg O HELP CHAN OP)
 * 
 * When the user tries to get help for this command, using /msg O HELP CHAN OP,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP CHAN OP
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_chan_op_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Chan Op Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help chan op")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chan op"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_CHAN_OP",snick);
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O CHAN Kick Help function for the O CHAN Command (/msg O HELP CHAN KICK)
 * 
 * When the user tries to get help for this command, using /msg O HELP CHAN KICK,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP CHAN KICK
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_chan_kick_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Chan Kick Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help chan kick")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chan kick"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_CHAN_KICK");

	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O CHAN Deopall Help function for the O CHAN Command (/msg O HELP CHAN DEOPALL)
 * 
 * When the user tries to get help for this command, using /msg O HELP CHAN DEOPALL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP CHAN DEOPALL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_chan_deopall_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Chan DeopAll Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help chan deopall")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chan deopall"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_CHAN_DEOPALL");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief O Help List Command - CHAN part (/msg O help)
 * 
 * When the user tries to get general O help, using /msg O help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the CHAN command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_chan_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_O_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Chan (Help listing)
	 */
	if(Users_O_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"O_SHOWCOMMANDS_CHAN",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chan"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
