/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_spew.c
 *   @brief This is the Module file for O SPEW Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the O SPEW command.
 *
 *   The O SPEW Command allows an admin to search<br/>
 *   with patterns in various categories.
 *
 *   Syntax is: /msg O SPEW { RHOST | EMAIL | GLINE | IDENT | VHOST | CHAN } ...
 *
 *<table border="0">
 *    <tr><td> RHOST	</td><td> Syntax is: /msg O SPEW RHOST	</td><td> < *realhost-pattern* >	</td><td> (Dumps all realhosts matching the pattern)		</td></tr>
 *    <tr><td> EMAIL	</td><td> Syntax is: /msg O SPEW EMAIL	</td><td> < *email-pattern* >		</td><td> (Dumps all emails matching the pattern)			</td></tr>
 *    <tr><td> GLINE	</td><td> Syntax is: /msg O SPEW GLINE	</td><td> < *gline-pattern* >		</td><td> (Dumps all glines matching the pattern)			</td></tr>
 *    <tr><td> IDENT	</td><td> Syntax is: /msg O SPEW IDENT	</td><td> < *ident-pattern* >		</td><td> (Dumps all idents matching the pattern)			</td></tr>
 *    <tr><td> VHOST	</td><td> Syntax is: /msg O SPEW VHOST	</td><td> < *vhost-pattern* >		</td><td> (Dumps all virtual-hosts matching the pattern)	</td></tr>
 *    <tr><td> CHAN		</td><td> Syntax is: /msg O SPEW CHAN	</td><td> < *chan-pattern* >		</td><td> (Dumps all channels matching the pattern)			</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "O"
#define MODUL "spew"
#define MODUL_FILE "o_spew"
#define MODUL_VERSION "1.0"

MOD_BOOL O_spew_rhost(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_spew_email(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_spew_gline(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_spew_ident(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_spew_vhost(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_spew_chan(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_spew_rhost_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_spew_email_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_spew_gline_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_spew_ident_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_spew_vhost_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_spew_chan_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_spew_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_spew_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"spew option[1]","spew helpoption[1]");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,OPERONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addLeastModuleLevels(thread,&Module,"spew",720,INFINIT); /* just to add a displaylevel for /msg o help
															  this should always be the lowest used Module level! */

	addCommand(thread,&Module,"spew email",O_spew_email,720,INFINIT,5,5);
	addCommand(thread,&Module,"spew rhost",O_spew_rhost,720,INFINIT,5,5);
	addCommand(thread,&Module,"spew gline",O_spew_gline,720,INFINIT,5,5);
	addCommand(thread,&Module,"spew ident",O_spew_ident,720,INFINIT,5,5);
	addCommand(thread,&Module,"spew vhost",O_spew_vhost,720,INFINIT,5,5);
	addCommand(thread,&Module,"spew chan",O_spew_chan,720,INFINIT,5,5);
	
	addCommand(thread,&Module,"help spew",O_spew_help,720,INFINIT,4,4);
	addCommand(thread,&Module,"help spew email",O_spew_email_help,720,INFINIT,5,5);
	addCommand(thread,&Module,"help spew rhost",O_spew_rhost_help,720,INFINIT,5,5);
	addCommand(thread,&Module,"help spew gline",O_spew_gline_help,720,INFINIT,5,5);
	addCommand(thread,&Module,"help spew ident",O_spew_ident_help,720,INFINIT,5,5);
	addCommand(thread,&Module,"help spew vhost",O_spew_vhost_help,720,INFINIT,5,5);
	addCommand(thread,&Module,"help spew chan",O_spew_chan_help,720,INFINIT,5,5);
	
	addCommand(thread,&Module,"help",O_spew_help_listing,720,INFINIT,3,3); //100 - each Admin is able to see it.
	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief O SPEW Email function for the O SPEW Command.
 * 
 * The O SPEW Email Command allows an admin to search for known Users using an Email the<br/>
 *   Emailpattern matches.
 *  
 * Syntax is: /msg O SPEW EMAIL *email*
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_spew_email(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL select;
	char pattern[BUFSIZE];
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&select,sizeof(select));
	bzero(&pattern,sizeof(pattern));

	/*
	 * Spew Email Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"spew email") && strcor(param[5],"*")){
		dynncpy(pattern,param[5],sizeof(pattern));
		strrepc(pattern,'*','%');

		select.res    = doSelectMySQL(thread,"SELECT `authnick`,`password`,`email`,`orion_ns_aliases`.`nick` FROM `orion_ns_core` LEFT OUTER JOIN `orion_ns_aliases` ON (`orion_ns_aliases`.`display`=`orion_ns_core`.`authnick` AND `orion_ns_aliases`.`active`=1) WHERE `email` LIKE '%s' ORDER BY `email` ASC LIMIT 80",mysql_quote(thread,pattern));
		select.fields = mysql_num_rows(select.res);

		for(i=0;i<select.fields;i++){
			select.row = mysql_fetch_row(select.res);
			
			if(thread->sql.access.fields && thread->sql.access.olevel>=1000){
				if(select.row[3]!=NULL) 
					send_lang(thread,language,howtosend,snick,nick,"O_SPEW_EMAIL_LIST_FOUND_EMAILUSER_AS_LOGGED_IN_USER_SEE_PASSWORDS",i+1,select.row[3],select.row[0],select.row[2],select.row[1]);
				else
					send_lang(thread,language,howtosend,snick,nick,"O_SPEW_EMAIL_LIST_EMAILUSER_IS_NOT_LOGGED_IN_SEE_PASSWORDS",i+1,select.row[0],select.row[2],select.row[1]);
			}else{
				if(select.row[3]!=NULL)
					send_lang(thread,language,howtosend,snick,nick,"O_SPEW_EMAIL_LIST_FOUND_EMAILUSER_AS_LOGGED_IN_USER",i+1,select.row[3],select.row[0],select.row[2]);
				else
					send_lang(thread,language,howtosend,snick,nick,"O_SPEW_EMAIL_LIST_EMAILUSER_IS_NOT_LOGGED_IN",i+1,select.row[0],select.row[2]);
			}
		}

		mysql_free_result_myr(select.res); //free MySQL result

		send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
		//strrepc(param[4],'%','*');
		if(i != 0)	send_lang(thread,language,howtosend,snick,nick,"O_SPEW_EMAIL_FOUND_X_MATCHING_EMAILS_BY_SEARCHING_WITH_PATTERN",i,param[5]);
		else		send_lang(thread,language,howtosend,snick,nick,"O_SPEW_EMAIL_FOUND_NOTHING_BY_SEARCHING_WITH_PATTERN",param[5]);
	}else{
	/*
	 *  Error Messages
	 */
		if(!strcor(param[5],"*"))	send_lang(thread,language,howtosend,snick,nick,"WRONG_INPUT_PATTERN_INPUT",param[5]);
		else 						send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O SPEW Rhost function for the O SPEW Command.
 * 
 * The O SPEW Rhost Command allows an admin to search for Users using an Rhost the<br/>
 *   Rhostpattern matches.
 *  
 * Syntax is: /msg O SPEW RHOST *rhost*
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_spew_rhost(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL rhostlist;
	char pattern[BUFSIZE];
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&rhostlist,sizeof(rhostlist));
	bzero(&pattern,sizeof(pattern));

	/*
	 * Spew Rhost Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"spew rhost") && strcor(param[5],"*")){
		dyncpy (pattern,param[5]);
		strrepc(pattern,'*','%');
		
		rhostlist.res    = doSelectMySQL(thread,"SELECT `nick`,`rhost`,`vhost` FROM `orion_core_condata` WHERE `rhost` LIKE '%s' ORDER BY `nick` ASC LIMIT 100",mysql_quote(thread,pattern));
		rhostlist.fields = mysql_num_rows(rhostlist.res);

		for(i=0;i<rhostlist.fields;i++){
			rhostlist.row = mysql_fetch_row(rhostlist.res);
			
			send_lang(thread,language,howtosend,snick,nick,"O_SPEW_RHOST_LIST_FOUND",i+1,rhostlist.row[0],rhostlist.row[1],rhostlist.row[2]);
		}
		//strrepc(param[4],'%','*');
		
		if(i != 0)
			send_lang(thread,language,howtosend,snick,nick,"O_SPEW_RHOST_FOUND_X_MATCHING_RHOSTS_BY_SEARCHING_WITH_PATTERN",i,param[5]);
		else
			send_lang(thread,language,howtosend,snick,nick,"O_SPEW_RHOST_FOUND_NOTHING_BY_SEARCHING_WITH_PATTERN",param[5]);
		
		/* mysql_free_result_myr(); - rhostlist */
		mysql_free_result_myr(rhostlist.res);
	}else{
	/*
	 *  Error Messages
	 */
		if(!strcor(param[5],"*"))	send_lang(thread,language,howtosend,snick,nick,"WRONG_INPUT_PATTERN_INPUT",param[5]);
		else 						send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O SPEW Gline function for the O SPEW Command.
 * 
 * The O SPEW Gline Command allows an admin to search for Glines<br/>
 *   which match the Glinepattern
 *  
 * Syntax is: /msg O SPEW GLINE *gline*
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_spew_gline(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL gline;
	char pattern[BUFSIZE];
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&gline,sizeof(gline));
	bzero(&pattern,sizeof(pattern));

	/*
	 * Spew Gline Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"spew gline") && strcor(param[5],"*")){
		dyncpy (pattern,param[5]);
		strrepc(pattern,'*','%');

		gline.res	 = doSelectMySQL(thread,"SELECT `type`,`mask`,`addedby` FROM `orion_core_lines` WHERE `type`='G' AND `mask` LIKE '%s'",mysql_quote(thread,pattern));
		gline.fields = mysql_num_rows(gline.res);
		
		for(i=0;i<gline.fields;i++){
			gline.row = mysql_fetch_row(gline.res);
			
			send_lang(thread,language,howtosend,snick,nick,"O_SPEW_GLINE_LIST_FOUND",i+1,gline.row[0],gline.row[1],gline.row[2]);
		}
		mysql_free_result_myr(gline.res); //free MySQL result

		send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
		//strrepc(param[4],'%','*');
		if(i != 0)	send_lang(thread,language,howtosend,snick,nick,"O_SPEW_GLINE_FOUND_X_MATCHING_GLINES_BY_SEARCHING_WITH_PATTERN",i,param[5]);
		else		send_lang(thread,language,howtosend,snick,nick,"O_SPEW_GLINE_FOUND_NOTHING_BY_SEARCHING_WITH_PATTERN",param[5]);
	}else{
	/*
	 *  Error Messages
	 */
		if(!strcor(param[5],"*"))	send_lang(thread,language,howtosend,snick,nick,"WRONG_INPUT_PATTERN_INPUT",param[5]);
		else 						send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O SPEW Ident function for the O SPEW Command.
 * 
 * The O SPEW Ident Command allows an admin to search for Idents<br/>
 *   which match the Identpattern
 *  
 * Syntax is: /msg O SPEW IDENT *ident*
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_spew_ident(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL rnames;
	char pattern[BUFSIZE];
	int i=0;

	/* Initialise Variables to (NULL) */
	bzero(&rnames,sizeof(rnames));
	bzero(&pattern,sizeof(pattern));

	/*
	 * Spew Ident Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"spew ident") && strcor(param[5],"*")){
		dyncpy (pattern,param[5]);
		strrepc(pattern,'*','%');

		rnames.res	  = doSelectMySQL(thread,"SELECT count( ident ) as countreal, `ident` FROM `orion_core_condata` WHERE `ident` LIKE '%s' GROUP BY `ident` ORDER BY countreal ASC LIMIT 80",mysql_quote(thread,pattern));
		rnames.fields = mysql_num_rows(rnames.res);

		if(rnames.fields>0){
			for(i=0;i<rnames.fields;i++){
				rnames.row = mysql_fetch_row(rnames.res);

				send_lang(thread,language,howtosend,snick,nick,"O_SPEW_IDENT_LISTING_IDENTS_MATCHING_THE_IDENTPATTERN",i+1,rnames.row[1],stoi(rnames.row[0]));
			}
		}

		mysql_free_result_myr(rnames.res); //free MySQL result

		//strrepc(param[4],'%','*');
		if(i != 0)	send_lang(thread,language,howtosend,snick,nick,"O_SPEW_IDENT_FOUND_X_MATCHING_IDENTS_BY_SEARCHING_WITH_PATTERN",i,param[5]);
		else		send_lang(thread,language,howtosend,snick,nick,"O_SPEW_IDENT_FOUND_NOTHING_BY_SEARCHING_WITH_PATTERN",param[5]);
	}else{
	/*
	 *  Error Messages
	 */
		if(!strcor(param[5],"*"))	send_lang(thread,language,howtosend,snick,nick,"WRONG_INPUT_PATTERN_INPUT",param[5]);
		else 						send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O SPEW Vhost function for the O SPEW Command.
 * 
 * The O SPEW Vhost Command allows an admin to search for Vhosts<br/>
 *   which match the Vhostpattern
 *  
 * Syntax is: /msg O SPEW VHOST *vhost*
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_spew_vhost(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL vhostlist;
	char pattern[BUFSIZE];
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&vhostlist,sizeof(vhostlist));
	bzero(&pattern,sizeof(pattern));

	/*
	 * Spew Vhost Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"spew vhost") && strcor(param[5],"*")){
		dyncpy (pattern,param[5]);
		strrepc(pattern,'*','%');

		vhostlist.res    = doSelectMySQL(thread,"SELECT `nick`,`rhost`,`vhost` FROM `orion_core_condata` WHERE `vhost` LIKE '%s' ORDER BY `nick` ASC LIMIT 100",mysql_quote(thread,pattern));
		vhostlist.fields = mysql_num_rows(vhostlist.res);

		for(i=0;i<vhostlist.fields;i++){
			vhostlist.row = mysql_fetch_row(vhostlist.res);
			
			send_lang(thread,language,howtosend,snick,nick,"O_SPEW_VHOST_LIST_VHOSTS_MATCHING_THE_VHOSTPATTERN",i+1,vhostlist.row[0],vhostlist.row[1],vhostlist.row[2]);
		}
		//strrepc(param[4],'%','*');
		if(i != 0)
			send_lang(thread,language,howtosend,snick,nick,"O_SPEW_VHOST_FOUND_X_MATCHING_BY_SEARCHING_WITH_PATTERN",i,param[5]);
		else
			send_lang(thread,language,howtosend,snick,nick,"O_SPEW_VHOST_FOUND_NOTHING_BY_SEARCHING_WITH_PATTERN",param[5]);

		mysql_free_result_myr(vhostlist.res);
	}else{
	/*
	 *  Error Messages
	 */
		if(!strcor(param[5],"*"))	send_lang(thread,language,howtosend,snick,nick,"WRONG_INPUT_PATTERN_INPUT",param[5]);
		else 						send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O SPEW Chan function for the O SPEW Command.
 * 
 * The O SPEW Chan Command allows an admin to search for Channels<br/>
 *   which match the Chanpattern
 *  
 * Syntax is: /msg O SPEW CHAN *chan*
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_spew_chan(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL channels;
	char pattern[BUFSIZE];
	int i=0;

	/* Initialise Variables to (NULL) */
	bzero(&channels,sizeof(channels));
	bzero(&pattern,sizeof(pattern));

	/*
	 * Spew Chan Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"spew chan") && strcor(param[5],"*")){
		dyncpy (pattern,param[5]);
		strrepc(pattern,'*','%');
		
		channels.res    = doSelectMySQL(thread,"SELECT `channel` FROM `orion_cs_core` WHERE `channel` LIKE '%s' ORDER BY `channel` ASC LIMIT 80",mysql_quote(thread,pattern));
		channels.fields = mysql_num_rows(channels.res);

		if(channels.fields>0){
			for(i=0;i<channels.fields;i++){
				channels.row = mysql_fetch_row(channels.res);
				
				send_lang(thread,language,howtosend,snick,nick,"O_SPEW_CHAN_LIST_MATCHING_CHANNELS",i+1,channels.row[0]);
			}
		}

		//strrepc(param[4],'%','*');
		if(i != 0){
			send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
			send_lang(thread,language,howtosend,snick,nick,"O_SPEW_CHAN_FOUND_X_MATCHING_CHANNEL_BY_SEARCHING_WITH_PATTERN",i,param[5]);
		}else send_lang(thread,language,howtosend,snick,nick,"O_SPEW_CHAN_FOUND_NOTHING_BY_SEARCHING_FOR_A_CHANNEL",param[5]);

		mysql_free_result_myr(channels.res); //free MySQL result
	}else{
	/*
	 *  Error Messages
	 */
		if(!strcor(param[5],"*"))	send_lang(thread,language,howtosend,snick,nick,"WRONG_INPUT_PATTERN_INPUT",param[5]);
		else 						send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the O SPEW Command (/msg O HELP SPEW)
 * 
 * When the user tries to get help for this command, using /msg O HELP SPEW,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP SPEW
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_spew_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Spew Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help spew")) {
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_SPEW");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O SPEW Vhost Help function for the O SPEW Command (/msg O HELP SPEW VHOST)
 * 
 * When the user tries to get help for this command, using /msg O HELP SPEW VHOST,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP SPEW VHOST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_spew_vhost_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Spew Vhost Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help spew vhost")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"spew vhost"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_SPEW_VHOST");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O SPEW Rhost Help function for the O SPEW Command (/msg O HELP SPEW RHOST)
 * 
 * When the user tries to get help for this command, using /msg O HELP SPEW RHOST,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP SPEW RHOST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_spew_rhost_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Spew Rhost Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help spew rhost")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"spew rhost"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_SPEW_RHOST");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O SPEW Email Help function for the O SPEW Command (/msg O HELP SPEW EMAIL)
 * 
 * When the user tries to get help for this command, using /msg O HELP SPEW EMAIL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP SPEW EMAIL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_spew_email_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Spew Email Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help spew email")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"spew email"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_SPEW_EMAIL");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O SPEW Ident Help function for the O SPEW Command (/msg O HELP SPEW IDENT)
 * 
 * When the user tries to get help for this command, using /msg O HELP SPEW IDENT,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP SPEW IDENT
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_spew_ident_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Spew Ident Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help spew ident")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"spew ident"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_SPEW_IDENT");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O SPEW Chan Help function for the O SPEW Command (/msg O HELP SPEW CHAN)
 * 
 * When the user tries to get help for this command, using /msg O HELP SPEW CHAN,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP SPEW CHAN
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_spew_chan_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Spew Chan Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help spew chan")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"spew chan"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_SPEW_CHAN");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O SPEW Gline Help function for the O SPEW Command (/msg O HELP SPEW GLINE)
 * 
 * When the user tries to get help for this command, using /msg O HELP SPEW GLINE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP SPEW GLINE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_spew_gline_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Spew Gline Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help spew gline")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"spew gline"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_SPEW_GLINE");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief O Help List Command - SPEW part (/msg O help)
 * 
 * When the user tries to get general O help, using /msg O help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the SPEW command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_spew_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_O_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Spew (Help listing)
	 */
	if(Users_O_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"O_SHOWCOMMANDS_SPEW",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"spew"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
