/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 204                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-03 20:33:44 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file orion_modinfo.c
 *   @brief This is the Module file for Orion MODINFO Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the Orion MODINFO command.
 *
 *   The Orion MODINFO Command allows an admin to dump<br/>
 *   some useful information about a specific modul.
 *
 *   Syntax is: /msg Orion MODINFO < service > < modul >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "orion"
#define MODUL "modinfo"
#define MODUL_FILE "orion_modinfo"
#define MODUL_VERSION "1.0"

MOD_BOOL orion_modinfo(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_modinfo_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_modinfo_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"modinfo","modinfo");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,OPERONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"modinfo",orion_modinfo,1000,INFINIT,5,5);
	addCommand(thread,&Module,"help modinfo",orion_modinfo_help,1000,INFINIT,4,4);
	addCommand(thread,&Module,"help",orion_modinfo_help_listing,1000,INFINIT,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the Orion MODINFO Command.
 * 
 * The Orion MODINFO Command allows an admin to dump<br/>
 *   some useful information about a specific modul.
 *
 * Syntax is: /msg Orion MODINFO < service > < modul >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL orion_modinfo(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	ModCo *module= NULL;
	CMD *command = NULL;
	int i = 1;
	int b = 0;
	/*
	 * Modinfo Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"modinfo")){
		module = findModule(thread,param[4],param[5]);
		
		for(module = findModule(thread,param[4],param[5]); module; module = findModule_next(thread,param[4],param[5],module)){
			if(module!=NULL){
				do_privmsg(thread,m->nick,nick,"Info for modul: %s.",module->modulename);
				do_privmsg(thread,m->nick,nick,"--------------------");
				do_privmsg(thread,m->nick,nick,"Service: %s",module->servicetag);
				do_privmsg(thread,m->nick,nick,"Modul: %s",module->modulename);
				do_privmsg(thread,m->nick,nick,"-");
				do_privmsg(thread,m->nick,nick,"Author: %s",module->Author);
				do_privmsg(thread,m->nick,nick,"Version: %s",module->Version);
				do_privmsg(thread,m->nick,nick,"Options: %d",module->options);
				do_privmsg(thread,m->nick,nick,"Access: %d",module->access);
				do_privmsg(thread,m->nick,nick,"Type: %d",module->type);
				do_privmsg(thread,m->nick,nick,"--------------------");
		
				do_privmsg(thread,m->nick,nick,"----- COMMANDS -----");
				do_privmsg(thread,m->nick,nick,"********************");
		
				i = 1;
				for (command = module->commands; command!=NULL; command = command->next) {
					do_privmsg(thread,snick,nick,"%d Command: %s",i,command->call);
					do_privmsg(thread,snick,nick,"Root Level: %d",command->root_level);
					do_privmsg(thread,snick,nick,"User Level: %d",command->user_level);
					do_privmsg(thread,snick,nick,"Low Param: %d",command->low_param);
					do_privmsg(thread,snick,nick,"High Param: %d",command->high_param);
					do_privmsg(thread,m->nick,nick,"--------------------");
					i++;
				}
				do_privmsg(thread,snick,nick,"End of module information.");
				b++;
			}
		}
		if(b>0)
			do_privmsg(thread,snick,nick,"Modul not loaded.");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,1,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the Orion MODINFO Command (/msg Orion HELP MODINFO)
 * 
 * When the user tries to get help for this command, using /msg Orion HELP MODINFO,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Orion HELP MODINFO
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL orion_modinfo_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Modinfo Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help modinfo")){
		send_lang(thread,language,1,snick,nick,"REQUIRED_LEVEL",ModuleFile_RootCommandLevel(thread,MODUL_FILE,"modinfo"));
		send_lang(thread,language,1,snick,nick,"ORION_HELP_MODINFO",snick);
	}else send_lang(thread,language,1,snick,nick,"ORION_NO_HELP_AVAIL");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief Orion Help List Command - MODINFO part (/msg Orion help)
 * 
 * When the user tries to get general Orion help, using /msg Orion help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the MODINFO command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL orion_modinfo_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_ORION_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Modinfo (Help listing)
	 */
	if(Users_ORION_Level>=command_level)
		send_lang(thread,language,1,snick,nick,"ORION_HELP_CMD_MODINFO",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"modinfo"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
