/*  ************************************************************************************************
 *
 *  THIS PART WAS TAKEN FROM "Anope"!
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Based on the original code of Epona by Lara.
 *  Based on the original code of Services by Andy Church. 
 *  Based on the original code of Anope by Daniel Engel. 
 *
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 194                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-16 18:33:19 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file encrypt.c
 *   @brief This is the Module file for Encrypting Password's. (Main File)
 *
 *   It contains all functions which are neccessary to create and check
 *   MD5/SHA1 hashed strings.
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "encrypt.h"

Encryption encryption;

/******************************************************************************/
void encmodule_encrypt(int (*func)
                        (const char *src, size_t len, char *dest, size_t size))
{
    encryption.encrypt = func;
}

void encmodule_encrypt_in_place(int (*func) (char *buf, size_t size))
{
    encryption.encrypt_in_place = func;
}

void encmodule_encrypt_check_len(int (*func) (size_t passlen, size_t bufsize))
{
    encryption.encrypt_check_len = func;
}

void encmodule_decrypt(int (*func) (const char *src, char *dest, size_t size))
{
    encryption.decrypt = func;
}

void encmodule_check_password(int (*func)
                               (const char *plaintext,
                                const char *password))
{
    encryption.check_password = func;
}

void encmodule_check_encryption(char* (*func) (void))
{
    encryption.check_encryption = func;
}

/******************************************************************************/


/** 
 * Encrypt string `src' of length `len', placing the result in buffer
 * `dest' of size `size'.  Returns 0 on success, -1 on error.
 **/
int orion_encrypt(const char *src, size_t len, char *dest, size_t size)
{
    if (encryption.encrypt) {
        return encryption.encrypt(src, len, dest, size);
    }
    return -1;
}

/**
 * Encrypt null-terminated string stored in buffer `buf' of size `size',
 * placing the result in the same buffer.  Returns 0 on success, -1 on
 * error.
 **/
int orion_encrypt_in_place(char *buf, size_t size)
{
    if (encryption.encrypt_in_place) {
        return encryption.encrypt_in_place(buf, size);
    }
    return -1;
}

/** 
 * Check whether the result of encrypting a password of length `passlen'
 * will fit in a buffer of size `bufsize'.  Returns 0 if the encrypted
 * password would fit in the buffer, otherwise returns the maximum length
 * password that would fit (this value will be smaller than `passlen').
 * If the result of encrypting even a 1-byte password would exceed the
 * specified buffer size, generates a fatal error.
 **/
int orion_encrypt_check_len(size_t passlen, size_t bufsize)
{
    if (encryption.encrypt_check_len) {
        return encryption.encrypt_check_len(passlen, bufsize);
    }
    return -1;
}

/**
 * Decrypt encrypted string `src' into buffer `dest' of length `len'.
 * Returns 1 (not 0) on success, 0 if the encryption algorithm does not
 * allow decryption, and -1 if another failure occurred (e.g. destination
 * buffer too small).
 **/
int orion_decrypt(const char *src, char *dest, size_t size)
{
    if (encryption.decrypt) {
        return encryption.decrypt(src, dest, size);
    }
    return -1;
}

/**
 * Check an input password `plaintext' against a stored, encrypted password
 * `password'.  Return value is:
 *   1 if the password matches
 *   0 if the password does not match
 *   -1 if an error occurred while checking
 **/
int orion_check_password(const char *plaintext, const char *password)
{
    if (encryption.check_password) {
        return encryption.check_password(plaintext, password);
    }
    return -1;
}

/**
 * Check what encryption we are using
 * Return value is:
 *   sha1 if we use sha1 encryption
 *   md5 if we use md5 encryption
 *   plain if we use only binary
 **/
char* orion_check_encryption(void)
{
    if (encryption.check_encryption) {
        return encryption.check_encryption();
    }
    return NULL;
}

/*************************************************************************/

/**
 * Simple function to convert binary data to hex.
 * Taken from Anope who took it from hybrid-ircd ( http://ircd-hybrid.com/ )
 */
void binary_to_hex(unsigned char *bin, char *hex, size_t length)
{
    static const char trans[] = "0123456789ABCDEF";
    size_t i;

    for (i = 0; i < length; i++) {
        hex[i << 1] = trans[bin[i] >> 4];
        hex[(i << 1) + 1] = trans[bin[i] & 0xf];
    }

    hex[i << 1] = '\0';
}
/* EOF */
