/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 207                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-05 12:32:05 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file mode.c
 *   @brief This is the Module file for MODE (Server Side) Command.
 *
 *   It contains all functions which are neccessary to process and use 
 *   the MODE command.
 *
 *   The MODE File is used to process MODES sent from Servers
 *   and to Servers. (Protocol-Process-File -> PPF)
 *   (Mode functions for received modes, modes to send and as well usermodes.)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"

/**
 * @brief do_mode:  Sends out a MODE (depending on the Loaded Protocol) - using MODE
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick that sends the MODE Information
 * @param target The Channel to recive the Modes
 * @param date The Channels creation date
 * @param buf The Modebuffer
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_mode(Threads *thread, char *nick, char *target, time_t date, const char *buf, ...){
	PROTOCOL_BOOL analyzer = CMD_ERROR;
	
	/* Bits & Param Initialisation */
	char receive[BUFSIZE];
	char *param[ARRAY_SIZE];
	char *paramsend[ARRAY_SIZE];
	/* Modes Initialisation */
	char modes[35];
	char selection[(MAX_MODE_PARAMS_IRCD*MAX_NICK_LEN+MAX_MODE_PARAMS_IRCD*3)*2];
		
	deepSQL MySQL;
	UserChanModes ucmodes[MAX_MODE_PARAMS_IRCD];
	
	size_t ucmodes_count = 0;
	size_t i             = 0;
	size_t nr_modes      = 0;
	bool type            = 1;
	size_t nr            = 0;
	int userparam_nr     = 0;
	size_t splicer		 = 0;
	size_t send		     = 0;
	
	bzero(&receive,sizeof(receive));
	bzero(param,sizeof(param));
	
	va_list    va;
	va_start  (va, buf);
	vsnprintf (receive, sizeof(receive)-1, buf, va);
	va_end    (va);
	
	/* First if */
	
	if(nick && target && buf){
		//modebuf = +ntkl-Omp lolbor 25
		strsplit(receive,param,' ');
		// < :Q MODE #lala +b techadmin.deepirthread->c.chan.net
		bzero(&modes,sizeof(modes));
		
		dynncpy(modes,param[0],sizeof(modes));
		nr_modes=dynlen(modes);
		paramsend[send++]=modes;

		/*printf("param[0] = %s\n",param[0]);
		printf("param[1] = %s\n",param[1]);*/
		
		dynsplice(param,0,0);
		
		/*printf("param[0] = %s\n",param[0]);
		printf("param[1] = %s\n",param[1]);*/
		//printf("newbuf = %s\n",_djoin(' ',param));

		for(i=0;i<nr_modes && modes[i]!='\0';i++){
			
			if(!strcin(modes[i],"ovqahklLfbe+-")) splicer++; /* not sure if it works - seems nice (29.04.2008) */
			
			if(modes[i]=='+'){
				type=1;
				splicer++;
			}
			else if(modes[i]=='-'){
				type=0;
				splicer++;
			}
			
			else if(modes[i]=='o'){
				//dlog(thread,2,"if(IRC.OpMode) (i = %d)\n",i);
				modes[i]=IRC.OpMode;
				if(IRC.p10protocol || IRC.p10new)
					paramsend[send++]=GetUser(param[i-splicer])->uid;
				else{
					//dlog(thread,2,"i-splicer = %d, param[i-splicer] = %s\n",i-splicer,param[i-splicer]);
					paramsend[send++]=param[i-splicer];
					//dlog(thread,2,"paramsend[send++] = %s, send = %d\n",paramsend[send],send);
				}
			}
			
			else if(modes[i]=='v'){
				//dlog(thread,2,"if(IRC.VMode)\n");
				modes[i]=IRC.VMode;
				if(IRC.p10protocol || IRC.p10new)
					paramsend[send++]=GetUser(param[i-splicer])->uid;
				else{
					//dlog(thread,2,"i-splicer = %d, param[i-splicer] = %s\n",i-splicer,param[i-splicer]);
					paramsend[send++]=param[i-splicer];
					//dlog(thread,2,"paramsend[send++] = %s, send = %d\n",paramsend[send],send);
				}
			}
			
			else if(modes[i]=='q'){
				if(IRC.HasOwner){
					//dlog(thread,2,"if(IRC.HasOwner)\n");
					modes[i]=IRC.OwnerMode;
					if(IRC.p10protocol || IRC.p10new){
						paramsend[send++]=GetUser(param[i-splicer])->uid;
					}else{
						//dlog(thread,2,"i-splicer = %d, param[i-splicer] = %s\n",i-splicer,param[i-splicer]);
						paramsend[send++]=param[i-splicer];
						//dlog(thread,2,"paramsend[send++] = %s, send = %d\n",paramsend[send],send);
					}
				}else{
					substr(modes,i);
					//printf("deleted modeNR: %d, modeparam: %d - (mode left = %s)\n",i,i-splicer,modes);
					//printf("complete param before splice = %s\n",_djoin(' ',param));
					//printf("param[0] = %s\n",param[0]);
					//printf("param[1] = %s\n",param[1]);
					//printf("param[2] = %s\n",param[2]);
					//printf("param[3] = %s\n",param[3]);
					dynsplice(param,i-splicer,i-splicer);
					//printf("param[0] = %s\n",param[0]);
					//printf("param[1] = %s\n",param[1]);
					//printf("param[2] = %s\n",param[2]);
					//printf("param[3] = %s\n",param[3]);
					i--;
					//printf("complete param = %s\n",_djoin(' ',param));
				}
			}
			else if(modes[i]=='a'){
				if(IRC.HasProtect){
					//dlog(thread,2,"if(IRC.HasProtect)\n");
					modes[i]=IRC.ProtectMode;
					if(IRC.p10protocol || IRC.p10new){
						paramsend[send++]=GetUser(param[i-splicer])->uid;
					}else{
						//dlog(thread,2,"i-splicer = %d, param[i-splicer] = %s\n",i-splicer,param[i-splicer]);
						paramsend[send++]=param[i-splicer];
						//dlog(thread,2,"paramsend[send++] = %s, send = %d\n",paramsend[send],send);
					}
				}else{
					substr(modes,i);
					//printf("deleted modeNR: %d, modeparam: %d - (mode left = %s)\n",i,i-splicer,modes);
					//printf("complete param before splice = %s\n",_djoin(' ',param));
					//printf("param[0] = %s\n",param[0]);
					//printf("param[1] = %s\n",param[1]);
					//printf("param[2] = %s\n",param[2]);
					//printf("param[3] = %s\n",param[3]);
					dynsplice(param,i-splicer,i-splicer);
					//printf("param[0] = %s\n",param[0]);
					//printf("param[1] = %s\n",param[1]);
					//printf("param[2] = %s\n",param[2]);
					//printf("param[3] = %s\n",param[3]);
					i--;
					//printf("complete param = %s\n",_djoin(' ',param));
				}
			}
			else if(modes[i]=='h'){
				if(IRC.SupportHalfOp){
					//dlog(thread,2,"if(IRC.SupportHalfOp)\n");
					modes[i]=IRC.HOpMode;
					if(IRC.p10protocol || IRC.p10new){
						paramsend[send++]=GetUser(param[i-splicer])->uid;
					}else{
						//dlog(thread,2,"i-splicer = %d, param[i-splicer] = %s\n",i-splicer,param[i-splicer]);
						paramsend[send++]=param[i-splicer];
						//dlog(thread,2,"paramsend[send++] = %s, send = %d\n",paramsend[send],send);
					}
				}else{
					substr(modes,i);
					//printf("deleted modeNR: %d, modeparam: %d\n",i,i-splicer);
					dynsplice(param,i-splicer,i-splicer);
					i--;
				}
			}
			else if(modes[i]=='k'){
				//dlog(thread,2,"if(IRC.KeyCMode)\n");
				modes[i]=IRC.KeyCMode;
				paramsend[send++]=param[i-splicer];
				//printf("added %s as KEY\n",param[i-splicer]);
			}
			else if(modes[i]=='l'){
				//dlog(thread,2,"if(IRC.LimitCMode)\n");
				modes[i]=IRC.LimitCMode;
				paramsend[send++]=param[i-splicer];
				//printf("added %s as LIMIT\n",param[i-splicer]);
			}
			else if(modes[i]=='L'){
				if(IRC.SupportLinkMode){
					//dlog(thread,2,"if(IRC.SupportLinkMode)\n");
					modes[i]=IRC.LinkCMode;
					paramsend[send++]=param[i-splicer];
				}else{
					substr(modes,i);
					if(type)
						dynsplice(param,i-splicer,i-splicer);
					i--;
				}
			}
			else if(modes[i]=='f'){
				if(IRC.SupportFloodMode){
					//dlog(thread,2,"if(IRC.FloodCMode)\n");
					modes[i]=IRC.FloodCMode;
					paramsend[send++]=param[i-splicer];
				}else{
					substr(modes,i);
					if(type)
						dynsplice(param,i-splicer,i-splicer);
					i--;
				}
			}
			else if(modes[i]=='b'){
				if(IRC.SupportBanMode){
					dlog(thread,2,"if(IRC.SupportBanMode)\n");
					/*printf("banmode - %d = %s\n",i-splicer,param[i-splicer]);
					printf("newbuf = %s\n",_djoin(' ',param));
					printf("param[0] = %s\n",param[0]);
					printf("param[1] = %s\n",param[1]);*/
					
					modes[i]=IRC.BanMode;
					paramsend[send++]=param[i-splicer];
				}else{
					substr(modes,i);
					dynsplice(param,i-splicer,i-splicer);
					i--;
				}
			}
			else if(modes[i]=='e'){
				if(IRC.SupportExceptMode){
					//dlog(thread,2,"if(IRC.SupportExceptMode)\n");
					modes[i]=IRC.ExceptMode;
					paramsend[send++]=param[i-splicer];
				}else{
					substr(modes,i);
					dynsplice(param,i-splicer,i-splicer);
					i--;
				}
			}
		}
		
		paramsend[send++]=NULL;
		nr_modes = dynlen(modes);
		i        = 0;
		
		analyzer = orion_WRCtranslate_cmd_mode(thread,nick,target,date,_djoin(' ',paramsend));
		if(analyzer >= CMD_NOT_SUPPORTED)
			return analyzer;
		
		dlog(thread,3,"MODES = %s\n",modes);
								
		//modes erstellen (ohvqbe..)
		if(target[0]=='#' || target[0]=='&'){
			
			/* Updates the Threads Channel Management */
			UpdateChannelInformation3(thread,target,NULL);
			
			if(thread->c.chan.fields && thread->c.chan.datecreate==0){
				thread->c.chan.datecreate=time(NULL);
				domysql(thread,"UPDATE `orion_cs_core` SET `datecreate`='%ld' WHERE `channel`='%s'",thread->c.chan.datecreate,target);
			}
			
			bzero(&selection,sizeof(selection));
			
	/* PREPROCESSING MODES FOR ENHANCING EXECUTION */
			for(i=0;i<nr_modes;i++){
				char current=modes[i];
				dlog(thread,3,"MODE = %c\n",current);
				
				if(current=='+')
					type=1;
				else if(current=='-')
					type=0;
					
				else{
					dlog(thread,3,"+|- decision\n");
					if(type==1){
						dlog(thread,3,"Channel: %s\n",target);
						dlog(thread,3,"param[%d]: %s\n",nr,param[nr]);
						if(strcin(current,IRC.SupportedUChanModes)){
							/* PREPROCESSOR */
							
							dlog(thread,3,"ENTERED PREPROCESSOR\n");
							if(current==IRC.BanMode && IRC.SupportBanMode)
								domysql(thread,"REPLACE `orion_chan_bans` (`ban`,`channel`) VALUES ('%s','%s')",mysql_quote(thread,param[nr]),mysql_quote(thread,target));
							else if(current==IRC.ExceptMode && IRC.SupportExceptMode)
								domysql(thread,"REPLACE `orion_chan_expbans` (`exception`,`channel`) VALUES ('%s','%s')",mysql_quote(thread,param[nr]),mysql_quote(thread,target));
							else if(strcin(current,IRC.SupportedU2ChanModes)){
								if(!multiply_occurance(nr,nr_modes,param)){
									dlog(thread,3,"ADD MODENICK: %s \n",param[nr]);
									if(userparam_nr>0) straddnc(selection,',',sizeof(selection));
									straddnc(selection,toascii(39),sizeof(selection));
									straddns(selection,mysql_quote(thread,param[nr]),sizeof(selection));
									straddnc(selection,toascii(39),sizeof(selection));
									dlog(thread,3,"SELECTION: %s \n",selection);
									userparam_nr++;
								}
							}
							nr++;
							
							/****************/
						}else if(strcin(current,IRC.ParametricChannelModes)){
							//c=sql_chan(thread,bits[2]);
							if(current==IRC.FloodCMode){
								bool flood_mode_valid = orion_WRCtranslate_flood_mode_check(thread,param[nr]);
								
								if(thread->c.chan.fields && strcin(current,thread->c.chan.mode) && flood_mode_valid == true){
									dynncpy(thread->c.chan.flood,param[nr],sizeof(thread->c.chan.flood));
								}
								else if(thread->c.chan.fields && flood_mode_valid == true){
									straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
									dynncpy(thread->c.chan.flood,param[nr],sizeof(thread->c.chan.flood));
								}
								else if(flood_mode_valid == true){
									straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
									dynncpy(thread->c.chan.flood,param[nr],sizeof(thread->c.chan.flood));
									dynncpy(thread->c.chan.name,target,sizeof(thread->c.chan.name));
								}
								//do_rawmode(thread->c.chan.serv,thread->c.chan.name,"+f %s",thread->c.chan.flood);
							}else if(current==IRC.KeyCMode){
								if(thread->c.chan.fields && strcin(current,thread->c.chan.mode)){
									dynncpy(thread->c.chan.key,param[nr],sizeof(thread->c.chan.key));
								}
								else if(thread->c.chan.fields){
									straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
									dynncpy(thread->c.chan.key,param[nr],sizeof(thread->c.chan.key));
								}
								else{
									straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
									dynncpy(thread->c.chan.key,param[nr],sizeof(thread->c.chan.key));
									dynncpy(thread->c.chan.name,target,sizeof(thread->c.chan.name));
								}
								//do_rawmode(thread->c.chan.serv,thread->c.chan.name,"+k %s",thread->c.chan.key);
								//printf("MODES (MODE_NEW.C - 178) = %s\n",thread->c.chan.mode);
								//printf("KEY (MODE_NEW.C - 179) = %s\n",thread->c.chan.key);
							}else if(current==IRC.LinkCMode){
								if(thread->c.chan.fields && strcin(current,thread->c.chan.mode)){
									dynncpy(thread->c.chan.link,param[nr],sizeof(thread->c.chan.link));
								}
								else if(thread->c.chan.fields){
									straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
									dynncpy(thread->c.chan.link,param[nr],sizeof(thread->c.chan.link));
								}
								else{
									straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
									dynncpy(thread->c.chan.link,param[nr],sizeof(thread->c.chan.link));
									dynncpy(thread->c.chan.name,target,sizeof(thread->c.chan.name));
								}
								do_rawmode(thread,thread->c.chan.serv,thread->c.chan.name,thread->c.chan.datecreate,"+%c %s",IRC.LinkCMode,thread->c.chan.link);
							}else if(current==IRC.LimitCMode){
								if(thread->c.chan.fields && strcin(current,thread->c.chan.mode)){
									thread->c.chan.limit=stol(param[nr]);
								}
								else if(thread->c.chan.fields){
									straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
									thread->c.chan.limit=stol(param[nr]);
								}
								else{
									straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
									thread->c.chan.limit=stol(param[nr]);
									dynncpy(thread->c.chan.name,target,sizeof(thread->c.chan.name));
								}
								//do_rawmode(thread->c.chan.serv,thread->c.chan.name,"+l %ld",thread->c.chan.limit);
							}else{
								if(thread->c.chan.fields && !strcin(current,thread->c.chan.mode)){
									straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
								}
							}
							nr++;
                        }else{
							if(thread->c.chan.fields && !strcin(current,thread->c.chan.mode)){
								dlog(thread,4,"channel mode\n");
								straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
							}
						}
					}else if(type==0){
						if(strcin(current,IRC.SupportedUChanModes)){
						
							/* PREPROCESSOR */
							
							if(current==IRC.BanMode && IRC.SupportBanMode)
								domysql(thread,"DELETE FROM `orion_chan_bans` WHERE `channel`='%s' AND `ban`='%s'",mysql_quote(thread,target),mysql_quote(thread,param[nr]));
							else if(current==IRC.ExceptMode && IRC.SupportExceptMode)
								domysql(thread,"DELETE FROM `orion_chan_expbans` WHERE `channel`='%s' AND `exception`='%s'",mysql_quote(thread,target),mysql_quote(thread,param[nr]));					
							else if(strcin(current,IRC.SupportedU2ChanModes)){
								if(!multiply_occurance(nr,nr_modes,param)){
									dlog(thread,4,"(type==0)ADD MODENICK: %s \n",param[nr]);
									if(userparam_nr>0) straddnc(selection,',',sizeof(selection));
									straddnc(selection,toascii(39),sizeof(selection));
									straddns(selection,mysql_quote(thread,param[nr]),sizeof(selection));
									straddnc(selection,toascii(39),sizeof(selection));
									dlog(thread,4,"SELECTION: %s \n",selection);
									userparam_nr++;	
								}
							}
							nr++;	
							
							/****************/
							
						}else if(strcin(current,IRC.ParametricChannelModes)){
							if(thread->c.chan.fields && strcin(current,thread->c.chan.mode)){
								strdelc(thread->c.chan.mode,current);
								if(current==IRC.FloodCMode){
									//do_rawmode(thread->c.chan.serv,thread->c.chan.name,"-f %s",thread->c.chan.flood);
									bzero(&thread->c.chan.flood,sizeof(thread->c.chan.flood));
									
								}else if(current==IRC.KeyCMode){
									//do_rawmode(thread->c.chan.serv,thread->c.chan.name,"-k %s",thread->c.chan.key);
									bzero(&thread->c.chan.key,sizeof(thread->c.chan.key));
									
								}else if(current==IRC.LinkCMode){
									//do_rawmode(thread->c.chan.serv,thread->c.chan.name,"-L %s",thread->c.chan.link);
									bzero(&thread->c.chan.link,sizeof(thread->c.chan.link));
									
								}else if(current==IRC.LimitCMode){
									thread->c.chan.limit=-1;
								}else{
									if(thread->c.chan.fields && strcin(current,thread->c.chan.mode)){
										strdelc(thread->c.chan.mode,current);
									}
								}
							}
						}else{
							if(thread->c.chan.fields && strcin(current,thread->c.chan.mode)){
								strdelc(thread->c.chan.mode,current);
							}
						}
					}
				}
			}
			sort(thread->c.chan.mode);
			domysql(thread,"UPDATE `orion_cs_core` SET `mode`='%s' ,`flood`='%s' ,`limit`='%ld' ,`link`='%s' ,`key`='%s' WHERE `channel`='%s' AND `serv`='%s'"
				,mysql_quote(thread,thread->c.chan.mode),mysql_quote(thread,thread->c.chan.flood),thread->c.chan.limit,mysql_quote(thread,thread->c.chan.link),mysql_quote(thread,thread->c.chan.key),mysql_quote(thread,thread->c.chan.name),mysql_quote(thread,thread->c.chan.serv));
	
			i = 0;
			if(selection[0]!='\0'){
				selection[strlen(selection)]='\0';
				dlog(thread,3,"Selection: %s\n",selection);
				
				MySQL.res = doSelectMySQL(thread,
					"SELECT "
					"`orion_chan_users`.`nick`,`orion_ns_aliases`.`display`,`orion_chan_users`.`modes`,`orion_ns_aliases`.`active`,`orion_cs_access`.`level`,`orion_ns_core`.`qlevel`,`orion_ns_core`.`olevel`,`orion_ns_core`.`llevel`,`orion_ns_core`.`slevel`,`orion_ns_core`.`rights`,`orion_chan_users`.`faketype` "
					"FROM `orion_chan_users` "
						"LEFT OUTER JOIN `orion_ns_aliases` "
							"LEFT OUTER JOIN `orion_ns_core` "
								"LEFT OUTER JOIN `orion_cs_access` "
								"ON (`orion_ns_core`.`authnick`=`orion_cs_access`.`authnick` AND `orion_cs_access`.`channel`='%s' AND `orion_ns_core`.`authnick` IS NOT NULL) "
							"ON (`orion_ns_aliases`.`display`=`orion_ns_core`.`authnick` AND `orion_ns_aliases`.`display` IS NOT NULL) "
						"ON (`orion_chan_users`.`nick` = `orion_ns_aliases`.`nick` AND `orion_ns_aliases`.`active`=1) "
					"WHERE `orion_chan_users`.`nick` IN (%s) AND `orion_chan_users`.`channel` = '%s' "
					"ORDER BY `orion_chan_users`.`nick` ASC"
				,mysql_quote(thread,target),selection,mysql_quote(thread,target));
				
				MySQL.fields = mysql_num_rows(MySQL.res);
				
				for(i=0;i<MySQL.fields;i++){
					MySQL.row = mysql_fetch_row(MySQL.res);
					
					dynncpy(ucmodes[ucmodes_count].nick,MySQL.row[0],sizeof(ucmodes[ucmodes_count].nick));
					dynncpy(ucmodes[ucmodes_count].authnick,MySQL.row[1],sizeof(ucmodes[ucmodes_count].authnick));
					dynncpy(ucmodes[ucmodes_count].mode,MySQL.row[2],sizeof(ucmodes[ucmodes_count].mode));
					ucmodes[ucmodes_count].active=stoi(MySQL.row[3]);
					dynncpy(ucmodes[ucmodes_count].access,MySQL.row[4],sizeof(ucmodes[ucmodes_count].access));
					ucmodes[ucmodes_count].qlevel=stoi(MySQL.row[5]);
					ucmodes[ucmodes_count].olevel=stoi(MySQL.row[6]);
					ucmodes[ucmodes_count].llevel=stoi(MySQL.row[7]);
					ucmodes[ucmodes_count].slevel=stoi(MySQL.row[8]);
					ucmodes[ucmodes_count].rights=stoi(MySQL.row[9]);
					ucmodes[ucmodes_count].faketype=stoi(MySQL.row[10]);
					ucmodes_count++;
				}
				mysql_free_result_myr(MySQL.res);
			}
			dlog(thread,4,"END OF PREPROCESSING\n");
		/* END OF PREPROCESSING */
		
		/* START WITH EXECUTING MODES */
			nr=0;
			for(i=0;i<nr_modes;i++){
				char current=modes[i];
				
				if(current=='+')
					type=1;
	
				else if(current=='-')
					type=0;
				
				else{
					if(type==1){
						if(strcin(current,IRC.SupportedU2ChanModes)){
							
							//dlog(thread,2,"CURRENT = %c, NR = %d\n",current,nr);
							//dlog(thread,2,"param[nr] = %s\n",param[nr]);
							
							int user=getModeNick(ucmodes,ucmodes_count,param[nr]);
							//dlog(thread,2,"USER: %d, MODES: %s \n",user,ucmodes[user].mode);
							if(user==-1){
								dlog(thread,0,"exiting because of an invalid user in do_mode(), using getModeNick()\n");
								dlog(thread,0,"%s\n",_djoin(' ',param));
								//continue;
								_orion_exit();
							}
							
							if(ucmodes[user].faketype>0)
								continue;
								
							Users *conuser = GetUser(ucmodes[user].nick);
							if(conuser && getUline(thread,conuser->server))
								continue;
							
							dlog(thread,4,"CURRENT = %c, NR = %d\n",current,nr);
							dlog(thread,4,"param[nr] = %s\n",param[nr]);
							dlog(thread,4,"USER: %d, MODES: %s \n",user,ucmodes[user].mode);
							
							if(streqnc(thread->c.chan.serv,"chanserv") && stoi(ucmodes[user].access)>IDENTIFIED_NO_ACCESS)
								domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%d' WHERE `channel`='%s' AND `serv`='ChanServ'",time(NULL),mysql_quote(thread,target));
							else if(streqnc(thread->c.chan.serv,"q") && strcor(ucmodes[user].access,"ahmnoptv"))
								domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%d' WHERE `channel`='%s' AND `serv`='Q'",time(NULL),mysql_quote(thread,target));			
	
							if(current==IRC.OpMode && !strcin(IRC.OpMode,ucmodes[user].mode)){
								dlog(thread,4," 1.) ADD O!!, user: %d \n",user);
								if(thread->c.chan.fields && strcor("b",thread->c.chan.flags) && streqnc(thread->c.chan.serv,"q")){
									if(!strcor("o",ucmodes[user].access) && !streqnc(nick,Q->nick))
										do_raw2mode(thread,Q->nick,target,"-ho",param[nr],thread->c.chan.datecreate);
										
									else{
										straddnc(ucmodes[user].mode,IRC.OpMode,sizeof(ucmodes[user].mode));
										dlog(thread,4," ADD O!!, user: %d \n",user);
									}
								}
								else if(thread->c.chan.fields && thread->c.chan.settings&CS_SECUREOPS_BIT && streqnc(thread->c.chan.serv,"chanserv")){
									if(stoi(ucmodes[user].access)<thread->c.chan.levels.opdeop)
										do_raw2mode(thread,cs->nick,target,"-ho",param[nr],thread->c.chan.datecreate);
										
									else
										straddnc(ucmodes[user].mode,IRC.OpMode,sizeof(ucmodes[user].mode));	
								}/* AUTODEOP LEVEL CHECK*/
								else if(thread->c.chan.settings&CS_SECUREOPS_BIT && thread->c.chan.fields && stoi(ucmodes[user].access)<=thread->c.chan.levels.autodeop && streqnc(thread->c.chan.serv,"chanserv")){
									do_raw2mode(thread,cs->nick,target,"-oa",param[nr],thread->c.chan.datecreate);
									//printf("jo warum auch immer autodeop = %d - i am level %d\n",thread->c.chan.levels.autodeop,stoi(ucmodes[user].access));
									
								}else
									straddnc(ucmodes[user].mode,IRC.OpMode,sizeof(ucmodes[user].mode));
							}
							
							else if(IRC.SupportHalfOp && current==IRC.HOpMode && !strcin(IRC.HOpMode,ucmodes[user].mode))
								straddnc(ucmodes[user].mode,IRC.HOpMode,sizeof(ucmodes[user].mode));
								
							else if(current==IRC.VMode && !strcin(IRC.VMode,ucmodes[user].mode))
								straddnc(ucmodes[user].mode,IRC.VMode,sizeof(ucmodes[user].mode));
								
							else if(IRC.HasOwner && current==IRC.OwnerMode && !strcin(IRC.OwnerMode,ucmodes[user].mode)){
								
								if(thread->c.chan.fields && thread->c.chan.settings&CS_SECUREOPS_BIT && streqnc(thread->c.chan.serv,"chanserv")){
									if(stoi(ucmodes[user].access)<thread->c.chan.levels.opdeop)
										do_raw2mode(thread,cs->nick,target,"-q",param[nr],thread->c.chan.datecreate);
									else
										straddnc(ucmodes[user].mode,IRC.OwnerMode,sizeof(ucmodes[user].mode));
								}
								else
									straddnc(ucmodes[user].mode,IRC.OwnerMode,sizeof(ucmodes[user].mode));
							}
							else if(IRC.HasProtect && current==IRC.ProtectMode && !strcin(IRC.ProtectMode,ucmodes[user].mode)){
								
								if(thread->c.chan.fields && thread->c.chan.settings&CS_SECUREOPS_BIT && streqnc(thread->c.chan.serv,"chanserv")){
									if(stoi(ucmodes[user].access)<thread->c.chan.levels.protect)
										do_raw2mode(thread,cs->nick,target,"-a",param[nr],thread->c.chan.datecreate);
									else
										straddnc(ucmodes[user].mode,IRC.ProtectMode,sizeof(ucmodes[user].mode));
								}
								else
									straddnc(ucmodes[user].mode,IRC.ProtectMode,sizeof(ucmodes[user].mode));
							}
							
							nr++;
						}else if(strcin(current,IRC.ChannelModes)){
							/* PART OF PREPROCESSING */
							nr++;
						}else{
							/* PART OF PREPROCESSING */
						}
					}else if(type==0){
						if(strcin(current,IRC.SupportedU2ChanModes)){
							
							int user=getModeNick(ucmodes,ucmodes_count,param[nr]);
							
							if(user==-1){
								dlog(thread,0,"exiting because of an invalid user in do_mode(), using getModeNick()");
								dlog(thread,0,_djoin(' ',param));
								//orion_exit();
								continue;
							}
							
							Users *conuser = GetUser(ucmodes[user].nick);
							if(conuser && getUline(thread,conuser->server))
								continue;
							
							if(current==IRC.OpMode){
								
								if(ucmodes[user].faketype>0){
									do_raw2mode(thread,ucmodes[user].nick,target,"+o",ucmodes[user].nick,thread->c.chan.datecreate);
								}else if(thread->c.chan.fields && strcor("p",thread->c.chan.flags) && streqnc(thread->c.chan.serv,"q")){
									if(strcor("o",ucmodes[user].access))
										do_raw2mode(thread,Q->nick,target,"+o",param[nr],thread->c.chan.datecreate);
										
									else if(strcor("o",thread->c.chan.flags))
										do_raw2mode(thread,Q->nick,target,"+o",param[nr],thread->c.chan.datecreate);
									
									else
										strdelc(ucmodes[user].mode,IRC.OpMode);
										
								}else if(thread->c.chan.fields && thread->c.chan.settings&CS_SECUREOPS_BIT && streqnc(thread->c.chan.serv,"chanserv")){
									if(stoi(ucmodes[user].access)>=thread->c.chan.levels.opdeop)
										do_raw2mode(thread,cs->nick,target,"+o",param[nr],thread->c.chan.datecreate);
										
									else
										strdelc(ucmodes[user].mode,IRC.OpMode);
								}
								else
									strdelc(ucmodes[user].mode,IRC.OpMode);
							}
							else if(IRC.SupportHalfOp && current==IRC.HOpMode)
								strdelc(ucmodes[user].mode,IRC.HOpMode);
								
							else if(current==IRC.VMode)
								strdelc(ucmodes[user].mode,IRC.VMode);
								
							else if(IRC.HasOwner && current==IRC.OwnerMode)
								strdelc(ucmodes[user].mode,IRC.OwnerMode);
								
							else if(IRC.HasProtect && current==IRC.ProtectMode)
								strdelc(ucmodes[user].mode,IRC.ProtectMode);
								
							nr++;
						}else{
							/* PART OF PREPROCESSING */
						}
					}
				}
			}
			
			/* COMMIT THE NEW USERMODES */
			for(i=0;i<ucmodes_count;i++){
				dlog(thread,4,"UPDATE MODES!\n");
				sort(ucmodes[i].mode);
				dlog(thread,4,"MODES: %s, NICK: %s \n",ucmodes[i].mode,ucmodes[i].nick);
				if(ucmodes[i].nick[0]!='\0')
					domysql(thread,"UPDATE `orion_chan_users` SET `modes`='%s' WHERE `nick`='%s' AND `channel`='%s'",mysql_quote(thread,ucmodes[i].mode),mysql_quote(thread,ucmodes[i].nick),mysql_quote(thread,target));
				else
					domysql(thread,"INSERT INTO `orion_chan_users` (`modes`,`nick`,`channel`) VALUES ('%s','%s','%s') ON DUPLICATE KEY UPDATE `modes`=VALUES(`modes`)",mysql_quote(thread,ucmodes[i].mode),mysql_quote(thread,ucmodes[i].nick),mysql_quote(thread,target));
			}
		}
	}
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief do_svs2mode:  Sends out a SVSMODE (depending on the Loaded Protocol) - using SVS2MODE/SVSMODE
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick that sends the SVSMODE Information
 * @param target The Nick to recive the Modes
 * @param modes The Modes which we like to set to the target
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_svs2mode(Threads *thread, char *nick, char *target, const char *modes, ...){
	PROTOCOL_BOOL analyzer;
	
	if(orion_WRCtranslate_valid_nick(thread, target) == false)
		return CMD_ERROR;
		
	if(target == NULL || modes == NULL)
	    return CMD_ERROR;
	    
	if(target[0] != '#' && target[0] != '&'){
		if(GetUser(target)==NULL && GetUserByUID(target)==NULL)
	    	return CMD_ERROR;
	}
	
	/* Deklarate Variables */
	va_list va;
	char tmp[BUFFER+1];
	
	/* Initialise Variables to (NULL) */	
	bzero(&tmp,sizeof(tmp));

	/* Join reason string */
	va_start	(va,  modes);
	vsnprintf	(tmp, sizeof(tmp)-1, modes, va);
	va_end		(va);
	
	analyzer = orion_WRCtranslate_cmd_svs2mode(thread,nick,target,tmp);
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief do_svsmode:  Sends out a MODE (depending on the Loaded Protocol) - using SVS2MODE/SVSMODE
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick that sends the SVSMODE Information
 * @param target The Nick to recive the Modes
 * @param modes The Modes which we like to set to the target
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_svsmode(Threads *thread, char *nick, char *target, const char *modes, ...){
	PROTOCOL_BOOL analyzer;
	
	if(orion_WRCtranslate_valid_nick(thread, target) == false)
		return CMD_ERROR;
	
	if(target == NULL || modes == NULL)
	    return CMD_ERROR;

	if(target[0] != '#' && target[0] != '&'){
		if(GetUser(target)==NULL && GetUserByUID(target)==NULL)
	    	return CMD_ERROR;
	}
	
	/* Deklarate Variables */
	va_list va;
	char tmp[BUFFER+1];
	
	/* Initialise Variables to (NULL) */	
	bzero(&tmp,sizeof(tmp));

	/* Join reason string */
	va_start	(va,  modes);
	vsnprintf	(tmp, sizeof(tmp)-1, modes, va);
	va_end		(va);
	
	analyzer = orion_WRCtranslate_cmd_svs2mode(thread,nick,target,tmp);
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief svsmode: Processes the X-MODE for the Database
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick that sends the SVSMODE/SVS2MODE Information
 * @param target The Nick to recive the Modes
 * @param modes The Modes which should be set to target nick
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL svsmode(Threads *thread, char *nick, char *target, char *modes){

	if(target == NULL || modes == NULL)
	    return CMD_ERROR;

	char addmodes[70]; //Modes for Usermode add
	char delmodes[70]; //Modes for Usermode del
	char allmode[80]; //DelModes and AddModes
	
    // < :O SVSMODE shock -w
	//printf("addmodes = %s\n",modes);
	if(target[0]!='#' && target[0] != '&'){

		if(GetUser(target)==NULL && GetUserByUID(target)==NULL)
	    	return CMD_ERROR;
	    	
		allmode[0]='\0'; //set allmode NULL
		addmodes[0]='\0'; //set addmodes NULL
		delmodes[0]='\0'; //set delmodes NULL
    	
		char *mode=currmodes(thread,target);
		//printf("currmodes = %s\n",mode);
		bool type=1;
		size_t i=0;
		size_t a=0, b=0;
		for(i=0;i<dynlen(modes);i++){
			char current=modes[i];
			if(current=='+'){type=1;}
			else if(current=='-'){type=0;}
			else if(strcin(current,IRC.SupportedUserModes)){
				if(type==0 && strcin(current,mode)){
					delmodes[a]=current;
					delmodes[a+1]='\0';
					a++;
				}
				if(type==1 && !strcin(current,mode)){
					addmodes[b]=current;
					addmodes[b+1]='\0';
					b++;
				}
			}
		}
		dynncpy(allmode,mode,sizeof(allmode)); //copy sql saved modes to allmode
		strncat(allmode,addmodes,strlen(addmodes)); //add addmodes to allmode
		for(i=0;delmodes[i];i++){
			strdelc(allmode,delmodes[i]); //delete delmodes[i] from allmode
		}
		
		//printf("allmode = %s\n",allmode);
		
		if(noop!=1){
			/* 	Update our Cache	*/
			if(streqnc(thread->c.Cdata.nick,target)){
				bzero(&thread->c.Cdata.modes,sizeof(thread->c.Cdata.modes));
				dynncpy(thread->c.Cdata.modes,allmode,sizeof(thread->c.Cdata.modes));
			}else if(streqnc(thread->sql.Cdata.nick,target)){
				bzero(&thread->sql.Cdata.modes,sizeof(thread->sql.Cdata.modes));
				dynncpy(thread->sql.Cdata.modes,allmode,sizeof(thread->sql.Cdata.modes));
			}
		/**			***			**/
		
			//Last Bug Found: 6.9.2005 13:11:20 UTC
			
			// > :shock MODE shock :+owghaACeW
			// < :O SVSMODE shock -xowghCeW
			// < :O SVSMODE O -
			// < :O SVSMODE O -
			// < :O SVSMODE O -
			domysql(thread,"UPDATE `orion_core_condata` SET `modes`='%s' WHERE `nick`='%s'",mysql_quote(thread,allmode),mysql_quote(thread,target));
		}
		else if(noop==1){
			bzero(&thread->c,sizeof(thread->c));
			do_kill(thread,m->servername,target,"your olines are nooped.");
		}
	}
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */

/**
 * @brief on_mode:  Receives a MODE / SVSMODE (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick that sent the MODE Information
 * @param target The Channel which for the Modes were set
 * @param date The Channels creation date
 * @param buf The Modebuffer
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_mode(Threads *thread, char *nick, char *target, long date, char* buf){
	/* Bits & Param Initialisation */
	char receive[BUFSIZE];
	char sender[MAX_NICK_LEN];
	char modes[35];
	char positive[70]; //Modes for Usermode add
	char negative[70]; //Modes for Usermode del
	char allmode[70]; //Allmodes
	char selection[(MAX_MODE_PARAMS_IRCD*MAX_NICK_LEN+MAX_MODE_PARAMS_IRCD*3)*2];
	
	char *param[ARRAY_SIZE];
	/* Modes Initialisation */
	
	size_t i             = 0;
	size_t nr_modes      = 0;
	bool type            = 1;
	size_t nr            = 1;
	size_t userparam_nr  = 0;
	size_t ucmodes_count = 0;
	//modes erstellen (ohvqbe..)
	
	bzero(&positive,sizeof(positive)); //Modes for Usermode add
	bzero(&negative,sizeof(negative)); //Modes for Usermode del
	bzero(&allmode,sizeof(allmode));   //Allmodes
	bzero(&selection,sizeof(selection));
	bzero(&receive,sizeof(receive));
	bzero(&modes,sizeof(modes));
	
	deepSQL MySQL;
	
	UserChanModes ucmodes[MAX_MODE_PARAMS_IRCD];
	
	bzero(&MySQL,sizeof(MySQL));
	bzero(&ucmodes,sizeof(ucmodes));
	
	//Flush Inserts - Since on serverlink or not the NICK connect Event comes before SJOIN!
	//MySQL_FlushInserts(thread,SQL_BUFFER_CONDATA,SQL_BUFFER_SJOIN);
	
	if(target[0]=='#' || target[0]=='&'){
		
		dlog(thread,3,"BUF = %s\n",buf);
		
		dynncpy(receive,buf,sizeof(receive));
		strsplit(receive,param,' ');
		dynncpy(modes,param[0],sizeof(modes));
		
		nr_modes=strlen(modes);
		//c = sql_chan_data(thread,target,3,COUNTUSERS,WITH_LEVELS,WITH_BOTSERV);
		/* Updates the Threads Channel Management */
		UpdateChannelInformation3(thread,target,NULL);
		
		if(thread->c.chan.bot[0]=='\0' || (~thread->c.chan.botserv.set&BS_SET_SYMBIOSIS_BIT && thread->c.chan.bot[0]!='\0')){
			if(streqnc(thread->c.chan.serv,"chanserv"))
				dynncpy(sender,cs->nick,sizeof(sender));
			else if(streqnc(thread->c.chan.serv,"q"))
				dynncpy(sender,Q->nick,sizeof(sender));
		}else if(thread->c.chan.botserv.set&BS_SET_SYMBIOSIS_BIT && thread->c.chan.bot[0]!='\0'){
			dynncpy(sender,thread->c.chan.bot,sizeof(sender));
		}
		
	/* PREPROCESSING MODES FOR ENHANCING EXECUTION */
		for(i=0;i<nr_modes;i++){
			char current=modes[i];
			if(current=='+'){
				type=1;
				
				if(thread->c.chan.name[0]=='\0'){
					domysql(thread,"INSERT IGNORE INTO `orion_cs_core` (`datecreate`,`channel`) VALUES ('%ld','%s')",date ? date : time(NULL),mysql_quote(thread,target));
					thread->c.chan.datecreate=date;
					dynncpy(thread->c.chan.name,target,sizeof(thread->c.chan.name));
				}else if((thread->c.chan.datecreate==0 || date<thread->c.chan.datecreate) && date>0){
					domysql(thread,"UPDATE `orion_cs_core` SET `datecreate`='%ld' WHERE `channel` = '%s'",date,mysql_quote(thread,target));
					thread->c.chan.datecreate=date;
				}
				
			}
			else if(current=='-')
				type=0;
			else{
				if(type==1){

					if(strcin(current,IRC.SupportedUChanModes)){
						/* PREPROCESSOR */
						
						if(current==IRC.BanMode && IRC.SupportBanMode)
							domysql(thread,"REPLACE `orion_chan_bans` (`ban`,`channel`) VALUES ('%s','%s')",mysql_quote(thread,param[nr]),mysql_quote(thread,target));
						else if(current==IRC.ExceptMode && IRC.SupportExceptMode)
							domysql(thread,"REPLACE `orion_chan_expbans` (`exception`,`channel`) VALUES ('%s','%s')",mysql_quote(thread,param[nr]),mysql_quote(thread,target));
						else if(strcin(current,IRC.SupportedU2ChanModes)){
							if(!multiply_occurance(nr,nr_modes,param)){
								dlog(thread,4,"NR = %d\n",nr);
								dlog(thread,4,"ADD MODENICK: %s \n",param[nr]);
								if(userparam_nr>0) straddnc(selection,',',sizeof(selection));
								straddnc(selection,toascii(39),sizeof(selection));
								straddns(selection,mysql_quote(thread,param[nr]),sizeof(selection));
								straddnc(selection,toascii(39),sizeof(selection));
								dlog(thread,4,"SELECTION: %s \n",selection);
								userparam_nr++;
							}
						}
						nr++;
						
						/****************/

					}else if(strcin(current,IRC.ParametricChannelModes)){

						if(current==IRC.FloodCMode && IRC.SupportFloodMode){
							bool flood_mode_valid = orion_WRCtranslate_flood_mode_check(thread,param[nr]);
							
							if(flood_mode_valid == true){
								/* ChanServ - POSITIVE Mlock Processing */
								if(!streqnc(nick,sender) && strcin(current,thread->c.chan.mlock_positive) && streqnc(thread->c.chan.serv,"chanserv") && !streqnc(param[nr],thread->c.chan.mlock_flood))
									do_rawmode(thread,sender,target,thread->c.chan.datecreate,"+%c %s",IRC.FloodCMode,thread->c.chan.mlock_flood);
								
								/* ChanServ - NEGATIVE Mlock Processing */
								else if(!streqnc(nick,sender) && strcin(current,thread->c.chan.mlock_negative) && streqnc(thread->c.chan.serv,"chanserv"))
									do_rawmode(thread,sender,target,thread->c.chan.datecreate,"-%c %s",IRC.FloodCMode,param[nr]);
							
							
								/* Common processing */
								else if(thread->c.chan.fields && strcin(current,thread->c.chan.mode)){
									dynncpy(thread->c.chan.flood,param[nr],sizeof(thread->c.chan.flood));
								}
								else if(thread->c.chan.fields){
									straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
									dynncpy(thread->c.chan.flood,param[nr],sizeof(thread->c.chan.flood));
								}
								else{
									straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
									dynncpy(thread->c.chan.flood,param[nr],sizeof(thread->c.chan.flood));
									dynncpy(thread->c.chan.name,target,sizeof(thread->c.chan.name));
								}
							}
								
						}else if(current==IRC.KeyCMode){
							/* Q -  Processing */
    						if(thread->c.chan.fields && !streqnc(nick,sender) && strcor("k",thread->c.chan.flags) && !streqnc(param[nr],thread->c.chan.key)){
								if(thread->c.chan.key[0]!='\0')
									do_rawmode(thread,sender,target,thread->c.chan.datecreate,"+%c %s",IRC.KeyCMode,thread->c.chan.key);
								else{
									if(!strcin(current,thread->c.chan.mode))
										straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
										
									dynncpy(thread->c.chan.key,param[nr],sizeof(thread->c.chan.key));
								}
							}

							/* ChanServ - POSITIVE Mlock Processing */
							else if(thread->c.chan.fields && !streqnc(nick,sender) && strcin(current,thread->c.chan.mlock_positive) && streqnc(thread->c.chan.serv,"chanserv") && !streqnc(param[nr],thread->c.chan.mlock_key))
								do_rawmode(thread,sender,target,thread->c.chan.datecreate,"+%c %s",IRC.KeyCMode,thread->c.chan.mlock_key);
							
							/* ChanServ - NEGATIVE Mlock Processing */
							else if(thread->c.chan.fields && !streqnc(nick,sender) && strcin(current,thread->c.chan.mlock_negative) && streqnc(thread->c.chan.serv,"chanserv"))
								do_rawmode(thread,sender,target,thread->c.chan.datecreate,"-%c %s",IRC.KeyCMode,param[nr]);
								
								
							/* Common processing */
							else if(thread->c.chan.fields && strcin(current,thread->c.chan.mode)){
								dynncpy(thread->c.chan.key,param[nr],sizeof(thread->c.chan.key));
							}
							else if(thread->c.chan.fields){
								straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
								dynncpy(thread->c.chan.key,param[nr],sizeof(thread->c.chan.key));
							}
							else{
								straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
								dynncpy(thread->c.chan.key,param[nr],sizeof(thread->c.chan.key));
								dynncpy(thread->c.chan.name,target,sizeof(thread->c.chan.name));
							}
							
						}else if(current==IRC.LinkCMode && IRC.SupportLinkMode){
							/* ChanServ - POSITIVE Mlock Processing */
							if(!streqnc(nick,sender) && strcin(current,thread->c.chan.mlock_positive) && streqnc(thread->c.chan.serv,"chanserv") && !streqnc(param[nr],thread->c.chan.mlock_redirect))
								do_rawmode(thread,sender,target,thread->c.chan.datecreate,"+%c %s",IRC.LinkCMode,thread->c.chan.mlock_redirect);
							
							/* ChanServ - NEGATIVE Mlock Processing */
							else if(!streqnc(nick,sender) && strcin(current,thread->c.chan.mlock_negative) && streqnc(thread->c.chan.serv,"chanserv"))
								do_rawmode(thread,sender,target,thread->c.chan.datecreate,"-%c %s",IRC.LinkCMode,param[nr]);
							
							
							/* Common processing */
							else if(thread->c.chan.fields && strcin(current,thread->c.chan.mode)){
								dynncpy(thread->c.chan.link,param[nr],sizeof(thread->c.chan.link));
							}
							else if(thread->c.chan.fields){
								straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
								dynncpy(thread->c.chan.link,param[nr],sizeof(thread->c.chan.link));
							}
							else{
								straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
								dynncpy(thread->c.chan.link,param[nr],sizeof(thread->c.chan.link));
								dynncpy(thread->c.chan.name,target,sizeof(thread->c.chan.name));
							}
							
						}else if(current==IRC.LimitCMode){
							/* Q -  Processing (LIMIT) */
							if(thread->c.chan.fields && streqnc(thread->c.chan.serv,"q") && !streqnc(param[nr],sender) && strcor("l",thread->c.chan.flags) && stol(param[nr])!=thread->c.chan.limit){
								if(thread->c.chan.limit!=0) //If channel-limit != 0 - There is already a limit - SO WE FORCE IT!
									do_rawmode(thread,sender,target,thread->c.chan.datecreate,"+%c %ld",IRC.LimitCMode,thread->c.chan.limit);
								else{ //If channel-limit == 0 - There is no limit atm - SO WE ADD THIS LIMIT! (We force it later)
									if(!strcin(current,thread->c.chan.mode))
										straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
										
									thread->c.chan.limit=stol(param[nr]); //Add as new channel-limit
								}
							}
							
							/* Q -  Processing (AUTOLIMIT) */
							else if(thread->c.chan.fields && streqnc(thread->c.chan.serv,"q") && strcor("c",thread->c.chan.flags) && stol(param[nr])!=thread->c.chan.alimit+thread->c.chan.countusers){
								long limit=thread->c.chan.countusers;
								limit+=thread->c.chan.alimit;
								do_rawmode(thread,sender,target,thread->c.chan.datecreate,"+%c %ld",IRC.LimitCMode,limit);
							}
							
							/* ChanServ - POSITIVE Mlock Processing */
							else if(!streqnc(nick,sender) && strcin(current,thread->c.chan.mlock_positive) && streqnc(thread->c.chan.serv,"chanserv") && stol(param[nr])!=thread->c.chan.mlock_limit)
								do_rawmode(thread,sender,target,thread->c.chan.datecreate,"+%c %ld",IRC.LimitCMode,thread->c.chan.mlock_limit);
								
							/* ChanServ - NEGATIVE Mlock Processing */
							else if(!streqnc(nick,sender) && strcin(current,thread->c.chan.mlock_negative) && streqnc(thread->c.chan.serv,"chanserv"))
								do_rawmode(thread,sender,target,thread->c.chan.datecreate,"-%c %ld",IRC.LimitCMode,stol(param[nr]));


							/* Common processing */
							else if(thread->c.chan.fields && strcin(current,thread->c.chan.mode)){
								thread->c.chan.limit=stol(param[nr]);
							}
							else if(thread->c.chan.fields){
								straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
								thread->c.chan.limit=stol(param[nr]);
							}
							else{
								straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
								thread->c.chan.limit=stol(param[nr]);
								dynncpy(thread->c.chan.name,target,sizeof(thread->c.chan.name));
							}
							
						}else{
							if(thread->c.chan.fields){
								straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
							}
						}
						nr++;
					}else{
						if(thread->c.chan.fields && !strcin(current,thread->c.chan.mode)){
							dlog(thread,4,"channel mode\n");
							straddnc(thread->c.chan.mode,current,sizeof(thread->c.chan.mode));
							
							if(strcin(current,thread->c.chan.mlock_negative)){
								straddnc(negative,current,sizeof(negative));
								//printf("negative added = %c\n",current);
							}
						}
					}
				}else if(type==0){
					if(strcin(current,IRC.SupportedUChanModes)){
						
						/* PREPROCESSOR */
						
						if(current==IRC.BanMode && IRC.SupportBanMode){
							domysql(thread,"DELETE FROM `orion_chan_bans` WHERE `channel`='%s' AND `ban`='%s'",mysql_quote(thread,target),mysql_quote(thread,param[nr]));
							dynsplice(param,nr,nr);
							nr--;
						}
						else if(current==IRC.ExceptMode && IRC.SupportExceptMode){
							domysql(thread,"DELETE FROM `orion_chan_expbans` WHERE `channel`='%s' AND `exception`='%s'",mysql_quote(thread,target),mysql_quote(thread,param[nr]));					
							dynsplice(param,nr,nr);
							nr--;
						}
						else if(strcin(current,IRC.SupportedU2ChanModes)){
							if(!multiply_occurance(nr,nr_modes,param)){
								dlog(thread,4,"(type==0)ADD MODENICK: %s \n",param[nr]);
								if(userparam_nr>0) straddnc(selection,',',sizeof(selection));
								straddnc(selection,toascii(39),sizeof(selection));
								straddns(selection,mysql_quote(thread,param[nr]),sizeof(selection));
								straddnc(selection,toascii(39),sizeof(selection));
								dlog(thread,4,"SELECTION: %s \n",selection);
								userparam_nr++;	
							}
						}
						nr++;
						
						/****************/
						
					}else if(strcin(current,IRC.ParametricChannelModes)){

						if(thread->c.chan.fields && strcin(current,thread->c.chan.mode)){
							
							if(current==IRC.FloodCMode && IRC.SupportFloodMode){
								/* ChanServ - POSITIVE Mlock Processing */
								bzero(&thread->c.chan.flood,sizeof(thread->c.chan.flood));
								if(!streqnc(nick,sender) && strcin(current,thread->c.chan.mlock_positive) && streqnc(thread->c.chan.serv,"chanserv")){
									do_mode(thread,sender,target,thread->c.chan.datecreate,"+%c %s",IRC.FloodCMode,thread->c.chan.mlock_flood);
									dynncpy(thread->c.chan.flood,thread->c.chan.mlock_flood,sizeof(thread->c.chan.flood));
								}
								
								/* Common processing */
								else
									strdelc(thread->c.chan.mode,current);
									
							}else if(current==IRC.KeyCMode){
								/* ChanServ - POSITIVE Mlock Processing */
								if(!streqnc(nick,sender) && strcor("k",thread->c.chan.flags) && streqnc(thread->c.chan.serv,"q"))
									do_mode(thread,sender,target,thread->c.chan.datecreate,"+%c %s",IRC.KeyCMode,thread->c.chan.key);
									
								else if(!streqnc(nick,sender) && strcin(current,thread->c.chan.mlock_positive) && streqnc(thread->c.chan.serv,"chanserv")){
									do_mode(thread,sender,target,thread->c.chan.datecreate,"+%c %s",IRC.KeyCMode,thread->c.chan.mlock_key);
									dynncpy(thread->c.chan.key,thread->c.chan.mlock_key,sizeof(thread->c.chan.key));
								}
								
								/* Common processing */
								else{
									strdelc(thread->c.chan.mode,current);
									bzero(&thread->c.chan.key,sizeof(thread->c.chan.key));
								}
							}
							else if(current==IRC.LinkCMode && IRC.SupportLinkMode){
								/* ChanServ - POSITIVE Mlock Processing */
								bzero(&thread->c.chan.link,sizeof(thread->c.chan.link));
								if(!streqnc(nick,sender) && strcin(current,thread->c.chan.mlock_positive) && streqnc(thread->c.chan.serv,"chanserv")){
									do_mode(thread,sender,target,thread->c.chan.datecreate,"+%c %s",IRC.LinkCMode,thread->c.chan.mlock_redirect);
									dynncpy(thread->c.chan.link,thread->c.chan.mlock_redirect,sizeof(thread->c.chan.mlock_redirect));
								}
								
								/* Common processing */
								else
									strdelc(thread->c.chan.mode,current);
							}	
							else if(current==IRC.LimitCMode){
								/* Q - Processing (LIMIT) FORCE */
								if(!streqnc(nick,sender) && strcor("l",thread->c.chan.flags) && streqnc(thread->c.chan.serv,"q"))
									do_mode(thread,sender,target,thread->c.chan.datecreate,"+%c %ld",IRC.LimitCMode,thread->c.chan.limit);
								
								/* Q - Processing (AUTOLIMIT) FORCE */
								else if(!streqnc(nick,sender) && strcor("c",thread->c.chan.flags) && streqnc(thread->c.chan.serv,"q")){
									long limit=thread->c.chan.countusers;
									limit+=thread->c.chan.alimit;
									do_mode(thread,sender,target,thread->c.chan.datecreate,"+%c %ld",IRC.LimitCMode,limit);
								}
								
								/* ChanServ - POSITIVE Mlock Processing */
								else if(!streqnc(nick,sender) && strcin(current,thread->c.chan.mlock_positive) && streqnc(thread->c.chan.serv,"chanserv")){
									do_mode(thread,sender,target,thread->c.chan.datecreate,"+%c %ld",IRC.LimitCMode,thread->c.chan.mlock_limit);
									thread->c.chan.limit=thread->c.chan.mlock_limit;
								}
								
								/* Common processing */
								else{
									thread->c.chan.limit=0;
									strdelc(thread->c.chan.mode,current);
								}
							}else
								strdelc(thread->c.chan.mode,current);
						}
					}else{
						if(thread->c.chan.fields && strcin(current,thread->c.chan.mode)){
							/* This part is for L (but L is not implemented) */
							/*if(streqnc(thread->c.chan.serv,"l") && !streqnc(nick,L->nick)){
								if(strcin(IRC.InviteCMode,thread->c.chan.flags))
									do_mode(thread,L->nick,target,thread->c.chan.datecreate,"+%c",IRC.InviteCMode);
								else
									strdelc(thread->c.chan.mode,current);
							}
							else*/
								strdelc(thread->c.chan.mode,current);
								
							if(strcin(current,thread->c.chan.mlock_positive)){
								straddnc(positive,current,sizeof(positive));
								//printf("positve added = %c\n",current);
							}
						}
					}
				}
			}
		}
		
		sort(thread->c.chan.mode);
		domysql(thread,"UPDATE `orion_cs_core` SET `mode`='%s' ,`flood`='%s' ,`limit`='%ld' ,`link`='%s' ,`key`='%s' WHERE `channel`='%s' AND `serv`='%s'"
			,mysql_quote(thread,thread->c.chan.mode),mysql_quote(thread,thread->c.chan.flood),thread->c.chan.limit,mysql_quote(thread,thread->c.chan.link),mysql_quote(thread,thread->c.chan.key),mysql_quote(thread,thread->c.chan.name),mysql_quote(thread,thread->c.chan.serv));
			
		if(streqnc(thread->c.chan.serv,"chanserv") && (positive[0]!='\0' || negative[0]!='\0'))
			do_mode(thread,sender,target,thread->c.chan.datecreate,"+%s-%s",positive,negative);

		i = 0;
		if(selection[0]!='\0'){
			selection[strlen(selection)]='\0';
			
			dlog(thread,4,"selection = %s\n",selection);
			
			MySQL.res = doSelectMySQL(thread,
				"SELECT "
				"`orion_chan_users`.`nick`,`orion_ns_aliases`.`display`,`orion_chan_users`.`modes`,`orion_ns_aliases`.`active`,`orion_cs_access`.`level`,`orion_ns_core`.`qlevel`,`orion_ns_core`.`olevel`,`orion_ns_core`.`llevel`,`orion_ns_core`.`slevel`,`orion_ns_core`.`rights`,`orion_chan_users`.`faketype`  "
				"FROM `orion_chan_users` "
					"LEFT OUTER JOIN `orion_ns_aliases` "
						"LEFT OUTER JOIN `orion_ns_core` "
							"LEFT OUTER JOIN `orion_cs_access` "
							"ON (`orion_ns_core`.`authnick`=`orion_cs_access`.`authnick` AND `orion_cs_access`.`channel`='%s' AND `orion_ns_core`.`authnick` IS NOT NULL) "
						"ON (`orion_ns_aliases`.`display`=`orion_ns_core`.`authnick` AND `orion_ns_aliases`.`display` IS NOT NULL) "
					"ON (`orion_chan_users`.`nick` = `orion_ns_aliases`.`nick` AND `orion_ns_aliases`.`active`=1) "
				"WHERE `orion_chan_users`.`nick` IN (%s) AND `orion_chan_users`.`channel` = '%s' ORDER BY `orion_chan_users`.`nick` ASC"
			,mysql_quote(thread,target),selection,mysql_quote(thread,target));
			
			MySQL.fields = mysql_num_rows(MySQL.res);
			
			dlog(thread,4,"MySQL.fields = %d\n",MySQL.fields);
						
			for(i=0;i<MySQL.fields;i++){
				MySQL.row = mysql_fetch_row(MySQL.res);
				dlog(thread,4,"[%d] ADDING NICK = %s\n",i,MySQL.row[0]);
				dynncpy(ucmodes[ucmodes_count].nick,MySQL.row[0],sizeof(ucmodes[ucmodes_count].nick));
				dynncpy(ucmodes[ucmodes_count].authnick,MySQL.row[1],sizeof(ucmodes[ucmodes_count].authnick));
				dynncpy(ucmodes[ucmodes_count].mode,MySQL.row[2],sizeof(ucmodes[ucmodes_count].mode));
				ucmodes[ucmodes_count].active=stoi(MySQL.row[3]);
				dynncpy(ucmodes[ucmodes_count].access,MySQL.row[4],sizeof(ucmodes[ucmodes_count].access));
				ucmodes[ucmodes_count].qlevel=stoi(MySQL.row[5]);
				ucmodes[ucmodes_count].olevel=stoi(MySQL.row[6]);
				ucmodes[ucmodes_count].llevel=stoi(MySQL.row[7]);
				ucmodes[ucmodes_count].slevel=stoi(MySQL.row[8]);
				ucmodes[ucmodes_count].rights=stoi(MySQL.row[9]);
				ucmodes[ucmodes_count].faketype=stoi(MySQL.row[10]);
				ucmodes_count++;
			}
			mysql_free_result_myr(MySQL.res);
			
			dlog(thread,4,"ucmodes_count = %d\n",ucmodes_count);
		}
	/* END OF PREPROCESSING */
	
	/* START WITH EXECUTING MODES */
		nr=1;
		for(i=0;i<nr_modes;i++){
			char current=modes[i];

			if(current=='+')
				type=1;

			else if(current=='-')
				type=0;
			
			else{
				if(type==1){
					
					//dlog(thread,3,"param[%d] = %s (first)\n",nr,param[nr]);
					if(strcin(current,IRC.SupportedU2ChanModes)){
						int user=getModeNick(ucmodes,ucmodes_count,param[nr]);
						if(user==-1){
							dlog(thread,0,"exiting because of an invalid user in on_mode(), using getModeNick()\n");
							dlog(thread,0,"%s - ERROR WAS HERE \"%s\"\n",_djoin(' ',param),param[nr]);
							
							int counter = 0;
							for(counter=0;counter<ucmodes_count;counter++){
								dlog(thread,0,"ucmodes[%d].nick = %s\n",counter,ucmodes[counter].nick);
							}
							continue;
						}
						
						if(ucmodes[user].faketype>0)
							continue;
							
						Users *conuser = GetUser(ucmodes[user].nick);
						if(conuser && getUline(thread,conuser->server))
							continue;
						
						if(streqnc(thread->c.chan.serv,"chanserv")){
							if(!ucmodes[user].active) unknown_access(ucmodes[user].access,sizeof(ucmodes[user].access));
						}
						
						dlog(thread,4,"CURRENT = %c\n",current);
						dlog(thread,4,"param[nr] = %s\n",param[nr]);
						dlog(thread,4,"USER: %d, MODES: %s\n",user,ucmodes[user].mode);


						if(current==IRC.OpMode && !strcin(IRC.OpMode,ucmodes[user].mode)){
							dlog(thread,4," 1.) ADD O!!!!!!, user: %d\n",user);
							if(thread->c.chan.fields && !streqnc(param[nr],sender) && strcor("b",thread->c.chan.flags)){
								if(!strcor("o",ucmodes[user].access))
									do_raw2mode(thread,sender,target,"-ho",param[nr],thread->c.chan.datecreate);
									
								else{
									straddnc(ucmodes[user].mode,IRC.OpMode,sizeof(ucmodes[user].mode));
									dlog(thread,4," ADD O!!!!!!, user: %d\n",user);
								}
							}
							else if(thread->c.chan.fields && thread->c.chan.settings&CS_SECUREOPS_BIT && streqnc(thread->c.chan.serv,"chanserv")){
								if(stoi(ucmodes[user].access)<thread->c.chan.levels.opdeop)
									do_raw2mode(thread,sender,target,"-ho",param[nr],thread->c.chan.datecreate);
									
								else
									straddnc(ucmodes[user].mode,IRC.OpMode,sizeof(ucmodes[user].mode));	
							}/* AUTODEOP LEVEL CHECK*/
							else if(thread->c.chan.settings&CS_SECUREOPS_BIT && thread->c.chan.fields && stoi(ucmodes[user].access)<=thread->c.chan.levels.autodeop && streqnc(thread->c.chan.serv,"chanserv"))
								do_raw2mode(thread,sender,target,"-oa",param[nr],thread->c.chan.datecreate);
								
							else
								straddnc(ucmodes[user].mode,IRC.OpMode,sizeof(ucmodes[user].mode));
						}
						
						else if(IRC.SupportHalfOp && current==IRC.HOpMode && !strcin(IRC.HOpMode,ucmodes[user].mode))
							straddnc(ucmodes[user].mode,IRC.HOpMode,sizeof(ucmodes[user].mode));
							
						else if(current==IRC.VMode && !strcin(IRC.VMode,ucmodes[user].mode))
							straddnc(ucmodes[user].mode,IRC.VMode,sizeof(ucmodes[user].mode));
							
						else if(IRC.HasOwner && current==IRC.OwnerMode && !strcin(IRC.OwnerMode,ucmodes[user].mode)){
							
							if(thread->c.chan.fields && thread->c.chan.settings&CS_SECUREOPS_BIT && streqnc(thread->c.chan.serv,"chanserv")){
								if(stoi(ucmodes[user].access)<thread->c.chan.levels.opdeop)
									do_raw2mode(thread,sender,target,"-q",param[nr],thread->c.chan.datecreate);
								else
									straddnc(ucmodes[user].mode,IRC.OwnerMode,sizeof(ucmodes[user].mode));
							}
							else
								straddnc(ucmodes[user].mode,IRC.OwnerMode,sizeof(ucmodes[user].mode));
						}
						else if(IRC.HasProtect && current==IRC.ProtectMode && !strcin(IRC.ProtectMode,ucmodes[user].mode)){
							
							if(thread->c.chan.fields && thread->c.chan.settings&CS_SECUREOPS_BIT && streqnc(thread->c.chan.serv,"chanserv")){
								if(stoi(ucmodes[user].access)<thread->c.chan.levels.protect)
									do_raw2mode(thread,sender,target,"-a",param[nr],thread->c.chan.datecreate);
								else
									straddnc(ucmodes[user].mode,IRC.ProtectMode,sizeof(ucmodes[user].mode));
							}
							else
								straddnc(ucmodes[user].mode,IRC.ProtectMode,sizeof(ucmodes[user].mode));
						}
						
						nr++;
					}else if(strcin(current,IRC.ParametricChannelModes)){
						/* PART OF PREPROCESSING */
						nr++;
					}else{
						/* PART OF PREPROCESSING */
					}
				}else if(type==0){
					if(strcin(current,IRC.SupportedU2ChanModes)){
						
						int user=getModeNick(ucmodes,ucmodes_count,param[nr]);
						
						if(user==-1){
							dlog(thread,0,"exiting because of an invalid user in on_mode(), using getModeNick()");
							dlog(thread,0,"%s - ERROR WAS HERE \"%s\"\n",_djoin(' ',param),param[nr]);
							
							int counter = 0;
							for(counter=0;counter<ucmodes_count;counter++){
								dlog(thread,0,"ucmodes[%d].nick = %s\n",counter,ucmodes[counter].nick);
							}
							continue;
						}
						
						if(streqnc(thread->c.chan.serv,"chanserv")){
							if(!ucmodes[user].active) unknown_access(ucmodes[user].access,sizeof(ucmodes[user].access));
						}
						
						if(current==IRC.OpMode){
							
							if(streqnc(param[nr],sender)){
								do_raw2mode(thread,sender,target,"+o",sender,thread->c.chan.datecreate);
							}else if(thread->c.chan.fields && strcor("p",thread->c.chan.flags) && streqnc(thread->c.chan.serv,"q")){
								if(strcor("o",ucmodes[user].access))
									do_raw2mode(thread,sender,target,"+o",param[nr],thread->c.chan.datecreate);
									
								else if(strcor("o",thread->c.chan.flags))
									do_raw2mode(thread,sender,target,"+o",param[nr],thread->c.chan.datecreate);
								
								else
									strdelc(ucmodes[user].mode,IRC.OpMode);
									
							}else if(thread->c.chan.fields && thread->c.chan.settings&CS_SECUREOPS_BIT && streqnc(thread->c.chan.serv,"chanserv")){
								if(stoi(ucmodes[user].access)>=thread->c.chan.levels.opdeop)
									do_raw2mode(thread,sender,target,"+o",param[nr],thread->c.chan.datecreate);
									
								else
									strdelc(ucmodes[user].mode,IRC.OpMode);
							}
							else
								strdelc(ucmodes[user].mode,IRC.OpMode);
						}
						else if(IRC.SupportHalfOp && current==IRC.HOpMode)
							strdelc(ucmodes[user].mode,IRC.HOpMode);
							
						else if(current==IRC.VMode)
							strdelc(ucmodes[user].mode,IRC.VMode);
							
						else if(IRC.HasOwner && current==IRC.OwnerMode)
							strdelc(ucmodes[user].mode,IRC.OwnerMode);
							
						else if(IRC.HasProtect && current==IRC.ProtectMode)
							strdelc(ucmodes[user].mode,IRC.ProtectMode);
							
						nr++;
					}else{
						/* PART OF PREPROCESSING */
					}
				}
			}
		}
		
		dlog(thread,4,"ucmodes_count = %d\n",ucmodes_count);
		
		/* COMMIT THE NEW USERMODES */
		for(i=0;i<ucmodes_count;i++){
			dlog(thread,4,"UPDATE MODES!\n");
			sort(ucmodes[i].mode);
			dlog(thread,4,"MODES: %s, NICK: %s \n",ucmodes[i].mode,ucmodes[i].nick);
			if(ucmodes[i].nick[0]!='\0'/* && stoi(mysql_sel_var("SELECT count(1) FROM `orion_core_condata` WHERE `nick`='%s'",mysql_quote(thread,ucmodes[i].nick))) */)
				domysql(thread,"UPDATE `orion_chan_users` SET `modes`='%s' WHERE `nick`='%s' AND `channel`='%s'",mysql_quote(thread,ucmodes[i].mode),mysql_quote(thread,ucmodes[i].nick),mysql_quote(thread,target));
			//else
			//	domysql("INSERT INTO `orion_chan_users` (`modes`,`nick`,`channel`) VALUES ('%s','%s','%s')",mysql_quote(thread,ucmodes[i].mode),mysql_quote(thread,ucmodes[i].nick),mysql_quote(thread,bits[2]));
		}
		
	}else{
		//> :shock MODE shock :-oaACeW	
		dynncpy(modes,buf,sizeof(modes));
		char *mode  = currmodes(thread,target);
		
		bool type=1;
		size_t i=0;
		size_t a=0, b=0;
		for(i=0;i<dynlen(modes);i++){
			char modenow=modes[i];
			if(modenow=='+'){type=1;}
			else if(modenow=='-'){type=0;}
			else if(strcin(modenow,IRC.SupportedUserModes)){
				if(type==0 && strcin(modenow,mode)){
					negative[a]=modenow;
					negative[a+1]='\0';
					a++;
				}
				if(type==1 && !strcin(modenow,mode)){
					positive[b]=modenow;
					positive[b+1]='\0';
					b++;
				}
			}
		}
		dynncpy(allmode,mode,sizeof(allmode)); //copy sql saved modes to allmode
		strncat(allmode,positive,strlen(positive)); //add addmodes to allmode
		
		for(i=0;negative[i];i++)
			strdelc(allmode,negative[i]); //delete delmodes[i] from allmode
			
			
		if(!noop){
        	domysql(thread,"UPDATE `orion_core_condata` SET `modes`='%s' WHERE `nick`='%s'",mysql_quote(thread,allmode),mysql_quote(thread,target));
			/* 	Update our Cache	*/
			if(streqnc(thread->c.Cdata.nick,target)){
				bzero(&thread->c.Cdata.modes,sizeof(thread->c.Cdata.modes));
				dynncpy(thread->c.Cdata.modes,allmode,sizeof(thread->c.Cdata.modes));
			}else if(streqnc(thread->sql.Cdata.nick,target)){
				bzero(&thread->sql.Cdata.modes,sizeof(thread->sql.Cdata.modes));
				dynncpy(thread->sql.Cdata.modes,allmode,sizeof(thread->sql.Cdata.modes));
			}
			/**			***			**/
		}
		else{
			bzero(&thread->c,sizeof(thread->c));
			do_kill(thread,m->servername,target,"your olines are nooped.");
		}
	}
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */
