/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 203                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 23:10:45 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file movedb.c
 *   @brief This is the Module file for moving the Database Information.
 *
 *   It contains all functions which are neccessary to
 *   move the AnopeDB to the orionDB. (converts a few things)
 *   NOTE: ALL MLOCK DATA IS LOST!
 *
 *	 MoveDB File - Moves the Information from the AnopeDB to the orionDB
 */
/*  ************************************************************************************************  */
 
#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"

/* Retain topic even after last person leaves channel */
#define ANOPE_CI_KEEPTOPIC			0x00000001
/* Don't allow non-authorized users to be opped */
#define ANOPE_CI_SECUREOPS			0x00000002
/* Hide channel from ChanServ LIST command */
#define ANOPE_CI_PRIVATE			0x00000004
/* Topic can only be changed by SET TOPIC */
#define ANOPE_CI_TOPICLOCK			0x00000008
/* Those not allowed ops are kickbanned */
#define ANOPE_CI_RESTRICTED			0x00000010
/* Don't allow ChanServ and BotServ commands to do bad things to bigger levels */
#define ANOPE_CI_PEACE  			0x00000020
/* Don't allow any privileges unless a user is IDENTIFY'd with NickServ */
#define ANOPE_CI_SECURE				0x00000040
/* Don't allow the channel to be registered or used */
#define ANOPE_CI_VERBOTEN			0x00000080
/* Channel password is encrypted */
#define ANOPE_CI_ENCRYPTEDPW		0x00000100
/* Channel does not expire */
#define ANOPE_CI_NO_EXPIRE			0x00000200
/* Channel memo limit may not be changed */
#define ANOPE_CI_MEMO_HARDMAX		0x00000400
/* Send notice to channel on use of OP/DEOP */
#define ANOPE_CI_OPNOTICE			0x00000800
/* Stricter control of channel founder status */
#define ANOPE_CI_SECUREFOUNDER 		0x00001000
/* Always sign kicks */
#define ANOPE_CI_SIGNKICK 			0x00002000
/* Sign kicks if level is < than the one defined by the SIGNKICK level */
#define ANOPE_CI_SIGNKICK_LEVEL 	0x00004000
/* Use the xOP lists */
#define ANOPE_CI_XOP 				0x00008000
/* Channel is suspended */
#define ANOPE_CI_SUSPENDED 			0x00010000

/* TEMPORARY - ChanServ is on the channel. */
#define ANOPE_CI_INHABIT 			0x80000000

/* Indices for cmd_access[]: */
#define ANOPE_CA_INVITE				0
#define ANOPE_CA_AKICK				1
#define ANOPE_CA_SET				2	/* but not FOUNDER or PASSWORD */
#define ANOPE_CA_UNBAN				3
#define ANOPE_CA_AUTOOP				4
#define ANOPE_CA_AUTODEOP			5	/* Maximum, not minimum */
#define ANOPE_CA_AUTOVOICE			6
#define ANOPE_CA_OPDEOP				7	/* ChanServ commands OP and DEOP */
#define ANOPE_CA_ACCESS_LIST		8
#define ANOPE_CA_CLEAR				9
#define ANOPE_CA_NOJOIN				10	/* Maximum */
#define ANOPE_CA_ACCESS_CHANGE 		11
#define ANOPE_CA_MEMO				12
#define ANOPE_CA_ASSIGN       		13  /* BotServ ASSIGN command */
#define ANOPE_CA_BADWORDS     		14  /* BotServ BADWORDS command */
#define ANOPE_CA_NOKICK       		15  /* Not kicked by the bot */
#define ANOPE_CA_FANTASIA   		16
#define ANOPE_CA_SAY				17
#define ANOPE_CA_GREET            	18
#define ANOPE_CA_VOICEME			19
#define ANOPE_CA_VOICE				20
#define ANOPE_CA_GETKEY           	21
#define ANOPE_CA_AUTOHALFOP			22
#define ANOPE_CA_AUTOPROTECT		23
#define ANOPE_CA_OPDEOPME         	24
#define ANOPE_CA_HALFOPME         	25
#define ANOPE_CA_HALFOP           	26
#define ANOPE_CA_PROTECTME        	27
#define ANOPE_CA_PROTECT          	28
#define ANOPE_CA_KICKME           	29
#define ANOPE_CA_KICK             	30
#define ANOPE_CA_SIGNKICK			31
#define ANOPE_CA_BANME            	32
#define ANOPE_CA_BAN              	33
#define ANOPE_CA_TOPIC            	34
#define ANOPE_CA_INFO             	35

#define ANOPE_CA_SIZE				36

/* BotServ SET flags */
#define ANOPE_BS_DONTKICKOPS      	0x00000001
#define ANOPE_BS_DONTKICKVOICES   	0x00000002
#define ANOPE_BS_FANTASY          	0x00000004
#define ANOPE_BS_SYMBIOSIS        	0x00000008
#define ANOPE_BS_GREET            	0x00000010
#define ANOPE_BS_NOBOT            	0x00000020

/* BotServ Kickers flags */
#define ANOPE_BS_KICK_BOLDS 		0x80000000
#define ANOPE_BS_KICK_COLORS 		0x40000000
#define ANOPE_BS_KICK_REVERSES		0x20000000
#define ANOPE_BS_KICK_UNDERLINES	0x10000000
#define ANOPE_BS_KICK_BADWORDS		0x08000000
#define ANOPE_BS_KICK_CAPS			0x04000000
#define ANOPE_BS_KICK_FLOOD			0x02000000
#define ANOPE_BS_KICK_REPEAT		0x01000000

/* Indices for TTB (Times To Ban) */
#define ANOPE_TTB_BOLDS 			0
#define ANOPE_TTB_COLORS 			1
#define ANOPE_TTB_REVERSES			2
#define ANOPE_TTB_UNDERLINES  		3
#define ANOPE_TTB_BADWORDS			4
#define ANOPE_TTB_CAPS				5
#define ANOPE_TTB_FLOOD				6
#define ANOPE_TTB_REPEAT			7
#define ANOPE_TTB_SIZE 				8

/* Nickname setting flags: */
#define ANOPE_NI_KILLPROTECT		0x00000001  /* Kill others who take this nick */
#define ANOPE_NI_SECURE				0x00000002  /* Don't recognize unless IDENTIFY'd */
#define ANOPE_NI_MSG            	0x00000004  /* Use PRIVMSGs instead of NOTICEs */
#define ANOPE_NI_MEMO_HARDMAX		0x00000008  /* Don't allow user to change memo limit */
#define ANOPE_NI_MEMO_SIGNON		0x00000010  /* Notify of memos at signon and un-away */
#define ANOPE_NI_MEMO_RECEIVE		0x00000020  /* Notify of new memos when sent */
#define ANOPE_NI_PRIVATE			0x00000040  /* Don't show in LIST to non-servadmins */
#define ANOPE_NI_HIDE_EMAIL			0x00000080  /* Don't show E-mail in INFO */
#define ANOPE_NI_HIDE_MASK			0x00000100  /* Don't show last seen address in INFO */
#define ANOPE_NI_HIDE_QUIT			0x00000200  /* Don't show last quit message in INFO */
#define ANOPE_NI_KILL_QUICK			0x00000400  /* Kill in 20 seconds instead of 60 */
#define ANOPE_NI_KILL_IMMED			0x00000800  /* Kill immediately instead of in 60 sec */
#define ANOPE_NI_SERVICES_OPER 		0x00001000  /* User is a Services operator */
#define ANOPE_NI_SERVICES_ADMIN		0x00002000  /* User is a Services admin */
#define ANOPE_NI_ENCRYPTEDPW		0x00004000  /* Nickname password is encrypted */
#define ANOPE_NI_SERVICES_ROOT  	0x00008000  /* User is a Services root */
#define ANOPE_NI_MEMO_MAIL      	0x00010000  /* User gets email on memo */
#define ANOPE_NI_HIDE_STATUS    	0x00020000  /* Don't show services access status */
#define ANOPE_NI_SUSPENDED			0x00040000  /* Nickname is suspended */
#define ANOPE_NI_AUTOOP				0x00080000  /* Autoop nickname in channels */

#define ANOPE_NS_VERBOTEN			0x0002      /* Nick may not be registered or used */
#define ANOPE_NS_NO_EXPIRE			0x0004      /* Nick never expires */

#define ANOPE_LANG_EN_US              0	/* United States English */
#define ANOPE_LANG_JA_JIS             1	/* Japanese (JIS encoding) */
#define ANOPE_LANG_JA_EUC             2	/* Japanese (EUC encoding) */
#define ANOPE_LANG_JA_SJIS            3	/* Japanese (SJIS encoding) */
#define ANOPE_LANG_ES                 4	/* Spanish */
#define ANOPE_LANG_PT                 5	/* Portugese */
#define ANOPE_LANG_FR                 6	/* French */
#define ANOPE_LANG_TR                 7	/* Turkish */
#define ANOPE_LANG_IT                 8	/* Italian */
#define ANOPE_LANG_DE                 9   	/* German */
#define ANOPE_LANG_CAT               10   	/* Catalan */
#define ANOPE_LANG_GR                11   	/* Greek */
#define ANOPE_LANG_NL                12   	/* Dutch */
#define ANOPE_LANG_RU                13   	/* Russian */
#define ANOPE_LANG_HUN               14   	/* Hungarian */
#define ANOPE_LANG_PL                15   	/* Polish */


#define ANOPE_BI_PRIVATE			0x0001

/**
 * @brief anope_to_orion:  Moves the DatabaseContent from Anope's DB to Orion's
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param blob The BLOB Identifier (NS passwords stored as BLOB @ AnopeDB -> 1, otherwise 0)
 * @return void
 **/
void anope_to_orion(Threads *thread, bool blob){

	/* Main MySQL */
	deepSQL channels;
	int i = 0;
	int maxmemos = 0;
	int tmplevel = 0;
	
	/* Moving ChannelDB */
	char reason[256];
	char botnick[MAX_NICK_LEN];
	char founder[MAX_NICK_LEN];
	char successor[MAX_NICK_LEN];
	char ns_password[ENC_PASSMAX];
	int chansettings = 0;
	int botsettings  = 0;
	int forbid		 = 0;
	
	/* Moving Chanlevels to ChannelDB */
	char currchan[MAX_CHANNELNAME_LEN];
	int cs_set 			= CS_AUTOOP;
	int cs_autodeop 	= CS_AUTODEOP;
	int cs_autohalfop 	= CS_AUTOHALFOP;
	int cs_autoop 		= CS_AUTOOP;
	int cs_autoprotect 	= CS_AUTOPROTECT;
	int cs_autovoice 	= CS_AUTOVOICE;
	int cs_nojoin 		= CS_NOJOIN;
	int cs_signkick 	= CS_SIGNKICK;
	int cs_acclist 		= CS_ACCESS_LIST;
	int cs_accchange	= CS_ACCESS_CHANGE;
	int cs_akick 		= CS_AKICK;
	int cs_ban 			= CS_BAN;
	int cs_banme 		= CS_BANME;
	int cs_cclear 		= CS_CLEAR;
	int cs_getkey 		= CS_GETKEY;
	int cs_halfop 		= CS_HALFOP;
	int cs_halfopme 	= CS_HALFOPME;
	int cs_info 		= CS_INFO;
	int cs_kick 		= CS_KICK;
	int cs_kickme 		= CS_KICKME;
	int cs_invite 		= CS_INVITE;
	int cs_opdeop 		= CS_OPDEOP;
	int cs_opdeopme 	= CS_OPDEOPME;
	int cs_protect 		= CS_PROTECT;
	int cs_protectme 	= CS_PROTECTME;
	int cs_topic 		= CS_TOPIC;
	int cs_unban 		= CS_UNBAN;
	int cs_voice 		= CS_VOICE;
	int cs_voiceme 		= CS_VOICEME;
	int cs_memo 		= CS_MEMO;
	int cs_botassign 	= CS_ASSIGN;
	int cs_badwords 	= CS_BADWORDS;
	int cs_fantasia 	= CS_FANTASIA;
	int cs_greet 		= CS_GREET;
	int cs_nokick 		= CS_NOKICK;
	int cs_say 			= CS_SAY;
	
	/* Moving BotKickers to ChannelDB */
	//char currchan[MAX_CHANNELNAME_LEN]; - Used for ChanLevels and BotKickers
	int bs_bolds_ttb 		= -1;
	int bs_colors_ttb 		= -1;
	int bs_reverses_ttb 	= -1;
	int bs_underlines_ttb 	= -1;
	int bs_badwords_ttb 	= -1;
	int bs_caps_ttb 		= -1;
	int bs_flood_ttb 		= -1;
	int bs_repeat_ttb 		= -1;
	
	/* Moving Vhosts to VhostDB */
	int vhosted = 0;
	char vhost[MAX_HOST_LEN+MAX_IDENT_LEN+2];
	
	/* Moving Access to AccessDB (DEPENDENCY for 2used_blob_pass") */
	int language=0;
	int flags   =0;
	int qlevel  =0;
	int olevel  =0;
	int slevel  =0;
	int llevel  =0;
	int rights  =0;
	
	/* Moving Aliases to AliasesDB */
	int expire = 0;
	//int forbid = 0;
	
	open_logs(thread);
	
	if(copyconfig.MysqlMove[0] == '\0'){
		dlog(thread,0,"You forgot to define \"MysqlMove\", this is REQUIRED in that case!, define it - Config Error\n");
    	_orion_exit();
	}
	
	if(LoadModule(thread,_dynadd("enc_%s",m->MysqlMove)).ModErr == MOD_ERR_OK){
		dlog(thread,0,"  *  ERROR: Loading Encryption Module (\"MysqlMove\" parameter in the Config -> Check the Config!)...\n");
		orion_exit(thread);
	}
	
	mysql_connect(thread);
	
/* Moving Memos from anope_os_exceptions to orion_os_trusts */
	domysql(thread,"INSERT INTO `orion_os_trusts` (`rhost`,`limit`,`description`) SELECT `mask`,`lim`,`reason` FROM `anope_os_exceptions`");
	domysql(thread,"UPDATE `orion_os_trusts` SET `art`='1'");
	
/* Moving BotDB */
	domysql(thread,"INSERT INTO `orion_bs_core` (`botnick`,`botident`,`bothost`,`botrealname`,`private`,`time`) SELECT `nick`,`user`,`host`,`rname`,`flags`,`created` FROM `anope_bs_core`");	
	
/* Moving Access from anope_ns_core to access - DEPENDENCY for using BLOB in DB or not!? */
	if(blob)
		channels.res    = doSelectMySQL(thread,"SELECT `display`,CAST(`pass` AS CHAR),`email`,`icq`,`url`,`flags`,`language`,`greet` FROM `anope_ns_core`");
	else
		channels.res    = doSelectMySQL(thread,"SELECT `display`,`pass`,`email`,`icq`,`url`,`flags`,`language`,`greet` FROM `anope_ns_core`");
	
	channels.fields = mysql_num_rows(channels.res);
	if(channels.fields){
		for(i=0;i<channels.fields;i++){
			channels.row = mysql_fetch_row(channels.res);
			
			language=0;
			flags   =0;
			qlevel  =0;
			olevel  =0;
			slevel  =0;
			llevel  =0;
			rights  =0;
			bzero(&ns_password,sizeof(ns_password));
			
			if(stoi(channels.row[5])&ANOPE_NI_KILLPROTECT) flags |= NS_SET_KILLPROTECT;
			if(stoi(channels.row[5])&ANOPE_NI_KILL_QUICK) flags |= NS_SET_KILL_QUICK;
			if(stoi(channels.row[5])&ANOPE_NI_KILL_IMMED) flags |= NS_SET_KILL_IMMED;
			if(stoi(channels.row[5])&ANOPE_NI_MSG) flags |= NS_SET_MSG;
			if(stoi(channels.row[5])&ANOPE_NI_SECURE) flags |= NS_SET_SECURE;
			if(stoi(channels.row[5])&ANOPE_NI_PRIVATE) flags |= NS_SET_PRIVATE;
			if(stoi(channels.row[5])&ANOPE_NI_HIDE_EMAIL) flags |= NS_SET_HIDE_EMAIL;
			if(stoi(channels.row[5])&ANOPE_NI_HIDE_MASK) flags |= NS_SET_HIDE_MASK;
			if(stoi(channels.row[5])&ANOPE_NI_HIDE_QUIT) flags |= NS_SET_HIDE_QUIT;
			if(stoi(channels.row[5])&ANOPE_NI_HIDE_STATUS) flags |= NS_SET_HIDE_STATUS;
			if(stoi(channels.row[5])&ANOPE_NI_SUSPENDED) flags |= NS_SUSPENDED;
		
			if(stoi(channels.row[5])&ANOPE_NI_SERVICES_ROOT){
				rights = is_services_root;
				qlevel = Q_SRA_LEVEL;
				olevel = O_SRA_LEVEL;
				slevel = S_SRA_LEVEL;
				llevel = 256;
			}else if(stoi(channels.row[5])&ANOPE_NI_SERVICES_ADMIN){
				rights = is_services_admin;
				qlevel = Q_SA_LEVEL;
				olevel = O_SA_LEVEL;
				slevel = S_SA_LEVEL;
				llevel = 254;
			}else if(stoi(channels.row[5])&ANOPE_NI_SERVICES_OPER){
				rights = is_services_oper;
				qlevel = Q_SO_LEVEL;
				olevel = O_SO_LEVEL;
				slevel = S_SO_LEVEL;
				llevel = 254;
			}else{
				rights = is_services_user;
				qlevel = 1;
				olevel = 0;
				slevel = 0;
				llevel = 1;
			}
			
			if(!streqnc(orion_check_encryption(),"none"))
				orion_encrypt(channels.row[1], strlen(channels.row[1]), ns_password, ENC_PASSMAX);
			else
				dynncpy(ns_password,channels.row[1],sizeof(channels.row[1]));

			switch(stoi(channels.row[6])){
				case ANOPE_LANG_EN_US: language = EN;
					break;
				case ANOPE_LANG_JA_JIS: language = EN;
					break;
				case ANOPE_LANG_JA_EUC: language = EN;
					break;
				case ANOPE_LANG_JA_SJIS: language = EN;
					break;
				case ANOPE_LANG_ES: language = EN;
					break;
				case ANOPE_LANG_PT: language = EN;
					break;
				case ANOPE_LANG_FR: language = EN;
					break;
				case ANOPE_LANG_TR: language = EN;
					break;
				case ANOPE_LANG_IT: language = EN;
					break;
				case ANOPE_LANG_DE: language = EN;
					break;
				case ANOPE_LANG_CAT: language = EN;
					break;
				case ANOPE_LANG_GR: language = EN;
					break;
				case ANOPE_LANG_NL: language = EN;
					break;
				case ANOPE_LANG_RU: language = EN;
					break;
				case ANOPE_LANG_HUN: language = EN;
					break;
				case ANOPE_LANG_PL: language = EN;
					break;
			}
			//`display`,`pass`,`email`,`icq`,`url`,`flags`,`language`,`greet`
			domysql(thread,"INSERT INTO `orion_ns_core` (`authnick`,`password`,`email`,`icq`,`url`,`settings`,`language`,`greet`,`rights`,`qlevel`,`olevel`,`slevel`,`llevel`) VALUES ('%s','%s','%s','%s','%s','%d','%d','%s','%d','%d','%d','%d','%d')",
				mysql_quote(thread,channels.row[0]),mysql_quote(thread,ns_password),mysql_quote(thread,channels.row[2]),
				mysql_quote(thread,channels.row[3]),mysql_quote(thread,channels.row[4]),flags,language,mysql_quote(thread,channels.row[7]),
				rights,qlevel,olevel,slevel,llevel
			);
		}
	}
	mysql_free_quotes(thread);
	mysql_free_result_myr(channels.res);
	bzero(&channels,sizeof(channels));

/* Moving Aliases from anope_ns_alias to aliases */
	channels.res    = doSelectMySQL(thread,"SELECT `display`,`nick`,`time_registered`,`last_seen`,`last_usermask`,`last_realname`,`last_quit`,`status` FROM `anope_ns_alias`");
	channels.fields = mysql_num_rows(channels.res);
	if(channels.fields){
		for(i=0;i<channels.fields;i++){
			channels.row = mysql_fetch_row(channels.res);

			expire = 1;
			forbid = 0;

			if(stoi(channels.row[7])&ANOPE_NS_NO_EXPIRE) expire=0;
			if(stoi(channels.row[7])&ANOPE_NS_VERBOTEN) forbid=1;
			
			domysql(thread,"INSERT INTO `orion_ns_aliases` (`display`,`nick`,`time_linked`,`last_used`,`last_usermask`,`last_realname`,`last_quit`,`expire`,`active`,`linked`,`forbid`) "
					"VALUES ('%s','%s','%ld','%ld','%s','%s','%s','%d','0','0','%d')",
					mysql_quote(thread,channels.row[0]),mysql_quote(thread,channels.row[1]),stol(channels.row[2])==0 ? (long)time(NULL) : stol(channels.row[2]),
					stol(channels.row[3])==0 ? (long)time(NULL) : stol(channels.row[3]),mysql_quote(thread,channels.row[4]),mysql_quote(thread,channels.row[5]),
					mysql_quote(thread,channels.row[6]),expire,forbid
					);
		}
	}
	mysql_free_quotes(thread);
	mysql_free_result_myr(channels.res);
	bzero(&channels,sizeof(channels));
	
/* Moving ChannelDB */
	channels.res = doSelectMySQL(thread,"SELECT "
		  "`name`,"
		  "`founder`,"
		  "`successor`,"
		  "`descr`,"
		  "`url`,"
		  "`email`,"
		  "`time_registered`,"
		  "`last_used`,"
		  "`last_topic`,"
		  "`last_topic_setter`,"
		  "`last_topic_time`,"
		  "`flags`,"
		  "`forbidby`,"
		  "`forbidreason`,"
		  "`bantype`,"
		  "`entry_message`,"
		  "`memomax`,"
		  "`botnick`,"
		  
		  "`botflags`,"
		  
		  "`capsmin`,"
		  "`capspercent`,"
		  "`floodlines`,"
		  "`floodsecs`,"
		  "`repeattimes`"
		  
		  " FROM `anope_cs_info`"
	);
		  
	channels.fields = mysql_num_rows(channels.res);
	if(channels.fields){
		for(i=0;i<channels.fields;i++){
			channels.row = mysql_fetch_row(channels.res);
			
			dlog(thread,0,"Moving channel = \"%s\"\n",channels.row[0]);
			
			chansettings = 0;
			botsettings  = 0;
			forbid		 = 0;
			expire		 = 1;
			bzero(&reason,sizeof(reason));
			bzero(&botnick,sizeof(botnick));
			bzero(&founder,sizeof(founder));
			bzero(&successor,sizeof(successor));
			
			maxmemos = stoi(channels.row[16]);
			
			if(stoi(channels.row[11])>0){		
				if(stoi(channels.row[11]) & ANOPE_CI_KEEPTOPIC) chansettings |= CS_KEEPTOPIC_BIT;
				if(stoi(channels.row[11]) & ANOPE_CI_PRIVATE) chansettings |= CS_PRIVATE_BIT;
				if(stoi(channels.row[11]) & ANOPE_CI_TOPICLOCK) chansettings |= CS_TOPICLOCK_BIT;
				if(stoi(channels.row[11]) & ANOPE_CI_RESTRICTED) chansettings |= CS_RESTRICTED_BIT;
				if(stoi(channels.row[11]) & ANOPE_CI_PEACE) chansettings |= CS_PEACE_BIT;
				//if(stoi(channels.row[11]) & ANOPE_CI_SECURE) chansettings |= CS_SECURE_BIT;
				if(stoi(channels.row[11]) & ANOPE_CI_SECUREFOUNDER) chansettings |= CS_SECUREFOUNDER_BIT;
				if(stoi(channels.row[11]) & ANOPE_CI_SECUREOPS) chansettings |= CS_SECUREOPS_BIT;
				if(stoi(channels.row[11]) & ANOPE_CI_VERBOTEN) chansettings |= CS_FORBID_BIT;
				if(stoi(channels.row[11]) & ANOPE_CI_NO_EXPIRE) expire=0;
				if(stoi(channels.row[11]) & ANOPE_CI_MEMO_HARDMAX) maxmemos=m->maxmemos;
				if(stoi(channels.row[11]) & ANOPE_CI_OPNOTICE) chansettings |= CS_OPNOTICE_BIT;

				if(stoi(channels.row[11]) & ANOPE_CI_SIGNKICK_LEVEL){
					chansettings |= CS_SIGNKICK_1_BIT;
					chansettings |= CS_SIGNKICK_2_BIT;
				}else if(stoi(channels.row[11]) & ANOPE_CI_SIGNKICK){
					chansettings |= CS_SIGNKICK_1_BIT;
					chansettings &= ~CS_SIGNKICK_2_BIT;
				}

				if(stoi(channels.row[11]) & ANOPE_CI_XOP) chansettings |= CS_XOP_BIT;
				if(stoi(channels.row[11]) & ANOPE_CI_SUSPENDED) chansettings |= CS_SUSPEND_BIT;
				
				if(stoi(channels.row[11]) & ANOPE_BS_DONTKICKVOICES) botsettings |= BS_SET_DONTKICKVOICES_BIT;
				if(stoi(channels.row[11]) & ANOPE_BS_FANTASY) botsettings |= BS_SET_FANTASY_BIT;
				if(stoi(channels.row[11]) & ANOPE_BS_SYMBIOSIS) botsettings |= BS_SET_SYMBIOSIS_BIT;
				if(stoi(channels.row[11]) & ANOPE_BS_GREET) botsettings |= BS_SET_GREET_BIT;
				if(stoi(channels.row[11]) & ANOPE_BS_NOBOT) chansettings |= BS_SET_NOBOT_BIT;
			}
			
			if(channels.row[13][0]!='\0')
				dynncpy(reason,channels.row[13],sizeof(reason));
				
			if(stoi(channels.row[18])>0){
				if(stoi(channels.row[18]) & ANOPE_BS_KICK_BOLDS) botsettings |= BS_KICK_BOLDS_BIT;
				if(stoi(channels.row[18]) & ANOPE_BS_KICK_COLORS) botsettings |= BS_KICK_COLORS_BIT;
				if(stoi(channels.row[18]) & ANOPE_BS_KICK_REVERSES) botsettings |= BS_KICK_REVERSES_BIT;
				if(stoi(channels.row[18]) & ANOPE_BS_KICK_UNDERLINES) botsettings |= BS_KICK_UNDERLINES_BIT;
				if(stoi(channels.row[18]) & ANOPE_BS_KICK_BADWORDS) botsettings |= BS_KICK_BADWORDS_BIT;
				if(stoi(channels.row[18]) & ANOPE_BS_KICK_CAPS) botsettings |= BS_KICK_CAPS_BIT;
				if(stoi(channels.row[18]) & ANOPE_BS_KICK_FLOOD) botsettings |= BS_KICK_FLOOD_BIT;
				if(stoi(channels.row[18]) & ANOPE_BS_KICK_REPEAT) botsettings |= BS_KICK_REPEAT_BIT;
			}
			
			if(stoi(channels.row[14]) == 0){
				if(chansettings&CS_BANTYPE_1_BIT) chansettings &= ~CS_BANTYPE_1_BIT;
				if(chansettings&CS_BANTYPE_2_BIT) chansettings &= ~CS_BANTYPE_2_BIT;
			}else if(stoi(channels.row[14]) == 1){
				if(~chansettings&CS_BANTYPE_1_BIT) chansettings |= CS_BANTYPE_1_BIT;
				if(chansettings&CS_BANTYPE_2_BIT)  chansettings &= ~CS_BANTYPE_2_BIT;
			}else if(stoi(channels.row[14]) == 2){
				if(chansettings&CS_BANTYPE_1_BIT)  chansettings &= ~CS_BANTYPE_1_BIT;
				if(~chansettings&CS_BANTYPE_2_BIT) chansettings |= CS_BANTYPE_2_BIT;
			}else if(stoi(channels.row[14]) == 3){
				if(~chansettings&CS_BANTYPE_1_BIT) chansettings |= CS_BANTYPE_1_BIT;
				if(~chansettings&CS_BANTYPE_2_BIT) chansettings |= CS_BANTYPE_2_BIT;
			}

			if(channels.row[17][0]=='\0'){
				dynncpy(botnick,"NULL",sizeof(botnick));
			}else
				snprintf(botnick,sizeof(botnick),"%c%s%c",toascii(39),mysql_quote(thread,channels.row[17]),toascii(39));
				
			if(channels.row[1][0]=='\0'){
				dynncpy(founder,"NULL",sizeof(founder));
			}else
				snprintf(founder,sizeof(botnick),"%c%s%c",toascii(39),mysql_quote(thread,channels.row[1]),toascii(39));
				
			if(channels.row[2][0]=='\0'){
				dynncpy(successor,"NULL",sizeof(successor));
			}else
				snprintf(successor,sizeof(botnick),"%c%s%c",toascii(39),mysql_quote(thread,channels.row[2]),toascii(39));

			domysql(thread,"INSERT INTO `orion_cs_core` "
				"(`serv`,`channel`,`botnick`,`founder`,`successor`,`email`,`settings`, "
				"`description`,`last_topic`,`last_topic_setter`,`last_topic_time`,`url`,`welcome`, "
				"`last_used`,`regdate`,`expire`,`maxmemos`,`bs_set`, "
					
				"`bs_caps_min`,"
				"`bs_caps_per`,"
				"`bs_flood_ln`,"
				"`bs_flood_secs`,"
				"`bs_repeat`,"
				
				"`reason`"
					
				") "
				"VALUES "
				"('ChanServ','%s',%s,%s,%s,'%s','%d',"
				"'%s','%s','%s','%s','%s','%s',"
				"'%ld','%ld','%d','%d',%ld,"
				"'%s',"
				"'%s',"
				"'%s',"
				"'%s',"
				"'%s',"
				"'%s')",
				mysql_quote(thread,channels.row[0]),botnick,founder,successor,mysql_quote(thread,channels.row[5]),chansettings,
				mysql_quote(thread,channels.row[3]),mysql_quote(thread,channels.row[8]),mysql_quote(thread,channels.row[9]),mysql_quote(thread,channels.row[10]),mysql_quote(thread,channels.row[4]),mysql_quote(thread,channels.row[15]),
				stol(channels.row[7])==0 ? (long)time(NULL) : stol(channels.row[7]),stol(channels.row[6]),expire,maxmemos,botsettings,
				mysql_quote(thread,channels.row[19]),
				mysql_quote(thread,channels.row[20]),
				mysql_quote(thread,channels.row[21]),
				mysql_quote(thread,channels.row[22]),
				mysql_quote(thread,channels.row[23]),
				mysql_quote(thread,reason)
			);
		}
	}
	mysql_free_quotes(thread);
	mysql_free_result_myr(channels.res);
	bzero(&channels,sizeof(channels));

/* Moving ChanLevels to ChannelDB */
	channels.res    = doSelectMySQL(thread,"SELECT `channel`,`position`,`level` FROM `anope_cs_levels`");
	channels.fields = mysql_num_rows(channels.res);
	
	if(channels.fields){
		for(i=0;i<channels.fields;i++){
			channels.row = mysql_fetch_row(channels.res);

			if(stoi(channels.row[2])<-ACCESS_SUCCESSOR)
				tmplevel = stoi(channels.row[2])+9000;
			else if(stoi(channels.row[2])>ACCESS_SUCCESSOR)
				tmplevel = stoi(channels.row[2])-9000;
			else
				tmplevel = stoi(channels.row[2]);
				
			if(stoi(channels.row[1])==ANOPE_CA_INVITE) cs_invite=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_AKICK) cs_akick=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_SET) cs_set=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_UNBAN) cs_unban=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_AUTOOP) cs_autoop=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_AUTODEOP) cs_autodeop=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_AUTOVOICE) cs_autovoice=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_OPDEOP) cs_opdeop=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_ACCESS_LIST) cs_acclist=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_CLEAR) cs_cclear=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_NOJOIN) cs_nojoin=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_ACCESS_CHANGE) cs_accchange=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_MEMO) cs_memo=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_ASSIGN) cs_botassign=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_BADWORDS) cs_badwords=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_NOKICK) cs_nokick=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_FANTASIA) cs_fantasia=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_SAY) cs_say=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_GREET) cs_greet=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_VOICEME) cs_voiceme=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_VOICE) cs_voice=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_GETKEY) cs_getkey=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_AUTOHALFOP) cs_autohalfop=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_AUTOPROTECT) cs_autoprotect=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_OPDEOPME) cs_opdeopme=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_HALFOPME) cs_halfopme=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_HALFOP) cs_halfop=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_PROTECTME) cs_protectme=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_PROTECT) cs_protect=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_KICKME) cs_kickme=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_KICK) cs_kick=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_SIGNKICK) cs_signkick=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_BANME) cs_banme=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_BAN) cs_ban=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_TOPIC) cs_topic=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_INFO) cs_info=tmplevel;
			
			if(!streqnc(currchan,channels.row[0])){
				if(currchan[0]!='\0'){
					domysql(thread,
						"UPDATE `orion_cs_core` SET "
							"`cs_set`='%d',"
							"`cs_autodeop`='%d',"
							"`cs_autohalfop`='%d',"
							"`cs_autoop`='%d',"
							"`cs_autoprotect`='%d',"
							"`cs_autovoice`='%d',"
							"`cs_nojoin`='%d',"
							"`cs_signkick`='%d',"
							"`cs_acclist`='%d',"
							"`cs_accchange`='%d',"
							"`cs_akick`='%d',"
							"`cs_ban`='%d',"
							"`cs_banme`='%d',"
							"`cs_cclear`='%d',"
							"`cs_getkey`='%d',"
							"`cs_halfop`='%d',"
							"`cs_halfopme`='%d',"
							"`cs_info`='%d',"
							"`cs_kick`='%d',"
							"`cs_kickme`='%d',"
							"`cs_invite`='%d',"
							"`cs_opdeop`='%d',"
							"`cs_opdeopme`='%d',"
							"`cs_protect`='%d',"
							"`cs_protectme`='%d',"
							"`cs_topic`='%d',"
							"`cs_unban`='%d',"
							"`cs_voice`='%d',"
							"`cs_voiceme`='%d',"
							"`cs_memo`='%d',"
							"`cs_botassign`='%d',"
							"`cs_badwords`='%d',"
							"`cs_fantasia`='%d',"
							"`cs_greet`='%d',"
							"`cs_nokick`='%d',"
							"`cs_say`='%d'"
						" WHERE `channel`='%s'",
						cs_set,
						cs_autodeop,
						cs_autohalfop,
						cs_autoop,
						cs_autoprotect,
						cs_autovoice,
						cs_nojoin,
						cs_signkick,
						cs_acclist,
						cs_accchange,
						cs_akick,
						cs_ban,
						cs_banme,
						cs_cclear,
						cs_getkey,
						cs_halfop,
						cs_halfopme,
						cs_info,
						cs_kick,
						cs_kickme,
						cs_invite,
						cs_opdeop,
						cs_opdeopme,
						cs_protect,
						cs_protectme,
						cs_topic,
						cs_unban,
						cs_voice,
						cs_voiceme,
						cs_memo,
						cs_botassign,
						cs_badwords,
						cs_fantasia,
						cs_greet,
						cs_nokick,
						cs_say,
						currchan
					);
					cs_set = CS_AUTOOP;
					cs_autodeop = CS_AUTODEOP;
					cs_autohalfop = CS_AUTOHALFOP;
					cs_autoop = CS_AUTOOP;
					cs_autoprotect = CS_AUTOPROTECT;
					cs_autovoice = CS_AUTOVOICE;
					cs_nojoin = CS_NOJOIN;
					cs_signkick = CS_SIGNKICK;
					cs_acclist = CS_ACCESS_LIST;
					cs_accchange = CS_ACCESS_CHANGE;
					cs_akick = CS_AKICK;
					cs_ban = CS_BAN;
					cs_banme = CS_BANME;
					cs_cclear = CS_CLEAR;
					cs_getkey = CS_GETKEY;
					cs_halfop = CS_HALFOP;
					cs_halfopme = CS_HALFOPME;
					cs_info = CS_INFO;
					cs_kick = CS_KICK;
					cs_kickme = CS_KICKME;
					cs_invite = CS_INVITE;
					cs_opdeop = CS_OPDEOP;
					cs_opdeopme = CS_OPDEOPME;
					cs_protect = CS_PROTECT;
					cs_protectme = CS_PROTECTME;
					cs_topic = CS_TOPIC;
					cs_unban = CS_UNBAN;
					cs_voice = CS_VOICE;
					cs_voiceme = CS_VOICEME;
					cs_memo = CS_MEMO;
					cs_botassign = CS_ASSIGN;
					cs_badwords = CS_BADWORDS;
					cs_fantasia = CS_FANTASIA;
					cs_greet = CS_GREET;
					cs_nokick = CS_NOKICK;
					cs_say = CS_SAY;

					bzero(&currchan,sizeof(currchan));
					dynncpy(currchan,channels.row[0],sizeof(currchan));
				}else{
					bzero(&currchan,sizeof(currchan));
					dynncpy(currchan,channels.row[0],sizeof(currchan));
				}
			}
		}
	}
	mysql_free_result_myr(channels.res);
	
	bzero(&channels,sizeof(channels));
	bzero(&currchan,sizeof(currchan));
	
/* Moving BotKickers to ChannelDB */
	channels.res    = doSelectMySQL(thread,"SELECT `channel`,`ttb_id`,`value` FROM `anope_cs_ttb`");
	channels.fields = mysql_num_rows(channels.res);
	if(channels.fields){
		for(i=0;i<channels.fields;i++){
			channels.row = mysql_fetch_row(channels.res);

			if(stoi(channels.row[1])==ANOPE_TTB_BOLDS) bs_bolds_ttb=stoi(channels.row[2]);
			else if(stoi(channels.row[1])==ANOPE_TTB_COLORS) bs_colors_ttb=stoi(channels.row[2]);
			else if(stoi(channels.row[1])==ANOPE_TTB_REVERSES) bs_reverses_ttb=stoi(channels.row[2]);
			else if(stoi(channels.row[1])==ANOPE_TTB_UNDERLINES) bs_underlines_ttb=stoi(channels.row[2]);
			else if(stoi(channels.row[1])==ANOPE_TTB_BADWORDS) bs_badwords_ttb=stoi(channels.row[2]);
			else if(stoi(channels.row[1])==ANOPE_TTB_CAPS) bs_caps_ttb=stoi(channels.row[2]);
			else if(stoi(channels.row[1])==ANOPE_TTB_FLOOD) bs_flood_ttb=stoi(channels.row[2]);
			else if(stoi(channels.row[1])==ANOPE_TTB_REPEAT) bs_repeat_ttb=stoi(channels.row[2]);
			
			if(!streqnc(currchan,channels.row[0])){
				if(currchan[0]!='\0'){
					domysql(thread,
						"UPDATE `orion_cs_core` SET "
							"`bs_bolds`='%d',"
							"`bs_colors`='%d',"
							"`bs_reverses`='%d',"
							"`bs_underlines`='%d',"
							"`bs_badwords`='%d',"
							"`bs_caps_ttb`='%d',"
							"`bs_flood_ttb`='%d',"
							"`bs_repeat_ttb`='%d'"
						" WHERE `channel`='%s'",
						bs_bolds_ttb,
						bs_colors_ttb,
						bs_reverses_ttb,
						bs_underlines_ttb,
						bs_badwords_ttb,
						bs_caps_ttb,
						bs_flood_ttb,
						bs_repeat_ttb,
						currchan
					);
					
					bzero(&currchan,sizeof(currchan));
					dynncpy(currchan,channels.row[0],sizeof(currchan));
					
				}else{
					bzero(&currchan,sizeof(currchan));
					dynncpy(currchan,channels.row[0],sizeof(currchan));
				}
			}
		}
	}
	mysql_free_result_myr(channels.res);
	bzero(&channels,sizeof(channels));

/* Moving ChanAccess from anope_cs_access to chanaccess */
	channels.res    = doSelectMySQL(thread,"SELECT `display`,`channel`,`level` FROM `anope_cs_access`");
	channels.fields = mysql_num_rows(channels.res);
	if(channels.fields){
		for(i=0;i<channels.fields;i++){
			channels.row = mysql_fetch_row(channels.res);

			dlog(thread,0,"Moving user = \"%s\", channel = \"%s\"\n",channels.row[0],channels.row[1]);

			if(stoi(channels.row[2])>ACCESS_SUCCESSOR)
				tmplevel = 1000;
			else
				tmplevel = stoi(channels.row[2]);

			domysql(thread,"INSERT INTO `orion_cs_access` (`serv`,`authnick`,`channel`,`level`) VALUES ('ChanServ','%s','%s','%d')",
			mysql_quote(thread,channels.row[0]),mysql_quote(thread,channels.row[1]),tmplevel);
		}
	}
	mysql_free_quotes(thread);
	mysql_free_result_myr(channels.res);
	bzero(&channels,sizeof(channels));		
	
/* Moving Badwords from anope_cs_badwords to botbadwords */
	domysql(thread,"INSERT INTO `orion_bs_badwords` (`art`,`channel`,`badword`) SELECT `type`,`channel`,`word` FROM `anope_cs_badwords`");
	
/* Moving Akicks from anope_cs_akicks to akick */
	domysql(thread,"INSERT INTO `orion_cs_akick` (`channel`,`flags`,`mask`,`reason`,`creator`,`addtime`) SELECT `channel`,`flags`,`dmask`,`reason`,`creator`,`addtime` FROM `anope_cs_akicks` WHERE `dmask`!=''");

/* Moving Memos from anope_ms_info to memos */
	domysql(thread,"INSERT INTO `orion_ms_core` (`receiver`,`sender`,`memo`,`mode`,`time`) SELECT `receiver`,`sender`,`text`,`flags`,`time` FROM `anope_ms_info`");

/* Moving Vhosts from anope_hs_core to vhosts */
	channels.res    = doSelectMySQL(thread,"SELECT `anope_hs_core`.`nick`,`anope_ns_alias`.`display`,`anope_hs_core`.`vident`,`anope_hs_core`.`vhost`,`anope_hs_core`.`creator`,`anope_hs_core`.`time` FROM `anope_hs_core` LEFT OUTER JOIN `anope_ns_alias` ON (`anope_ns_alias`.`nick` = `anope_hs_core`.`nick`)");
	channels.fields = mysql_num_rows(channels.res);
	if(channels.fields){
		for(i=0;i<channels.fields;i++){
			channels.row = mysql_fetch_row(channels.res);

			if(channels.row[1]==NULL)
				continue;

			vhosted = 0;
			bzero(&vhost,sizeof(vhost));
			if(channels.row[2][0]!='\0'){
				straddns(vhost,channels.row[2],sizeof(vhost));
				vhosted = 1;
			}
			if(channels.row[3][0]!='\0'){
				if(vhosted)
					straddnc(vhost,'@',sizeof(vhost));
				straddns(vhost,channels.row[3],sizeof(vhost));
			}
			
			dlog(thread,0,"Moving vhost for nick = \"%s\" Groupnick: \"%s\"\n",channels.row[0],channels.row[1]);
			
			domysql(thread,
						"INSERT INTO `orion_hs_core` (`nick`,`display`,`nick_type`,`vhost`,`authorized`,`changetime`,`active`) VALUES "
						"('%s','%s','0','%s','%s','%ld','0')",
						mysql_quote(thread,channels.row[0]),mysql_quote(thread,channels.row[1]),mysql_quote(thread,vhost),mysql_quote(thread,channels.row[4]),stol(channels.row[5])
					);
			mysql_free_quotes(thread);
		}
	}
	mysql_free_result_myr(channels.res);
	bzero(&channels,sizeof(channels));

	printf(" [*] MoveDB: Database moved successful ...\n");
	mysql_disconnect(thread);
}
