/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 194                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-16 18:33:19 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file notice.c
 *   @brief This is the Module file for NOTICE (Server Side) Command.
 *
 *   It contains all functions which are neccessary to process and use 
 *   the NOTICE command.
 *
 *   The NOTICE File is used to process NOTICES sent from Servers
 *   and to Servers. (Protocol-Process-File -> PPF)
 *   (Notice functions for the server, some functions are depending on the selected Language.)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"

/**
 * @brief do_notice:  Sends out a NOTICE (depending on the Loaded Protocol) - using NOTICE
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick (only services nicks) that sends out the NOTICE
 * @param target The target (NICK/CHANNEL) to receive that NOTICE
 * @param message The notice message itself
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_notice(Threads *thread, char *nick, char *target, char *message, ...){
	/* Let'z check out inputs - are they valid? */	
	if(nick==NULL || target==NULL || message==NULL)
		return CMD_ERROR;
		
	/* Deklarate Variables */
	va_list va;
	char tmp[BUFFER+1];
	
	/* Initialise Variables to (NULL) */	
	bzero(&tmp,sizeof(tmp));

	/* Join reason string */
	va_start	(va,  message);
	vsnprintf	(tmp, sizeof(tmp)-1, message, va);
	va_end		(va);

	return orion_WRCtranslate_cmd_notice(thread,nick,target,tmp);
}

/*  ************************************************************************************************  */

/**
 * @brief do_notice_lang:  Sends out a NOTICE (depending on the Loaded Protocol and the set language) - using NOTICE
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param lang The Language in which we want to send the notice (pointer to the .so file (dl_handle))
 * @param nick The Nick (only services nicks) that sends out the NOTICE
 * @param target The target (NICK/CHANNEL) to receive that NOTICE
 * @param message The notice message itself
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_notice_lang(Threads *thread, void *lang, char *nick, char *target, char *message, ...){
	PROTOCOL_BOOL analyzer = CMD_ERROR;
	
	/* Let'z check out inputs - are they valid? */	
	if(lang==NULL || nick==NULL || target==NULL || message==NULL)
		return CMD_ERROR;
		
	/* Deklarate Variables */
	va_list va;
	char tmp[PUTSERVER_BUF_LEN+1];
	char strout[PUTSERVER_BUF_LEN+1];
	char *param[ARRAY_SIZE];
	
	void *define	= NULL;
	size_t items	= 0;
	size_t i		= 0;
	
	/* Initialise Variables to (NULL) */	
	bzero(&tmp,sizeof(tmp));
	bzero(&strout,sizeof(strout));
	bzero(param,sizeof(param));

	/* Join reason string */
	define = dynsym(thread,lang,message);						// Get the language :>
	
	va_start	(va,  (const char*)message);
	vsnprintf	(tmp, sizeof(tmp)-1, (const char*)define, va);
	va_end		(va);

	items = strsplit(tmp,param,'\n');
	
	for(i=0;i<items;i++){

		bzero(&strout,sizeof(strout));
		straddns(strout,param[i],sizeof(strout));
		straddnc(strout,'\n',sizeof(strout));

		analyzer = orion_WRCtranslate_cmd_notice(thread,nick,target,strout);
		if(analyzer >= CMD_NOT_SUPPORTED)
			return analyzer;
	}
	return analyzer;
}

/*  ************************************************************************************************  */
