/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 207                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-05 12:32:05 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file part.c
 *   @brief This is the Module file for PART (Server Side) Command.
 *
 *   It contains all functions which are neccessary to process and use 
 *   the PART command.
 *
 *   The PART File is used to process PARTS sent from Servers
 *   and to Servers. (Protocol-Process-File -> PPF)
 *   (Part functions clients and server - manages the DB.)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"

/**
 * @brief do_sapart:  Sends out a PART (depending on the Loaded Protocol) - using SAPART
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param snick The ServicesNick which sends the PART
 * @param nick The Nick we want to part from target channel
 * @param chan The Channel where we want to part the nick
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_sapart(Threads *thread, char *snick, char *nick, char *chan){
	PROTOCOL_BOOL analyzer = CMD_ERROR;
	Users *user = GetUser(nick);
	
	/* Let'z check out inputs - are they valid? */
	if(snick==NULL || nick==NULL || chan==NULL)
		return CMD_ERROR;

	/* Let'z check if the channel is a valid one? */
	if(chan[0]=='#' || chan[0]=='&'){
		if(orion_WRCtranslate_valid_chan(thread, chan) == false)
			return CMD_ERROR;
			
		if(orion_WRCtranslate_valid_nick(thread, nick) == false)
			return CMD_ERROR;
			
		if(user==NULL)
			return CMD_ERROR;
		
		/* Analyze Command - is it supported from the IRCd !? */
		analyzer = orion_WRCtranslate_cmd_sapart(thread,snick,nick,chan);
		if(analyzer >= CMD_NOT_SUPPORTED)
			return analyzer;
		
		/* part channel - change user channel data (we keep data but set that the user isn't in that chan anymore) */
    	PartChannel(user,chan);
		
		/* Command supported .. we start analyzing it */
		if(!IRC.DoesGlobalChangeAfterSA)
			part(thread,nick,chan);
	}
	else return CMD_ERROR;
		
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief do_svspart:  Sends out a PART (depending on the Loaded Protocol) - using SVSPART
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param snick The ServicesNick which sends the PART
 * @param nick The Nick we want to part from target channel
 * @param chan The Channel where we want to part the nick
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_svspart(Threads *thread, char *snick, char *nick, char *chan){
	PROTOCOL_BOOL analyzer = CMD_ERROR;
	Users *user = GetUser(nick);
	
	/* Let'z check out inputs - are they valid? */
	if(snick==NULL || nick==NULL || chan==NULL)
		return CMD_ERROR;

	/* Let'z check if the channel is a valid one? */
	if(chan[0]=='#' || chan[0]=='&'){
		if(orion_WRCtranslate_valid_chan(thread, chan) == false)
			return CMD_ERROR;
			
		if(orion_WRCtranslate_valid_nick(thread, nick) == false)
			return CMD_ERROR;
			
		if(user==NULL)
			return CMD_ERROR;
		
		/* Analyze Command - is it supported from the IRCd !? */
		analyzer = orion_WRCtranslate_cmd_svspart(thread,snick,nick,chan);
		if(analyzer >= CMD_NOT_SUPPORTED)
			return analyzer;

		/* part channel - change user channel data (we keep data but set that the user isn't in that chan anymore) */
    	PartChannel(user,chan);

		/* Command supported .. we start analyzing it */
		if(!IRC.DoesGlobalChangeAfterSVS)
			part(thread,nick,chan);
	}
	else return CMD_ERROR;
		
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief do_part:  Sends out a PART (depending on the Loaded Protocol) - using PART
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The ServicesNick which should use PART for target Channel
 * @param chan The Channel which we want to part
 * @param partmsg The Partmessage we want to provide to the public (reason for parting)
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_part(Threads *thread, char *nick, char *chan, char *partmsg){
	PROTOCOL_BOOL analyzer = CMD_ERROR;
	Users *user = GetUser(nick);
/* PART Function Initialising */

	/* Let'z check out inputs - are they valid? */
	if(nick==NULL || chan==NULL)
		return CMD_ERROR;

	if(chan[0]=='#' || chan[0]=='&'){
		if(orion_WRCtranslate_valid_chan(thread, chan) == false)
			return CMD_ERROR;
			
		if(orion_WRCtranslate_valid_nick(thread, nick) == false)
			return CMD_ERROR;
			
		if(user==NULL)
			return CMD_ERROR;
		
		/* Analyze Command - is it supported from the IRCd !? */
		analyzer = orion_WRCtranslate_cmd_part(thread,nick,chan,partmsg);
		if(analyzer >= CMD_NOT_SUPPORTED)
			return analyzer;

		/* part channel - change user channel data (we keep data but set that the user isn't in that chan anymore) */
    	PartChannel(user,chan);
    
		/* Command supported .. we start analyzing it */
		part(thread,nick,chan);
	}
	else return CMD_ERROR;
		
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief on_part:  Receives a PART (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param user The Nick which PART's target Channel
 * @param in_channel The Channel which was parted by target Nick
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_part(Threads *thread, char *user, char *in_channel){
/* PART Function Initialising */
	char *eventarray[3];
	Users *user_info = GetUser(user);
	
	if(user==NULL || in_channel==NULL)
		return CMD_ERROR;
		
	if(user_info==NULL)
		return CMD_ERROR;

	if(in_channel[0]=='#' || in_channel[0]=='&'){
		/* part channel - change user channel data (we keep data but set that the user isn't in that chan anymore) */
    	PartChannel(user_info,in_channel);

		part(thread,user,in_channel);
		
		bzero(eventarray,sizeof(eventarray));
		eventarray[0] = user;
		eventarray[1] = in_channel;
		eventarray[2] = NULL;
		/*************** SEND on_part to the eventhandler ***************/
		_ExecuteEvent(user,eventarray,"on_part");
		/****************************************************************/
	}else return CMD_ERROR;
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */

/**
 * @brief part:  Processes the X-PART-Statements for deleting Information from SQL (NO OUTPUT!)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param in_user The Nick which PART's target Channel
 * @param in_channel The Channel which was parted by target Nick
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL part(Threads *thread, char *in_user, char *in_channel){
	//ChannelDB c;
	deepSQL channels;
	char nick[MAX_NICK_LEN*2];
	char channel[MAX_CHANNELNAME_LEN*2];
	Users *user = GetUser(in_user);
	
	//bzero(&c,sizeof(c));
	bzero(&channels,sizeof(channels));
	bzero(&nick,sizeof(nick));
	bzero(&channel,sizeof(channel));
	
	//receive -> *bits, receive2 -> *param
	if(user==NULL)
		return CMD_ERROR;

	/* Updates the Threads Channel Management */
	UpdateChannelInformation3(thread,in_channel,NULL);

	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, nick, in_user, strlen(in_user)); //escape bits[0] -> nick
	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, channel, in_channel, strlen(in_channel)); //escape bits[2] -> channel

	domysql(thread,"DELETE FROM `orion_chan_users` WHERE `nick`='%s' AND `channel`='%s'",nick,channel);
	thread->c.chan.countusers--;

	/* part channel - change user channel data (we keep data but set that the user isn't in that chan anymore) */
    PartChannel(user,in_channel);
	
	if(streqnc(in_user,thread->c.chan.bot))
		thread->c.chan.botinchan=0;
	
	if(thread->c.chan.countusers==0){
		if(thread->c.chan.serv[0]!='\0'){	// channel empty, but registered by a bot (Q/L/ChanServ/etc...)
		
			/* Update memory for caching */
#ifndef USE_THREADS
			thread->c.chan.datecreate 	= (time_t)0;
			thread->c.chan.limit 		= 0;
			bzero(&thread->c.chan.mode,sizeof(thread->c.chan.mode));
			bzero(&thread->c.chan.link,sizeof(thread->c.chan.link));
			bzero(&thread->c.chan.flood,sizeof(thread->c.chan.flood));
			bzero(&thread->c.chan.key,sizeof(thread->c.chan.key));
#endif
			
			if((streqnc(thread->c.chan.serv,"q") && !strcor(thread->c.chan.flags,"t")) || (streqnc(thread->c.chan.serv,"chanserv") && ~thread->c.chan.settings&CS_KEEPTOPIC_BIT))
				bzero(&thread->c.chan.last_topic,sizeof(thread->c.chan.last_topic));
		
			/* Update SQL as live update */
			if((streqnc(thread->c.chan.serv,"q") && strcor(thread->c.chan.flags,"t") && ~thread->c.chan.settings&CS_SUSPEND_BIT) || (streqnc(thread->c.chan.serv,"chanserv") && thread->c.chan.settings&CS_KEEPTOPIC_BIT && ~thread->c.chan.settings&CS_SUSPEND_BIT)){
				domysql(thread,"UPDATE `orion_cs_core` SET "
				"`datecreate`='',`mode`='',`link`='',`limit`='',`flood`='',`key`=''"
				" WHERE `channel`='%s'",channel);
			}else{
				domysql(thread,"UPDATE `orion_cs_core` SET "
				"`datecreate`='',`mode`='',`link`='',`limit`='',`flood`='',`key`='',`last_topic`=''"
				" WHERE `channel`='%s'",channel);
			}

			domysql(thread,"DELETE `orion_chan_bans`,`orion_chan_expbans` FROM `orion_chan_bans` LEFT OUTER JOIN `orion_chan_expbans` ON `orion_chan_expbans`.`channel`=`orion_chan_bans`.`channel` WHERE `orion_chan_bans`.`channel`='%s'",mysql_quote(thread,channel));
		}else{	// channel empty - no bots -> delete it...
			/* Update memory for caching */
#ifndef USE_THREADS
			bzero(&thread->c,sizeof(thread->c));
#endif
			
			/* Update SQL as live update */
			domysql(thread,"DELETE FROM `orion_cs_core` WHERE `channel`='%s'",channel);
		}
		return CMD_EXECUTED;
	}
	else if(((~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers==1) || (thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers==2)) && thread->c.chan.bot[0]!='\0' && bs->loaded){
		do_part(thread,thread->c.chan.bot,in_channel,NULL);	
		return CMD_EXECUTED;
	}
	return CMD_ERROR;
}


/*  ************************************************************************************************  */

/**
 * @brief partall:  Processes the X-PARTALL-Statements for deleting Information from SQL (NO OUTPUT!)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param user The Nick which PART's all channels (masspart)
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL partall(Threads *thread, char *nick){
	/* Global variables */
	deepSQL channels;
	char chan_update[(MAX_CHANNELNAME_LEN+3)*MAX_CHANS];
	char chan_delete[(MAX_CHANNELNAME_LEN+3)*MAX_CHANS];
	char chan_bot   [(MAX_CHANNELNAME_LEN+3)*MAX_CHANS];
	int i=0;
	Users *user = GetUser(nick);

	/* Initialise Variables to (NULL) */
	bzero(&channels,sizeof(channels));
	bzero(&chan_bot,sizeof(chan_bot));
	bzero(&chan_update,sizeof(chan_update));
	bzero(&chan_delete,sizeof(chan_delete));
	
	if(user==NULL)
		return CMD_ERROR;
	
	domysql(thread,"DELETE `orion_chan_users` FROM `orion_chan_users` WHERE `orion_chan_users`.`nick`='%s'",mysql_quote(thread,nick));

	/* part all channels - change user channel data (we keep data but set that the user isn't in that chan anymore) */
    PartAllChannels(user);

	/* 	Update our Cache	*/
	if(streqnc(thread->c.Cdata.nick,nick)){
		bzero(&thread->c.Cdata,sizeof(thread->c.Cdata));
		thread->c.aliases.active = 0;
		
		unknown_access(thread->c.chan.access.targetlevel,sizeof(thread->c.chan.access.targetlevel));
	}else if(streqnc(thread->sql.Cdata.nick,nick)){
		bzero(&thread->sql.Cdata,sizeof(thread->sql.Cdata));
		thread->sql.aliases.active = 0;
		
		unknown_access(thread->c.chan.access.ownlevel,sizeof(thread->c.chan.access.ownlevel));
	}
	//printf("thread cache umgeschrieben\n");
	/**			***			**/
	//printf("vor doSelectMySQL();\n");
	channels.res    = doSelectMySQL(thread,"SELECT `orion_cs_core`.`serv`,`orion_cs_core`.`channel`,`orion_cs_core`.`botnick`, SUM(`orion_chan_users`.`nick`=`orion_cs_core`.`botnick`) as `botinchan`, COUNT(`orion_chan_users`.`nick`) as `count_users`,`orion_cs_core`.`settings` FROM `orion_cs_core` LEFT JOIN `orion_chan_users` ON (`orion_cs_core`.`channel` = `orion_chan_users`.`channel`) WHERE `orion_cs_core`.`datecreate`!='' GROUP BY `orion_cs_core`.`channel`");
	channels.fields = mysql_num_rows(channels.res);
	//printf("vor for();\n");
	for(i=0;i<channels.fields;i++){
		channels.row = mysql_fetch_row(channels.res);
		int settings = stoi(channels.row[5]);
		
		if(
			(
				( ~settings&CS_INHABIT_BIT && stoi(channels.row[4])==1 && stoi(channels.row[3])==1 ) ||
				( settings&CS_INHABIT_BIT && stoi(channels.row[4])==2 && stoi(channels.row[3])==1)
			) ||
			stoi(channels.row[4])==0
		){
			if(streqnc(channels.row[1],thread->c.chan.name))
				thread->c.chan.countusers--;
			
			if(channels.row[0][0]!='\0'){	// channel empty, but registered by a bot (Q/L/ChanServ/etc...)
			
				straddnc(chan_update,toascii(39),sizeof(chan_update));
				straddns(chan_update,mysql_quote(thread,channels.row[1]),sizeof(chan_update));
				straddnc(chan_update,toascii(39),sizeof(chan_update));
				straddnc(chan_update,',',sizeof(chan_update));
			
				if(channels.row[2]!=NULL && stoi(channels.row[3])==1 && bs->loaded){
					orion_WRCtranslate_cmd_part(thread,channels.row[2],channels.row[1],NULL);
					
					straddnc(chan_bot,toascii(39),sizeof(chan_bot));
					straddns(chan_bot,mysql_quote(thread,channels.row[1]),sizeof(chan_bot));
					straddnc(chan_bot,toascii(39),sizeof(chan_bot));
					straddnc(chan_bot,',',sizeof(chan_bot));
					
				}
	
			}else{
				straddnc(chan_delete,toascii(39),sizeof(chan_delete));
				straddns(chan_delete,mysql_quote(thread,channels.row[1]),sizeof(chan_delete));
				straddnc(chan_delete,toascii(39),sizeof(chan_delete));
				straddnc(chan_delete,',',sizeof(chan_delete));
			}
		}
	}
	
	/** Update MySQL (Update channels which are empty & registered, Delete all bans (cauze channel is empty), 
		Delete and the Bot (it was parted in the loop), Delete unregistered Channels that are empty **/
		
	if(chan_update[0]!='\0'){
		chan_update[dynlen(chan_update)-1]='\0';
		
		domysql(thread,"UPDATE `orion_cs_core` SET "
			"`datecreate`='',`mode`='',`link`='',`limit`='',`flood`='',`key`=''"
			" WHERE `channel` IN(%s)",chan_update);
		
		domysql(thread,"DELETE `orion_chan_bans`,`orion_chan_expbans` FROM `orion_chan_bans` LEFT OUTER JOIN `orion_chan_expbans` ON `orion_chan_expbans`.`channel`=`orion_chan_bans`.`channel` WHERE `orion_chan_bans`.`channel` IN(%s)",chan_update);
	}
	
	if(chan_delete[0]!='\0'){
		chan_delete[dynlen(chan_delete)-1]='\0';
		domysql(thread,"DELETE FROM `orion_cs_core` WHERE `channel`='%s'",chan_delete);
	}
	
	if(chan_bot[0]!='\0'){
		chan_bot[dynlen(chan_bot)-1]='\0';
		domysql(thread,"DELETE `orion_chan_users` FROM `orion_chan_users` LEFT OUTER JOIN `orion_cs_core` ON (`orion_cs_core`.`botnick`=`orion_chan_users`.`nick` AND `orion_cs_core`.`channel`=`orion_chan_users`.`channel`) WHERE `orion_cs_core`.`botnick` IS NOT NULL AND `orion_chan_users`.`channel` IN(%s)",chan_bot);
	}
	//printf("vor mysql_free_result_myr();\n");
	mysql_free_result_myr(channels.res); //free result

	/* 
	InnoDB: using ON UPDATE/DELETE CASCADE -> Update/Delete not neccessary anymore
	NOTE: mode system changed, old tables not neccessary anylonger...
	*/
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */
