/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 204                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-03 20:33:44 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file protocol.c
 *   @brief This is the Module file for PROTOCOL (Server Side) protocol management.
 *
 *   It contains all functions which are neccessary to process and use 
 *   the PROTOCOL.
 *
 *   The PROTOCOL File is used to process ALL Inputs and Outputs.
 *   (Protocol-Process-File -> PPF)
 *   (This file is "responsible" for handling all server inputs - protocol options.)
 */
/*  ************************************************************************************************  */

#include "sysconf.h"	//ifdef USE_THREADS
#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"
#include "version.h"	//on_version (/VERSION *services* on IRC)


/**
 * @brief on_sethost:  Set's a new Host for target user (Received only)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick for which the Vhost was set
 * @param vhost The Vhost which was set for target nick
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_sethost(Threads *thread, char *nick, char *vhost){
	Users *u = NULL;
	char *eventarray[3];
	bzero(eventarray,sizeof(eventarray));

	u = GetUser(nick);
	if(u != NULL){
#ifndef USE_THREADS
		/* Update memory for caching */
		if(streqnc(thread->sql.Cdata.nick,nick)){
			//bzero(&thread->sql.Cdata.vhost,sizeof(thread->sql.Cdata.vhost));
			dynncpy(thread->sql.Cdata.vhost,vhost,sizeof(thread->sql.Cdata.vhost));
		}else if(streqnc(thread->c.Cdata.nick,nick)){
			//bzero(&thread->c.Cdata.vhost,sizeof(thread->c.Cdata.vhost));
			dynncpy(thread->c.Cdata.vhost,vhost,sizeof(thread->c.Cdata.vhost));
		}
#endif

		/* Update SQL as live update */
		domysql(thread,"UPDATE `orion_core_condata` SET `vhost`='%s' WHERE `nick`='%s'",mysql_quote(thread,vhost),mysql_quote(thread,nick));
#ifdef USE_THREADS
		my_pthread_mutex_lock(&users_lock);
#endif
		dynncpy(u->vhost,vhost,sizeof(u->vhost));
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&users_lock);
#endif
	
		eventarray[0] = nick;
		eventarray[1] = vhost;
		eventarray[2] = NULL;
		/**************************** SEND on_setident to the eventhandler ****************************/
		_ExecuteEvent(nick,eventarray,"on_sethost");
		/**********************************************************************************************/
		return CMD_EXECUTED;
	}
	return CMD_ERROR;
}

/*  ************************************************************************************************  */

/**
 * @brief on_chghost:  Changes the old VHost to a new one for target user (Received only)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick for which the Vhost was changed
 * @param vhost The Vhost which was changed for target nick
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_chghost(Threads *thread, char *nick, char *vhost){
	Users *u = NULL;
	char *eventarray[3];
	bzero(eventarray,sizeof(eventarray));
	
	u = GetUser(nick);
	if(u != NULL){
#ifndef USE_THREADS
		/* Update memory for caching */
		if(streqnc(thread->sql.Cdata.nick,nick)){
			//bzero(&thread->sql.Cdata.vhost,sizeof(thread->sql.Cdata.vhost));
			dynncpy(thread->sql.Cdata.vhost,vhost,sizeof(thread->sql.Cdata.vhost));
		}else if(streqnc(thread->c.Cdata.nick,nick)){
			//bzero(&thread->c.Cdata.vhost,sizeof(thread->c.Cdata.vhost));
			dynncpy(thread->c.Cdata.vhost,vhost,sizeof(thread->c.Cdata.vhost));
		}
#endif
	
		/* Update SQL as live update */
		domysql(thread,"UPDATE `orion_core_condata` SET `vhost`='%s' WHERE `nick`='%s'",mysql_quote(thread,vhost),mysql_quote(thread,nick));

#ifdef USE_THREADS
		my_pthread_mutex_lock(&users_lock);
#endif
		dynncpy(u->vhost,vhost,sizeof(u->vhost));
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&users_lock);
#endif
	
		eventarray[0] = nick;
		eventarray[1] = vhost;
		eventarray[2] = NULL;
		/**************************** SEND on_setident to the eventhandler ****************************/
		_ExecuteEvent(nick,eventarray,"on_chghost");
		/**********************************************************************************************/
		return CMD_EXECUTED;
	}
	return CMD_ERROR;
}

/*  ************************************************************************************************  */

/**
 * @brief on_setident:  Set's a new Ident for target user (Received only)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick for which the Ident was set
 * @param ident The Ident which was set for target nick
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_setident(Threads *thread, char *nick, char *ident){
	Users *u = NULL;
	char *eventarray[3];
	bzero(eventarray,sizeof(eventarray));
	
	u = GetUser(nick);
	if(u != NULL && IRC.SupportVIdents){
#ifndef USE_THREADS
		/* Update memory for caching */
		if(streqnc(thread->sql.Cdata.nick,nick)){
			//bzero(&thread->sql.Cdata.ident,sizeof(thread->sql.Cdata.ident));
			dynncpy(thread->sql.Cdata.ident,ident,sizeof(thread->sql.Cdata.ident));
		}else if(streqnc(thread->c.Cdata.nick,nick)){
			//bzero(&thread->c.Cdata.ident,sizeof(thread->c.Cdata.ident));
			dynncpy(thread->c.Cdata.ident,ident,sizeof(thread->c.Cdata.ident));
		}
#endif
	
		/* Update SQL as live update */
		domysql(thread,"UPDATE `orion_core_condata` SET `ident`='%s' WHERE `nick`='%s'",mysql_quote(thread,ident),mysql_quote(thread,nick));

#ifdef USE_THREADS
		my_pthread_mutex_lock(&users_lock);
#endif
		dynncpy(u->vident,ident,sizeof(u->vident));
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&users_lock);
#endif
	
		eventarray[0] = nick;
		eventarray[1] = ident;
		eventarray[2] = NULL;
		/**************************** SEND on_setident to the eventhandler ****************************/
		_ExecuteEvent(nick,eventarray,"on_setident");
		/**********************************************************************************************/
		return CMD_EXECUTED;
	}
	return CMD_ERROR;
}

/*  ************************************************************************************************  */

/**
 * @brief on_chgident:  Changes the old Ident to a new one for target user (Received only)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick for which the Ident was changed
 * @param ident The Ident which was changed for target nick
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_chgident(Threads *thread, char *nick, char *ident){
	Users *u = NULL;
	char *eventarray[3];
	bzero(eventarray,sizeof(eventarray));
	
	u = GetUser(nick);
	if(u != NULL && IRC.SupportVIdents){
#ifndef USE_THREADS
		/* Update memory for caching */
		if(streqnc(thread->sql.Cdata.nick,nick)){
			//bzero(&thread->sql.Cdata.ident,sizeof(thread->sql.Cdata.ident));
			dynncpy(thread->sql.Cdata.ident,ident,sizeof(thread->sql.Cdata.ident));
		}else if(streqnc(thread->c.Cdata.nick,nick)){
			//bzero(&thread->c.Cdata.ident,sizeof(thread->c.Cdata.ident));
			dynncpy(thread->c.Cdata.ident,ident,sizeof(thread->c.Cdata.ident));
		}
#endif
	
		/* Update SQL as live update */
		domysql(thread,"UPDATE `orion_core_condata` SET `ident`='%s' WHERE `nick`='%s'",mysql_quote(thread,ident),mysql_quote(thread,nick));

#ifdef USE_THREADS
		my_pthread_mutex_lock(&users_lock);
#endif
		dynncpy(u->vident,ident,sizeof(u->vident));
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&users_lock);
#endif
	
		eventarray[0] = nick;
		eventarray[1] = ident;
		eventarray[2] = NULL;
		/**************************** SEND on_setident to the eventhandler ****************************/
		_ExecuteEvent(nick,eventarray,"on_chgident");
		/**********************************************************************************************/
		return CMD_EXECUTED;
	}
	return CMD_ERROR;
}

/*  ************************************************************************************************  */

/**
 * @brief do_chghost:  Changes the old Host to a new one for target user (Send only)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param snick The ServicesNick which sends out the CHGHOST information
 * @param nick The Nick for which the Vhost should be changed
 * @param host The new Vhost for target nick
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_chghost(Threads *thread, char *snick, char *nick, char *host){
	Users *u = NULL;
	
	/* Let'z check our inputs - are they valid? */
	if(snick==NULL || host==NULL || nick==NULL)
		return CMD_ERROR;
	
	u  = GetUser(nick);	
	if(u != NULL && IRC.VHost){
#ifndef USE_THREADS
		/* Update memory for caching */
		if(streqnc(thread->sql.Cdata.nick,nick)){
			//bzero(&thread->sql.Cdata.vhost,sizeof(thread->sql.Cdata.vhost));
			dynncpy(thread->sql.Cdata.vhost,host,sizeof(thread->sql.Cdata.vhost));
		}else if(streqnc(thread->c.Cdata.nick,nick)){
			//bzero(&thread->c.Cdata.vhost,sizeof(thread->c.Cdata.vhost));
			dynncpy(thread->c.Cdata.vhost,host,sizeof(thread->c.Cdata.vhost));
		}
#endif
		
		/* Update SQL as live update */
		domysql(thread,"UPDATE `orion_core_condata` SET `vhost`='%s' WHERE `nick`='%s'",mysql_quote(thread,host),mysql_quote(thread,nick));

#ifdef USE_THREADS
		my_pthread_mutex_lock(&users_lock);
#endif
		bzero(&u->vhost,sizeof(u->vhost));
		dynncpy(u->vhost,host,sizeof(u->vhost));
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&users_lock);
#endif

		return orion_WRCtranslate_cmd_chghost(thread,snick,nick,host);
	}
	return CMD_ERROR;
}

/*  ************************************************************************************************  */

/**
 * @brief do_chgident:  Changes the old Ident to a new one for target user (Send only)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param snick The ServicesNick which sends out the CHGIDENT information
 * @param nick The Nick for which the Ident should be changed
 * @param ident The new Ident for target nick
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_chgident(Threads *thread, char *snick, char *nick, char *ident){
	Users *u = NULL;

	/* Let'z check our inputs - are they valid? */
	if(snick==NULL || nick==NULL || ident==NULL)
		return CMD_ERROR;
		
	u  = GetUser(nick);	
	if(u != NULL && IRC.SupportVIdents){
#ifndef USE_THREADS
		/* Update memory for caching */
		if(streqnc(thread->sql.Cdata.nick,nick)){
			//bzero(&thread->sql.Cdata.ident,sizeof(thread->sql.Cdata.ident));
			dynncpy(thread->sql.Cdata.ident,ident,sizeof(thread->sql.Cdata.ident));
		}else if(streqnc(thread->c.Cdata.nick,nick)){
			//bzero(&thread->c.Cdata.ident,sizeof(thread->c.Cdata.ident));
			dynncpy(thread->c.Cdata.ident,ident,sizeof(thread->c.Cdata.ident));
		}
#endif

		/* Update SQL as live update */
		domysql(thread,"UPDATE `orion_core_condata` SET `ident`='%s' WHERE `nick`='%s'",mysql_quote(thread,ident),mysql_quote(thread,nick));

#ifdef USE_THREADS
		my_pthread_mutex_lock(&users_lock);
#endif
		dynncpy(u->vident,ident,sizeof(u->vident));
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&users_lock);
#endif

		return orion_WRCtranslate_cmd_chgident(thread,snick,nick,ident);
	}
	return CMD_ERROR;
}

/*  ************************************************************************************************  */

/**
 * @brief on_chgname:  Changes the old Realname to a new one for target user (Receive only)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick for which the Realname was changed
 * @param name The new Realname for target nick
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_chgname(Threads *thread, char *nick, char *name){
	Users *u = NULL;
	
	if(nick==NULL || name==NULL)
		return CMD_ERROR;
		
	u  = GetUser(nick);	
	if(u != NULL && IRC.SupportRealnameChanges){
#ifndef USE_THREADS
		/* Update memory for caching */
		if(streqnc(thread->sql.Cdata.nick,nick)){
			//bzero(&thread->sql.Cdata.realname,sizeof(thread->sql.Cdata.realname));
			dynncpy(thread->sql.Cdata.realname,name,sizeof(thread->sql.Cdata.realname));
		}else if(streqnc(thread->c.Cdata.nick,nick)){
			//bzero(&thread->c.Cdata.realname,sizeof(thread->c.Cdata.realname));
			dynncpy(thread->c.Cdata.realname,name,sizeof(thread->c.Cdata.realname));
		}
#endif
	
		/* Update SQL as live update */
		domysql(thread,"UPDATE `orion_core_condata` SET `realname`='%s' WHERE `nick`='%s'",mysql_quote(thread,name),mysql_quote(thread,nick));

		//dynncpy(u->vident,ident,sizeof(u->vident));
		//orion_WRCtranslate_cmd_chgname(m->nick,nick,name);
		return CMD_EXECUTED;
	}
	return CMD_ERROR;
}

/*  ************************************************************************************************  */

/**
 * @brief version:  Sends out the version of the ServicesProgram (Receive only)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick which requested the version information
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL version(Threads *thread, char *nick){
	main_raw(thread,":%s 351 %s %s %s [compiled %s %s] :Copyright (C)%s, shock feat. Ally, cLiEbiG",m->servername,nick,VERSION,m->servername,__DATE__, __TIME__,YEAR);
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */

/**
 * @brief on_privmsg:  Processes the SocketInput (Receive only)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param param The Parameter-Array which holds the splitted IRCd-message
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_privmsg(Threads *thread, char *param[]){
	deepSQL useSQL;
	Aliases aliases;
	Access  access;

	char in_path[MAX_FILEPATH_LEN];
	
	int User_Level     = INFINIT;
	int Root_Level     = INFINIT;
	
	char *nick = NULL;
	char *cmd  = NULL;
	
	bzero(&useSQL,sizeof(useSQL));
	bzero(&aliases,sizeof(aliases));
	bzero(&access,sizeof(access));
	bzero(&in_path,sizeof(in_path));
	
	time_t now = time(NULL);

	if(param[2][0]!='#'){	
		if(m->OSMsgFloodProt){
			Users *user = GetUser(&param[0][1]);
			now 		= time(NULL);

#ifdef USE_THREADS
			my_pthread_mutex_lock(&users_lock);
#endif
			if(user==NULL){
#ifdef USE_THREADS
				my_pthread_mutex_unlock(&users_lock);
#endif
				return CMD_ERROR;
			}
			
			if(user->rights < is_services_oper){
				if(m->OSMsgFloodKill && (now - user->Ignoretime < m->OSMsgFloodKill)){
					user->Ignoretime = time(NULL);
#ifdef USE_THREADS
					my_pthread_mutex_unlock(&users_lock);
#endif
					return CMD_ERROR;
				}			

				if (now - user->LastServicesMsg > m->OSMsgFloodtime) {
					user->LastServicesMsg 	= time(NULL);
					user->ServicesMsgLines 	= 0;
				}
					
				user->ServicesMsgLines++;
				if (user->ServicesMsgLines >= m->OSMsgFloodlines) {
					if(!m->OSMsgFloodKill) do_kill(thread,m->servername,user->nick,"killed cause of MessageFlooding Services - ABUSE");
					else{
						if(O->loaded)	msend(thread,0,O->nick,user->nick,"Due to a Floodattack services are not available at the moment, please be patient");
						else			msend(thread,0,m->servername,user->nick,"Due to a Floodattack services are not available at the moment, please be patient");
						user->Ignoretime = time(NULL);
					}
#ifdef USE_THREADS
					my_pthread_mutex_unlock(&users_lock);
#endif
					return CMD_EXECUTED;
				}
			}
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&users_lock);
#endif
		}
		
		//printf("protocol.c: Message Input = %s\n",_djoin(' ',param));
		
		/***************** SEND do_message to the eventhandler ****************/
		_ExecuteEvent(&param[0][1],param,"on_message");
		/**********************************************************************/
	}
	
	nick = &param[0][1]; /* Update the nick */
	cmd  = &param[3][1]; /* Update our current command */
	
	ModInfo ModHandle;
	bzero(&ModHandle,sizeof(ModInfo));
	
	if(GetUser(nick)==NULL)
		return CMD_ERROR;
	
	if(strcor(currmodes(thread,nick),"o") && streqnc(param[2],m->nick)){
		bzero(&aliases,sizeof(aliases));
		aliases = sql_aliases(thread,nick);
		access  = sql_access(thread,aliases.authnick);
		
/* We execute an event requested by a User with Olevel >= INFINIT */
		if(aliases.active && access.olevel>=INFINIT && param[4] && streqnc(cmd,"event")){
			if(param[4]){
				/********** SEND do_message to the eventhandler ***********/
				_ExecuteEvent(NULL,param,param[4]);
				/**********************************************************/
			}
		}
		
/* We execute loadmod */
		else if(aliases.active && access.olevel>=1000 && param[4] && streqnc(cmd,"loadmod")){
			
			/* We load a service with all it's modules */
			if(!param[5]){
				if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_core_commands` WHERE `service`='%s'",mysql_quote(thread,param[4])))==0){
					if(load_modules(thread,param[4]).ModErr == MOD_ERR_OK){
						ExecuteSingleEvent(thread,nick,param,param[4],"on_connect_primary");
						ExecuteSingleEvent(thread,nick,param,param[4],"on_connect_secondary");
						do_privmsg(thread,m->nick,nick,"Done.");
					}else do_privmsg(thread,m->nick,nick,"No such Service.");
				}else do_privmsg(thread,m->nick,nick,"Service already loaded!");
			}

			/* We load a module for a specific service */
			else if(param[5]){
				if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_core_commands` WHERE `service`='%s' AND `modul`='%s'",mysql_quote(thread,param[4]),mysql_quote(thread,param[5])))==0){
					ModHandle = IRCLoadModule(thread,languages[m->language],0,m->nick,nick,param,param[4],param[5]);

					if(ModHandle.ModErr == MOD_ERR_OK){
						User_Level = ModHandle.module->LeastUserLevel;
						Root_Level = ModHandle.module->LeastRootLevel;
						domysql(thread,"INSERT INTO `orion_core_commands` (`service`,`modul`,`user_level`,`root_level`) VALUES ('%s','%s',%d,%d)",mysql_quote(thread,param[4]),mysql_quote(thread,param[5]),User_Level,Root_Level);
						do_privmsg(thread,m->nick,nick,"Done.");
					}else do_privmsg(thread,m->nick,nick,"Modul ERROR - Modul may not be supported (config-setting or IRCd-specific)");
				}else do_privmsg(thread,m->nick,nick,"Modul already loaded.");
			}
		}

/* We execute reloadmod */
		else if(aliases.active && access.olevel>=1000 && streqnc(cmd,"reloadmod")){
			if(streqnc(param[4],"languages")){
				/* Re Initializes the Languages */
				reinit_langs(thread);
				
				do_privmsg(thread,m->nick,nick,"Done - Langfiles reloaded.");
			}else if(param[4] && !param[5]){
				reload_modules(thread,param[4]);

				do_privmsg(thread,m->nick,nick,"Done.");
			}else if(param[4] && param[5]){
				if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_core_commands` WHERE `service`='%s' AND `modul`='%s'",mysql_quote(thread,param[4]),mysql_quote(thread,param[5])))!=0){
					if(IRCUnloadModule(thread,languages[m->language],0,m->nick,nick,param,param[4],param[5],DO_RELOAD)==MOD_ERR_OK){
						domysql(thread,"DELETE FROM `orion_core_commands` WHERE `service`='%s' AND `modul`='%s'",mysql_quote(thread,param[4]),mysql_quote(thread,param[5]));

						ModHandle = IRCLoadModule(thread,languages[m->language],0,m->nick,nick,param,param[4],param[5]);
						if(ModHandle.ModErr == MOD_ERR_OK){
							User_Level = ModHandle.module->LeastUserLevel;
							Root_Level = ModHandle.module->LeastRootLevel;
							domysql(thread,"INSERT INTO `orion_core_commands` (`service`,`modul`,`user_level`,`root_level`) VALUES ('%s','%s',%d,%d)",mysql_quote(thread,param[4]),mysql_quote(thread,param[5]),User_Level,Root_Level);
						}
					}
					do_privmsg(thread,m->nick,nick,"Done.");
				}else do_privmsg(thread,m->nick,nick,"Modul already loaded.");
			}else if(!param[4]){
				char *fgetstest = NULL;
				char si[10];
				bzero(&si,sizeof(si));
				
				FILE *pidcheck = fopen(m->pidfilename, "r" ); // open the pidfile in read mode to check if its represented
				if(pidcheck != NULL){ // if no pidfile is represented try to write one
					fgetstest = fgets(si, 10, pidcheck);
					strclean(si);
					
					int ppid = stoi(si);
					if(ppid){
						kill(ppid,SIGUSR2);
						do_privmsg(thread,m->nick,nick,"Done.");
					}else{
						do_privmsg(thread,m->nick,nick,"ERROR: Reloading failed.");
					}
				}
			}
		}
		
/* We execute unloadmod */
		else if(aliases.active && access.olevel>=1000 && param[4] && streqnc(cmd,"unloadmod")){
			if(param[4] && !param[5]){
				if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_core_commands` WHERE `service`='%s'",mysql_quote(thread,param[4])))!=0){
					unload_modules(thread,param[4]);
					do_privmsg(thread,m->nick,nick,"Done.");
					do_privmsg(thread,m->nick,nick,"modul unloaded.");
				}else do_privmsg(thread,m->nick,nick,"Service not loaded!");
			}else if(param[4] && param[5]){
				if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_core_commands` WHERE `service`='%s' AND `modul`='%s'",mysql_quote(thread,param[4]),mysql_quote(thread,param[5])))!=0){
					if(IRCUnloadModule(thread,languages[m->language],0,m->nick,nick,param,param[4],param[5],DO_UNLOAD)==MOD_ERR_OK){
						do_privmsg(thread,m->nick,nick,"Modul: %s of Service: %s unloaded.",param[5],param[4]);
						domysql(thread,"DELETE FROM `orion_core_commands` WHERE `service`='%s' AND `modul`='%s'",mysql_quote(thread,param[4]),mysql_quote(thread,param[5]));
					}
				}else do_privmsg(thread,m->nick,nick,"Modul not loaded!");
			}else do_privmsg(thread,m->nick,nick,"Mainmodul not loaded!");
		}

/* We execute restart */
		else if(aliases.active && access.olevel>=1000 && streqnc(cmd,"restart")){
			if(param[4]){
				do_global(thread,m->servername,"%s used restart for %s - Terminated",nick,param[4]);
				unload_modules(thread,param[4]);
				MOD_ERROR Error = load_modules(thread,param[4]).ModErr;
				if(Error == MOD_ERR_OK){
					
					ExecuteSingleEvent(thread,nick,param,param[4],"on_connect_primary");
					ExecuteSingleEvent(thread,nick,param,param[4],"on_connect_secondary");

					do_privmsg(thread,m->nick,nick,"Done.");
				}else
					do_privmsg(thread,m->nick,nick,"ERROR: %s",ModuleError(Error));
			}else if(!param[4]){
				do_global(thread,m->servername,"%s used restart for %s - Terminated",nick,m->servername);
				do_privmsg(thread,m->nick,nick,"Doing restart now.");

				char *fgetstest = NULL;
				char si[10];
				bzero(&si,sizeof(si));
				
				FILE *pidcheck = fopen(m->pidfilename, "r" ); // open the pidfile in read mode to check if its represented
				if(pidcheck != NULL){ // if no pidfile is represented try to write one
					fgetstest = fgets(si, 10, pidcheck);
					strclean(si);
					
					int ppid = stoi(si);
					if(ppid){
						kill(ppid,SIGHUP);
					}else exit(0);
				}
			}
		}else if(aliases.active && access.olevel>=1000 && param[4] && streqnc(cmd,"help")){
			if(streqnc(param[4],"unloadmod")){
				send_lang(thread,languages[access.language],1,m->nick,nick,"REQUIRED_LEVEL",1000);
				send_lang(thread,languages[access.language],1,m->nick,nick,"ORION_HELP_UNLOADMOD");
			}else if(streqnc(param[4],"reloadmod")){
				send_lang(thread,languages[access.language],1,m->nick,nick,"REQUIRED_LEVEL",1000);
				send_lang(thread,languages[access.language],1,m->nick,nick,"ORION_HELP_RELOADMOD");
			}else if(streqnc(param[4],"loadmod")){
				send_lang(thread,languages[access.language],1,m->nick,nick,"REQUIRED_LEVEL",1000);
				send_lang(thread,languages[access.language],1,m->nick,nick,"ORION_HELP_LOADMOD");
			}else if(streqnc(param[4],"restart")){
				send_lang(thread,languages[access.language],1,m->nick,nick,"REQUIRED_LEVEL",1000);
				send_lang(thread,languages[access.language],1,m->nick,nick,"ORION_HELP_RESTART",m->nick);
			}else do_privmsg(thread,m->nick,nick,"Access Denied.");
		}else if(aliases.active && access.olevel>=1000 && !param[4] && streqnc(cmd,"help")){
			send_lang(thread,languages[access.language],1,m->nick,nick,"ORION_HELP_CMD_LOADMOD",1000);
			send_lang(thread,languages[access.language],1,m->nick,nick,"ORION_HELP_CMD_RELOADMOD",1000);
			send_lang(thread,languages[access.language],1,m->nick,nick,"ORION_HELP_CMD_UNLOADMOD",1000);
			send_lang(thread,languages[access.language],1,m->nick,nick,"ORION_HELP_CMD_RESTART",1000);
		}
	}else if(streqnc(param[2],m->nick)){
	/*
	*  Error Messages
	*/
		if(!strcor(currmodes(thread,nick),"o"))	send_lang(thread,languages[m->language],1,m->nick,nick,"NOT_AN_IRCOP");
		else									send_lang(thread,languages[m->language],1,m->nick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	if(param[2][0]=='#'){
		if(GetUser(nick)==NULL)
			return CMD_ERROR;

		/* Execute an Event */
		_ExecuteEvent(nick,param,"on_channelmessage");
	}
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */

/**
 * @brief on_protocol:  Processes and Incoming PRIVMSG-Message
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param protocol The pointer to the Protocol struct (store point for the Protocol functions)
 * @param ac The Number of input arguments
 * @param param The Argument Array
 * @return bool 1 on Success, 0 on Error
 **/
bool on_protocol(Threads *thread, Protocol *protocol, size_t ac, char *param[]){
	Event *ptr = NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&protocol_lock);
#endif
	ptr = protocol->events;
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&protocol_lock);
#endif
	while (ptr) {
		//printf("Call \"%s\" - Checked \"%s\"\n",ptr->call,param[ptr->check_param]);
		if (streqnc(ptr->call,param[ptr->check_param])){
			if(ptr->func(thread,ac,param)==MOD_CONT)
				return 1;
			else
				return 0;
		}
#ifdef USE_THREADS
		my_pthread_mutex_lock(&protocol_lock);
#endif
		ptr = ptr->next;
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&protocol_lock);
#endif
	}
	return 0;
}

/*  ************************************************************************************************  */
