/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 210                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-05 18:15:28 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file UnrealIRCd.c
 *   @brief This is the Module file for UnrealIRCd (Server to Server) protocol.
 *
 *   It contains all functions which are neccessary to process
 *   Inputs from from Unreal and outputs to Unreal!
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "osocket.h"
#include "services.h"
#include "modules.h"

/* Inbound */
MOD_BOOL orion_event_version(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_stats(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_mode(Threads *thread, size_t ac, char **av);

MOD_BOOL orion_event_svsmode(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_svs2mode(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_svsnick(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_svsjoin(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_svspart(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_tkl(Threads *thread, size_t ac, char **av);

MOD_BOOL orion_event_sjoin_0(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_sjoin_1(Threads *thread, size_t ac, char **av);

MOD_BOOL orion_event_topic_0(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_topic_1(Threads *thread, size_t ac, char **av);

MOD_BOOL orion_event_ping(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_pong(Threads *thread, size_t ac, char **av);

MOD_BOOL orion_event_squit(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_quit(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_kill(Threads *thread, size_t ac, char **av);

MOD_BOOL orion_event_join(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_part(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_kick(Threads *thread, size_t ac, char **av);

MOD_BOOL orion_event_setident(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_chgident(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_sethost(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_chghost(Threads *thread, size_t ac, char **av);

MOD_BOOL orion_event_nick(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_server(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_privmsg(Threads *thread, size_t ac, char **av);

/* SERVER MODES */
MOD_BOOL orion_event_started(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_services_link(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_services_link_done(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_linking(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_eos(Threads *thread, size_t ac, char **av);

/* Outbound */
PROTOCOL_BOOL orion_cmd_initialize(Threads *thread);
PROTOCOL_BOOL orion_cmd_add_account(Threads *thread, char *uid, char *authnick);
PROTOCOL_BOOL orion_cmd_del_account(Threads *thread, char *uid);

PROTOCOL_BOOL orion_cmd_topic(Threads *thread, char *whosets, char *chan, char *whosetit, char *topic, time_t when);

PROTOCOL_BOOL orion_cmd_gline(Threads *thread, char *user, char *host, char *who, time_t when, time_t expires, char *reason);
PROTOCOL_BOOL orion_cmd_ungline(Threads *thread, char *unsetter, char *host, char *user, time_t time_in);

PROTOCOL_BOOL orion_cmd_svskill(Threads *thread, char *source, char *user, char *buf);
PROTOCOL_BOOL orion_cmd_kill(Threads *thread, char *source, char *user, char *buf);
PROTOCOL_BOOL orion_cmd_rawsvsmode(Threads *thread, char *source, char *nick, char *modes);
PROTOCOL_BOOL orion_cmd_svsmode(Threads *thread, char *nick, char *target, char *modes);
PROTOCOL_BOOL orion_cmd_svs2mode(Threads *thread, char *nick, char *target, char *modes);
PROTOCOL_BOOL orion_cmd_nick(Threads *thread, char *nick, char *ident, char *host, char *name, char *modes);
PROTOCOL_BOOL orion_cmd_mode(Threads *thread, char *source, char *dest, time_t time_in, char *buf);
PROTOCOL_BOOL orion_cmd_rawmode(Threads *thread, char *source, char *dest, time_t time_in, char *buf);

PROTOCOL_BOOL orion_cmd_kick(Threads *thread, char *source, char *chan, char *user, char *buf);
PROTOCOL_BOOL orion_cmd_notice(Threads *thread, char *source, char *dest, char *buf);
PROTOCOL_BOOL orion_cmd_privmsg(Threads *thread, char *source, char *dest, char *buf);
PROTOCOL_BOOL orion_cmd_serv_notice(Threads *thread, char *source, char *dest, char *msg);
PROTOCOL_BOOL orion_cmd_serv_privmsg(Threads *thread, char *source, char *dest, char *msg);

PROTOCOL_BOOL orion_cmd_quit(Threads *thread, char *source, char *buf);
PROTOCOL_BOOL orion_cmd_protoctl(Threads *thread);
PROTOCOL_BOOL orion_cmd_pass(Threads *thread, char *pass);
PROTOCOL_BOOL orion_cmd_server(Threads *thread, char *servname, int hop, char *descript);
PROTOCOL_BOOL orion_cmd_pong(Threads *thread, char *servname, char *who);
PROTOCOL_BOOL orion_cmd_ping(Threads *thread, char *servname, char *who);
PROTOCOL_BOOL orion_cmd_join(Threads *thread, char *user, char *channel, time_t chantime);
PROTOCOL_BOOL orion_cmd_services_join(Threads *thread, char *user, char *channel, time_t chantime, int fake);

PROTOCOL_BOOL orion_cmd_chghost(Threads *thread, char *source, char *nick, char *vhost);
PROTOCOL_BOOL orion_cmd_chgident(Threads *thread, char *source, char *nick, char *vIdent);
PROTOCOL_BOOL orion_cmd_invite(Threads *thread, char *source, char *chan, char *nick);
PROTOCOL_BOOL orion_cmd_part(Threads *thread, char *nick, char *chan, char *buf);

PROTOCOL_BOOL orion_cmd_wallops(Threads *thread, char *source, char *fmt);
PROTOCOL_BOOL orion_cmd_global(Threads *thread, char *source, char *buf);

PROTOCOL_BOOL orion_cmd_sqline(Threads *thread, char *sender, char *mask, time_t time_in, time_t expire, char *reason);
PROTOCOL_BOOL orion_cmd_unsqline(Threads *thread, char *servername, char *nick, char *snick);

PROTOCOL_BOOL orion_cmd_squit(Threads *thread, char *servname, char *message);
PROTOCOL_BOOL orion_cmd_svso(Threads *thread, char *source, char *nick, char *flag);
PROTOCOL_BOOL orion_cmd_chg_nick(Threads *thread, char *oldnick, char *newnick);
PROTOCOL_BOOL orion_cmd_svsnick(Threads *thread, char *source, char *oldnick, char *newnick, time_t when);
PROTOCOL_BOOL orion_cmd_connect(Threads *thread, bool how);
PROTOCOL_BOOL orion_cmd_svshold(Threads *thread, char *nick);
PROTOCOL_BOOL orion_cmd_release_svshold(Threads *thread, char *nick);

PROTOCOL_BOOL orion_cmd_szline(Threads *thread, char *servername, char *mask, char *snick, time_t when, time_t expire, char *reason);
PROTOCOL_BOOL orion_cmd_unszline(Threads *thread, char *servername, char *mask, char *snick, time_t in_time);

PROTOCOL_BOOL orion_cmd_unban(Threads *thread, char *name, char *nick);
PROTOCOL_BOOL orion_cmd_svsmode_chan(Threads *thread, char *name, char *mode, char *nick);
PROTOCOL_BOOL orion_cmd_svsjoin(Threads *thread, char *source, char *nick, char *chan, time_t date);
PROTOCOL_BOOL orion_cmd_svspart(Threads *thread, char *source, char *nick, char *chan);
PROTOCOL_BOOL orion_cmd_sapart(Threads *thread, char *source, char *nick, char *chan);
PROTOCOL_BOOL orion_cmd_sajoin(Threads *thread, char *source, char *nick, char *chan, time_t date);

PROTOCOL_BOOL orion_cmd_swhois(Threads *thread, char *source, char *who, char *mask);
PROTOCOL_BOOL orion_cmd_eob(Threads *thread);
PROTOCOL_BOOL orion_cmd_jupe(Threads *thread, char *jserver, char *who, char *reason);
PROTOCOL_BOOL orion_cmd_global_legacy(Threads *thread, char *source, char *fmt);

PROTOCOL_BOOL orion_cmd_raw(Threads *thread, char *param[]);
bool orion_valid_nick(Threads *thread, char *nick);
bool orion_valid_chan(Threads *thread, char *chan);
bool orion_flood_mode_check(Threads *thread, char *value);

/* *INDENT-OFF* */
void CreateProtocol(Threads *thread, Protocol *protocol, void *dl_handle) {
    
	//printf("Creating Protocol ...\n");
    event_init(thread, protocol, dl_handle, "shock", "UnrealIRCd 3.2.x and above [version 0.1]");

    createEvent(thread,protocol, "JOIN",      orion_event_join,     1);
    createEvent(thread,protocol, "SVSJOIN",   orion_event_svsjoin,  1);
    createEvent(thread,protocol, "PART",      orion_event_part,     1);
    createEvent(thread,protocol, "SVSPART",   orion_event_svspart,  1);

    createEvent(thread,protocol, "MODE",      orion_event_mode,     1);
    createEvent(thread,protocol, "SVSMODE",   orion_event_svsmode,  1);
    createEvent(thread,protocol, "SVS2MODE",  orion_event_svs2mode, 1);

    createEvent(thread,protocol, "KICK",      orion_event_kick,     1);
    createEvent(thread,protocol, "NICK",      orion_event_nick,     1);
    createEvent(thread,protocol, "NICK",      orion_event_nick,     0);
	createEvent(thread,protocol, "SVSNICK",   orion_event_svsnick,  1);
	
    createEvent(thread,protocol, "TOPIC",     orion_event_topic_0,  0);
    createEvent(thread,protocol, "TOPIC",     orion_event_topic_1,  1);
    createEvent(thread,protocol, "SJOIN",     orion_event_sjoin_0,  0);
    createEvent(thread,protocol, "SJOIN",     orion_event_sjoin_1,  1);
    
    createEvent(thread,protocol, "PING",      orion_event_ping,     0);
    createEvent(thread,protocol, "PONG",      orion_event_pong,     0);
    createEvent(thread,protocol, "PRIVMSG",   orion_event_privmsg,  1);
    createEvent(thread,protocol, "QUIT",      orion_event_quit,     1);
    createEvent(thread,protocol, "KILL",      orion_event_kill,     1);
    
    createEvent(thread,protocol, "TKL",       orion_event_tkl,      1);
    createEvent(thread,protocol, "CHGHOST",   orion_event_chghost,  1);
    createEvent(thread,protocol, "CHGIDENT",  orion_event_chgident, 1);
    
    createEvent(thread,protocol, "SETHOST",   orion_event_sethost,  1);
    createEvent(thread,protocol, "SETIDENT",  orion_event_chgident, 1);

    createEvent(thread,protocol, "VERSION",   orion_event_version,  1);
    createEvent(thread,protocol, "STATS",     orion_event_stats,    1);
    
    /** Server Modes **/
    createEvent(thread,protocol, "SERVER",    orion_event_server,					1);
    createEvent(thread,protocol, "SERVER",    orion_event_server,					0);
    createEvent(thread,protocol, "SQUIT",     orion_event_squit,					0);

    createEvent(thread,protocol, "PROTOCTL",  orion_event_started,					0);
    createEvent(thread,protocol, "NETINFO",   orion_event_services_link_done,		0);
    
    createEvent(thread,protocol, "Link",      orion_event_linking,					4);
    createEvent(thread,protocol, "EOS",       orion_event_eos,						1);
    
   protocol->NSMode						= "+oSq";
   protocol->CSMode						= "+oSq";
   protocol->MSMode						= "+oSq";
   protocol->HSMode						= "+oSq";
   protocol->OSMode						= "+oSq";
   protocol->BSMode						= "+oSq";
   protocol->HeServMode					= "+oSq";
   protocol->GlobMode					= "+oSq";
   protocol->QMode						= "+oSq";
   protocol->SCMode						= "+oSq";
   protocol->BSBotsMode					= "+iSq";
   protocol->ChannelModes				= "cfijklmnprstzACGHMKLNOQRSTVu";	/* Avail Channel Modes */
   protocol->ParametricChannelModes		= "klLfIj";							/* Channel Modes with an allowed Parameter (all possible) */
   protocol->MLockAbleModes				= "klLf";							/* Channel Modes provided by MLOCK */
   protocol->OnSuspend					= "+Opmntil 1";						/* Channel Modes on adding a suspend (forbid) */
   protocol->OnUnSuspend				= "-Opmntil";						/* Channel Modes on removing a suspend (forbid) */
   protocol->SupportSVS					= true;								/* Supports SVS-COMMANDS */
   protocol->SupportSVSJoin				= true;								/* Supports SVSJOIN */
   protocol->SvsNick					= true;								/* Supports SVSNICK */
   protocol->VHost						= true;								/* Supports VHosts */
   protocol->HasOwner					= true;								/* Supports ChannelOwnerMode (+q) */
   protocol->HasProtect					= true;								/* Supports ChannelProtectMode (+a) */
   protocol->OwnerMode					= 'q';								/* UserChannelMode for Owner */
   protocol->ProtectMode				= 'a';								/* UserChannelMode for Protected */
   protocol->OpMode						= 'o';								/* UserChannelMode for Op */
   protocol->HOpMode					= 'h';								/* UserChannelMode for HalfOp */
   protocol->VMode						= 'v';								/* UserChannelMode for Voice */
   protocol->BanMode					= 'b';								/* UserChannelMode for Ban */
   protocol->ExceptMode					= 'e';								/* UserChannelMode for BanException */
   protocol->SupportedUserModes			= "iowghraAsORTVSxNCWqBzvdHtGp";	/* Known UserModes */
   protocol->SupportedUChanModes		= "qaohvbe";						/* All UserChannelModes */
   protocol->SupportedU2ChanModes		= "qaohv";							/* All UserChannelModes except "Except (+e)" and Ban (+b) */
   protocol->ModeOnNickReg				= "+r";								/* Mode for Registered / Identified Nicks */
   protocol->ModeOnNickDrop				= "-r";								/* Mode for Dropping a registered Nick */
   protocol->ModeOnChannelReg			= "+r";								/* Mode for Registered Channels */
   protocol->ModeOnChannelDrop			= "-r";								/* Mode for Dropping a registered Channel */
   protocol->RegMode					= 'r';								/* Mode for Registration */
   protocol->ChanRegMode				= true;								/* Supports a Channel Registration Mode */
   protocol->NickRegMode				= true;								/* Supports a Nick Registration Mode */
   protocol->SupportSGlines				= true;								/* Supports Glines */
   protocol->SupportSQlines				= true;								/* Supports Qlines */
   protocol->SupportSZlines				= true;								/* Supports Zlines */
   protocol->SupportHalfOp				= true;								/* Supports HalfOpMode */
   protocol->SupportBanMode				= true;								/* Supports BanMode */
   protocol->SupportExceptMode			= true;								/* Supports ExceptMode */
   protocol->SupportProtectUmode		= true;								/* Supports a Protection UMode */
   protocol->ProtectUmode				= "Sq";								/* Mode for Protecting a Nickname */
   protocol->SupportInvisibleUMode		= true;								/* Supports InvisibleUMode */
   protocol->InvisibleUMode				= "I";								/* Mode for Invisibility */
   protocol->SupportChanSQlines			= false;							/* Supports Chan SQlines */
   protocol->SupportVIdents				= true;								/* Supports VIdent's */
   protocol->VHostOnNickChange			= true;								/* Supports setting VHosts on NickChanges */
   protocol->SupportRealnameChanges		= false;							/* Supports RealnameChanges */
   protocol->SupportAdminOnlyChanMode	= true;								/* Supports an AdminOnly ChannelMode */
   protocol->SupportOperOnlyChanMode	= true;								/* Supports an OperOnly ChannelMode */
   protocol->DoesGlobalChangeAfterSVS	= true;								/*  */
   protocol->DoesGlobalChangeAfterSA	= true;								/*  */
   protocol->DoesQuitAfterKill			= false;							/* Sends a Quit-Msg on Kill Execution */
   protocol->AdminOnlyCMode				= "A";								/* AdminOnly ChannelMode */
   protocol->OperOnlyCMode				= "O";								/* OperOnly ChannelMode */
   protocol->DefaultMLOCK				= "+ntr";							/* Default MLOCK Modes */
   protocol->VhostMode					= 'x';								/* Vhost Mode (is set on identify) */
   protocol->SupportFloodMode			= 1;								/* Supports a FloodMode */
   protocol->SupportLinkMode			= 1;								/* Supports a ChannelLinkMode */
   protocol->FloodCMode					= 'f';								/* ChannelFloodMode setting */
   protocol->LinkCMode					= 'L';								/* ChannelMode for linking Channels */
   protocol->KeyCMode					= 'k';								/* ChannelKeyMode setting */
   protocol->LimitCMode					= 'l';								/* ChannelMode for limit */
   protocol->InviteCMode				= 'i';								/* ChannelMode for inviteonly */
   protocol->SupportTokens				= false;							/* Supports Tokens */
   protocol->SupportSjoin				= true;								/* Supports SJOIN */
   protocol->SupportSjoinBanprefix		= true;								/* Supports a BanPrefix for SJOIN */
   protocol->SupportSjoinExceptprefix	= true;								/* Supports an ExceptPrefix for SJOIN */
   protocol->SjoinOwnerPrefix			= '*';								/* OwnerPrefix for SJOIN */
   protocol->SjoinProtectPrefix			= '~';								/* ProtectionPrefix for SJOIN */
   protocol->SjoinOpPrefix				= '@';								/* OpPrefix for SJOIN */
   protocol->SjoinHOpPrefix				= '%';								/* HalfOpPrefix for SJOIN */
   protocol->SjoinVoicePrefix			= '+';								/* VoicePrefix for SJOIN */
   protocol->SjoinBanPrefix				= '&';								/* BanPrefix for SJOIN */
   protocol->SjoinExceptPrefix			= '"';								/* ExceptPrefix for SJOIN */
   protocol->SjoinHasIgnoreChar			= false;							/* SJOIN has an ignored Mode (it will be skipped) */
   protocol->SjoinIgnoredChar			= ',';								/* Mode to be ignored from SJOIN */
   protocol->SupportHelperUMode			= true;								/* Supports Helper UMODE */
   protocol->SupportSyncState			= true;								/* Supports SyncState */
   protocol->p10protocol				= false;							/* Is p10 protocol */
   protocol->p10new						= false;							/* Is newp10 protocol */
   
   
	protocol->orion_cmd_initialize		= orion_cmd_initialize;
	protocol->orion_cmd_topic			= orion_cmd_topic;
	protocol->orion_cmd_add_account		= orion_cmd_add_account;
	protocol->orion_cmd_del_account		= orion_cmd_del_account;
	protocol->orion_cmd_kill			= orion_cmd_kill;
	protocol->orion_cmd_svskill			= orion_cmd_svskill;
	protocol->orion_cmd_rawsvsmode		= orion_cmd_rawsvsmode;
	protocol->orion_cmd_svsmode			= orion_cmd_svsmode;
	protocol->orion_cmd_svs2mode		= orion_cmd_svs2mode;
	protocol->orion_cmd_nick			= orion_cmd_nick;
	protocol->orion_cmd_mode			= orion_cmd_mode;
	protocol->orion_cmd_rawmode			= orion_cmd_rawmode;
	protocol->orion_cmd_kick			= orion_cmd_kick;
	protocol->orion_cmd_notice			= orion_cmd_notice;
	protocol->orion_cmd_privmsg			= orion_cmd_privmsg;
	protocol->orion_cmd_serv_notice		= orion_cmd_serv_notice;
	protocol->orion_cmd_serv_privmsg	= orion_cmd_serv_privmsg;
	
	protocol->orion_cmd_quit			= orion_cmd_quit;
	protocol->orion_cmd_protoctl		= orion_cmd_protoctl;
	protocol->orion_cmd_pass			= orion_cmd_pass;
	protocol->orion_cmd_server			= orion_cmd_server;
	protocol->orion_cmd_pong			= orion_cmd_pong;
	protocol->orion_cmd_ping			= orion_cmd_ping;
	protocol->orion_cmd_join			= orion_cmd_join;
	protocol->orion_cmd_services_join	= orion_cmd_services_join;
	protocol->orion_cmd_chghost			= orion_cmd_chghost;
	protocol->orion_cmd_chgident		= orion_cmd_chgident;
	protocol->orion_cmd_invite			= orion_cmd_invite;
	protocol->orion_cmd_part			= orion_cmd_part;
	
	protocol->orion_cmd_wallops			= orion_cmd_wallops;
	protocol->orion_cmd_global			= orion_cmd_global;
	
	protocol->orion_cmd_squit			= orion_cmd_squit;
	//protocol->orion_cmd_svso =orion_cmd_svso
	protocol->orion_cmd_chg_nick		= orion_cmd_chg_nick;
	protocol->orion_cmd_svsnick			= orion_cmd_svsnick;
	protocol->orion_cmd_connect			= orion_cmd_connect;
	/*protocol->orion_cmd_svshold =orion_cmd_svshold
	protocol->orion_cmd_release_svshold =orion_cmd_release_svshold*/
	
	protocol->orion_cmd_gline			= orion_cmd_gline;
	protocol->orion_cmd_ungline			= orion_cmd_ungline;
	
	protocol->orion_cmd_szline			= orion_cmd_szline;
	protocol->orion_cmd_unszline		= orion_cmd_unszline;
	
	protocol->orion_cmd_sqline			= orion_cmd_sqline;
	protocol->orion_cmd_unsqline		= orion_cmd_unsqline;
	
	protocol->orion_cmd_unban			= orion_cmd_unban;
	protocol->orion_cmd_svsmode_chan	= orion_cmd_svsmode_chan;
	protocol->orion_cmd_svsjoin			= orion_cmd_svsjoin;
	protocol->orion_cmd_svspart			= orion_cmd_svspart;
	
	protocol->orion_cmd_sajoin			= orion_cmd_sajoin;
	protocol->orion_cmd_sapart			= orion_cmd_sapart;
	
	protocol->orion_cmd_swhois			= orion_cmd_swhois;
	protocol->orion_cmd_eob				= orion_cmd_eob;
	protocol->orion_cmd_jupe			= orion_cmd_jupe;
	protocol->orion_cmd_global_legacy	= orion_cmd_global_legacy;
	protocol->orion_cmd_raw				= orion_cmd_raw;
	protocol->orion_valid_nick			= orion_valid_nick;
	protocol->orion_valid_chan			= orion_valid_chan;
	protocol->orion_flood_mode_check	= orion_flood_mode_check;
	
	dynncpy(global_target,m->host,sizeof(global_target));
	printf(" [*] Created UnrealIRCd Protocol (version: 0.1)...\n");
	return;
}

#ifdef _WIN32
MOD_BOOL DeleteProtocol(Threads *thread, size_t ac, char **av)
{
    return MOD_CONT;
}
#endif

/**
 * @brief orion_svsmode: Processes the X-MODE for the Database
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick that sends the SVSMODE/SVS2MODE Information
 * @param target The Nick to recive the Modes
 * @param modes The Modes which should be set to target nick
 * @return void
 **/
void orion_svsmode(Threads *thread, char *nick, char *target, char *modes){
	char addmodes[70]; //Modes for Usermode add
	char delmodes[70]; //Modes for Usermode del
	char allmode[80]; //DelModes and AddModes
	
	if(modes == NULL)
		return;
		
	Users *user = GetUser(nick);
	if(!user)
		return;
	
    // < :O SVSMODE shock -w
	//printf("addmodes = %s\n",modes);
	if(target[0]!='#'){
		allmode[0]='\0'; //set allmode NULL
		addmodes[0]='\0'; //set addmodes NULL
		delmodes[0]='\0'; //set delmodes NULL
    	
		char *mode=currmodes(thread,target);
		//printf("currmodes = %s\n",mode);
		bool type=1;
		size_t i=0;
		size_t a=0;
		size_t b=0;
		for(i=0;i<dynlen(modes);i++){
			char current=modes[i];
			if(current=='+'){type=1;}
			else if(current=='-'){type=0;}
			else if(strcin(current,IRC.SupportedUserModes)){
				if(current==IRC.VhostMode && !type) {
	                domysql(thread,"UPDATE `orion_core_condata` SET `vhost`=`rhost` WHERE `nick`='%s'",mysql_quote(thread,target));
				}
				if(type==0 && strcin(current,mode)){
					delmodes[a]=current;
					delmodes[a+1]='\0';
					a++;
				}
				if(type==1 && !strcin(current,mode)){
					addmodes[b]=current;
					addmodes[b+1]='\0';
					b++;
				}
			}
		}
		dynncpy(allmode,mode,sizeof(allmode)); //copy sql saved modes to allmode
		strncat(allmode,addmodes,strlen(addmodes)); //add addmodes to allmode
		for(i=0;delmodes[i];i++){
			strdelc(allmode,delmodes[i]); //delete delmodes[i] from allmode
		}
		
		//printf("allmode = %s\n",allmode);
		
		if(noop!=1){
			/* 	Update our Cache	*/
			if(streqnc(thread->c.Cdata.nick,target)){
				bzero(&thread->c.Cdata.modes,sizeof(thread->c.Cdata.modes));
				dynncpy(thread->c.Cdata.modes,allmode,sizeof(thread->c.Cdata.modes));
			}else if(streqnc(thread->sql.Cdata.nick,target)){
				bzero(&thread->sql.Cdata.modes,sizeof(thread->sql.Cdata.modes));
				dynncpy(thread->sql.Cdata.modes,allmode,sizeof(thread->sql.Cdata.modes));
			}
		/**			***			**/
		
			//Last Bug Found: 6.9.2005 13:11:20 UTC
			
			// > :shock MODE shock :+owghaACeW
			// < :O SVSMODE shock -xowghCeW
			// < :O SVSMODE O -
			// < :O SVSMODE O -
			// < :O SVSMODE O -
			domysql(thread,"UPDATE `orion_core_condata` SET `modes`='%s' WHERE `nick`='%s'",mysql_quote(thread,allmode),mysql_quote(thread,target));
		}
		else if(noop==1){
			bzero(&thread->c,sizeof(thread->c));
			do_kill(thread,m->servername,target,"your olines are nooped.");
		}
	}
	return;
}

/* :shock M shock +i */
/* ABAAA M +ih lol@roflman.de */
void orion_set_umode(Threads *thread, size_t ac, char **av)
{
	//int add = 1;                /* 1 if adding modes, 0 if deleting */
	int paramstart = 0;
	Users *u = NULL;
	deepSQL umode;
	char *modes = NULL;
	char *nick = NULL;
	//char *sethost = NULL;
	bzero(&umode,sizeof(umode));
		    
	//> :shock MODE shock :-x
		    
	/*printf("av[0] = %s\n",av[0]);
	printf("av[1] = %s\n",av[1]);
	printf("av[2] = %s\n",av[2]);
	printf("av[3] = %s\n",av[3]);*/
    
	if(getposition(av[3],'-')>0 || getposition(av[3],'+')>0){
		int neg_pos = getposition(av[3],'-');
		int pos_pos = getposition(av[3],'+');
		
		int pos = neg_pos<pos_pos && neg_pos>=0 ? neg_pos : pos_pos;
		modes = &av[3][pos];
		nick = av[2];
		paramstart = 4;
	}
	
	if(dynlen(modes)<=1)
		return;
	
	if(ac-1<paramstart)
		paramstart = 0;
	
	u = GetUserByUID(nick);
	if(!u)
		u = GetUser(nick);
		
	//printf("No User!?\n");
	if(!u)
		return;
	//printf("we found the User =D\n");
	
    //ac--;

    	dlog(thread,2,"debug: Changing mode for %s to %s\n", u->nick, modes);
        
	char addmodes[70]; //Modes for Usermode add
	char delmodes[70]; //Modes for Usermode del
	char allmode[80]; //DelModes and AddModes
	
    // < :O SVSMODE shock -w
	
	allmode[0]='\0'; //set allmode NULL
	addmodes[0]='\0'; //set addmodes NULL
	delmodes[0]='\0'; //set delmodes NULL
	
	umode.res		= doSelectMySQL(thread,"SELECT `orion_core_condata`.`modes`, COUNT(`orion_ns_aliases`.`nick`), `orion_ns_aliases`.`nick`, `orion_ns_aliases`.`display`,`orion_ns_aliases`.`last_usermask`,`orion_ns_aliases`.`last_realname`,`orion_ns_aliases`.`last_quit`,`orion_ns_aliases`.`time_linked`,`orion_ns_aliases`.`last_login`,`orion_ns_aliases`.`last_used`,`orion_ns_aliases`.`expire`,`orion_ns_aliases`.`linked`,`orion_ns_aliases`.`active`,`orion_ns_aliases`.`forbid` FROM `orion_core_condata` LEFT OUTER JOIN `orion_ns_aliases` ON `orion_ns_aliases`.`nick`=`orion_core_condata`.`nick` WHERE `orion_core_condata`.`nick`='%s' GROUP BY `orion_ns_aliases`.`nick`",mysql_quote(thread,u->nick));
	umode.fields	= mysql_num_rows(umode.res);
	umode.row		= mysql_fetch_row(umode.res);
	
	if(!umode.fields){
		/* Free Quotes-Array */
		mysql_free_quotes(thread);
		
		/* Free MySQL Result */
		mysql_free_result_myr(umode.res);
		return;
	}
	
	char *mode=umode.row[0];
	
	bool type=1;
	size_t i = 0;
	size_t a = 0;
	size_t b = 0;
	for(i=0;i<dynlen(modes);i++){
		char current=modes[i];
		if(current=='+'){type=1;}
		else if(current=='-'){type=0;}

		else if(strcin(current,IRC.SupportedUserModes)){
			if(current==IRC.VhostMode && !type) {
                domysql(thread,"UPDATE `orion_core_condata` SET `vhost`=`rhost` WHERE `nick`='%s'",mysql_quote(thread,u->nick));
			}
			if(type==0 && strcin(current,mode)){
				delmodes[a]=current;
				delmodes[a+1]='\0';
				a++;
			}
			else if(type==1 && !strcin(current,mode)){
				addmodes[b]=current;
				addmodes[b+1]='\0';
				b++;
			}
		}
	}
	dynncpy(allmode,mode,sizeof(allmode)); //copy sql saved modes to allmode
	strncat(allmode,addmodes,strlen(addmodes)); //add addmodes to allmode

	for(i=0;delmodes[i]!='\0';i++){
		strdelc(allmode,delmodes[i]); //delete delmodes[i] from allmode
	}
	
	//printf("Mode Result = %s\n",allmode);
	
	if(noop!=1){
		/* 	Update our Cache	*/
		if(streqnc(thread->c.Cdata.nick,nick)){
			bzero(&thread->c.Cdata.modes,sizeof(thread->c.Cdata.modes));
			dynncpy(thread->c.Cdata.modes,allmode,sizeof(thread->c.Cdata.modes));
		}else if(streqnc(thread->sql.Cdata.nick,nick)){
			bzero(&thread->sql.Cdata.modes,sizeof(thread->sql.Cdata.modes));
			dynncpy(thread->sql.Cdata.modes,allmode,sizeof(thread->sql.Cdata.modes));
		}
	/**			***			**/
	
		//Last Bug Found: 6.9.2005 13:11:20 UTC
		
		// > :shock MODE shock :+owghaACeW
		// < :O SVSMODE shock -xowghCeW
		// < :O SVSMODE O -
		// < :O SVSMODE O -
		// < :O SVSMODE O -
		domysql(thread,"UPDATE `orion_core_condata` SET `modes`='%s' WHERE `nick`='%s'",mysql_quote(thread,allmode),mysql_quote(thread,nick));
	}
	else if(noop==1){
		bzero(&thread->c,sizeof(thread->c));
		do_kill(thread,m->servername,nick,"your olines are nooped.");
	}
	/* Free MySQL Result */
	mysql_free_result_myr(umode.res);

	/* Free Quotes-Array */
	mysql_free_quotes(thread);
}

MOD_BOOL orion_event_version(Threads *thread, size_t ac, char **av)
{
    if (ac < 2)
        return MOD_CONT;
	
	version(thread,&av[0][1]);
	
	/**************************** SEND on_version to the eventhandler ****************************/
	_ExecuteEvent(&av[0][1],NULL,"on_version");
	/*********************************************************************************************/
    return MOD_CONT;
}

MOD_BOOL orion_event_stats(Threads *thread, size_t ac, char **av)
{
    if (ac < 4)
        return MOD_CONT;
        
    if(streqnc(&av[3][1],m->servername))
		on_stats(thread,m->servername,&av[0][1],av[2]);
    return MOD_CONT;
}

//> :shock MODE shock :+iowghaAxWt
//> :klebestreifen.dev.hub.deepirc.net MODE #deep-coderz +ntCN 1196189975
//> :shock MODE #lol +oo cLiEbiG [Anitech]
//> :lolboa MODE #qtest +l 10
MOD_BOOL orion_event_mode(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
	
	if(streqnc(&av[0][1],av[2])){
		//printf("yeah we recognized a usermode!\n");
		orion_set_umode(thread,ac,av);
		//on_mode(thread,&av[0][1],av[2],time(NULL),&av[3][1]);
	}
	
	else if(digitcol(av[ac-1]) && dynlen(av[ac-1])>=10 && dynlen(av[3])==1){
		time_t time_in=(time_t)stol(av[ac-1]);
		av[ac-1]=NULL;
		ac--;
		
		if(DuringServerlink)
		/* This is REALLY IMPORTANT - it avoid's Net Desyncs on linking, if modes happen
			before finishing the synchronisation process */
			MySQL_FlushInserts(thread,SQL_BUFFER_CONDATA,SQL_BUFFER_SJOIN);
		
		on_mode(thread,&av[0][1],av[2],time_in,_dstrjoin(' ',av,3));
	}
	
	else{
		on_mode(thread,&av[0][1],av[2],time(NULL),_dstrjoin(' ',av,3));
	}
    return MOD_CONT;
}

MOD_BOOL orion_event_svsmode(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
	
	//on_mode(thread,&av[0][1],av[2],time(NULL),_dstrjoin(' ',av,3));
	orion_set_umode(thread,ac,av);
    return MOD_CONT;
}

MOD_BOOL orion_event_svs2mode(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
	
	//on_mode(thread,&av[0][1],av[2],time(NULL),_dstrjoin(' ',av,3));
	orion_set_umode(thread,ac,av);
    return MOD_CONT;
}

MOD_BOOL orion_event_svsnick(Threads *thread, size_t ac, char **av)
{
    /* :source SVSNICK old new */
    if (ac < 4)
        return MOD_CONT;
	
    chgnick(thread,av[2],av[3]);
    return MOD_CONT;
}

//:<source server> SVSJOIN <nick> <channel>
MOD_BOOL orion_event_svsjoin(Threads *thread, size_t ac, char **av)
{
    if (ac < 4)
        return MOD_CONT;
    
    _join(thread,0,av[2],av[3],time(NULL));
    return MOD_CONT;
}

//:<source server> SVSPART <nick> <channel>
MOD_BOOL orion_event_svspart(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
    
    on_part(thread,av[2],av[3]);
    return MOD_CONT;
}

//> :markus.deepirc.net TKL + Q * nick shock!nMe@netadmin.deepirc.net 0 1077828192 :gline 1
MOD_BOOL orion_event_tkl(Threads *thread, size_t ac, char **av){
	if(av[3][0]=='G'){
		if(av[2][0]=='+'){
			on_gline(thread,&av[0][1],_dynadd("%s@%s",av[4],av[5]),av[6],(time_t)stol(av[7]),(time_t)stol(av[8]),&_dstrjoin(' ',av,9)[1]);
		}else if(av[2][0]=='-'){
			on_ungline(thread,&av[0][1],_dynadd("%s@%s",av[4],av[5]));
		}
	}
	else if(av[2][0]=='Z'){
		if(av[2][0]=='+'){
			on_szline(thread,&av[0][1],av[5],av[6],(time_t)stol(av[7]),(time_t)stol(av[8]),&_dstrjoin(' ',av,9)[1]);
		}else if(av[2][0]=='-'){
			on_unszline(thread,&av[0][1],av[5]);
		}
	}
	else if(av[3][0]=='Q'){
		if(av[2][0]=='+'){
			on_sqline(thread,av[6],av[5],(time_t)stol(av[7]),(time_t)stol(av[8]),&_dstrjoin(' ',av,9)[1]);
		}else if(av[2][0]=='-'){
			on_unsqline(thread,av[6],&av[0][1],av[5]);
		}
	}
	return MOD_CONT;
}

//> SJOIN 1207083528 #qtest +ntirCNlk 10 lol :@*~lolboa
MOD_BOOL orion_event_sjoin_0(Threads *thread, size_t ac, char **av)
{
	char *buf = NULL;
	char modebuf[BUFSIZE];
	int a=-1, b=-1, i=0;
	bzero(&modebuf,sizeof(modebuf));
	
	if(DuringServerlink)
		MySQL_FlushInserts(thread,SQL_BUFFER_CONDATA,SQL_BUFFER_CONDATA_NO_MODES);
	
	/* Updates the Threads Channel Management */
	UpdateChannelInformation3(thread,av[2],NULL);
	//> SJOIN 1206481045 #test +rl 5 :giorgos @+*~Gio 
	if(av[3][0]=='+'){
		if(dynlen(av[3])>1){
			for(i=0;i<ac;i++){
				if(av[i][0]==':' && b==-1){
					b = i;
					break; //added 8.8.2008 (keine ahnung welche auswirkungen das wirklich hat, wegen obigem SJOIN geadded)
				}
				else if(av[i][0]=='#' && a==-1)
					a = i+1;
			
				else if(a!=-1 && i>=a){
					straddns(modebuf,av[i],sizeof(modebuf));
					straddnc(modebuf,' ',sizeof(modebuf));
				}
			}
			strclean(modebuf);
		
			on_mode(thread,O->nick,av[2],(time_t)stol(av[1]),modebuf);
		}else
			b = 4;
		
		if(b>-1)
			buf = _dstrjoin(' ',av,b);
	}else
		buf = _dstrjoin(' ',av,3);
	
    if(buf!=NULL)
		sjoin(thread,av[2],(time_t)stol(av[1]),buf[0] == ':' ? &buf[1] : buf,0);
    return MOD_CONT;
}

//> :nexus.hub.eu.deepirc.net SJOIN 1179011136 #deepirc +ntrl 27 :+blubbtier +[Anitech] @+~Aftershock 
MOD_BOOL orion_event_sjoin_1(Threads *thread, size_t ac, char **av)
{
	char *buf = NULL;
	char modebuf[BUFSIZE];
	int a=-1, b=-1, i=0;
	bzero(&modebuf,sizeof(modebuf));
	
	if(DuringServerlink)
		MySQL_FlushInserts(thread,SQL_BUFFER_CONDATA,SQL_BUFFER_CONDATA_NO_MODES);
	
	/* Updates the Threads Channel Management */
	UpdateChannelInformation3(thread,av[3],NULL);
	//> :secure.deepirc.net SJOIN 1179011136 #services + :@SecureServ
	
	/*
> :xlracing.de.eu.deepirc.net SJOIN 1227028545 #deinemamarauchtkoks :@shock 
buf = @shock 
nbuf = @shock 
added nick = @shock
USERADD
Nick: shock - Mode: o
*/

/*
> :gemil.de.eu.deepirc.net SJOIN 1219666853 #Deep-Partypoker.net +r :@Guest193323 
BUF = 
buf = gemil.de.eu.deepirc.net SJOIN 1219666853 #Deep-Partypoker.net +r :@Guest193323 
nbuf = gemil.de.eu.deepirc.net SJOIN 1219666853 #Deep-Partypoker.net +r :@Guest193323 
added nick = gemil.de.eu.deepirc.net
USERADD
added nick = SJOIN
USERADD
added nick = 1219666853
USERADD
added nick = #Deep-Partypoker.net
USERADD
added nick = +r
USERADD
added nick = :@Guest193323
USERADD
*/
	if(av[4][0]=='+'){
		if(dynlen(av[4])>1){
			for(i=1;i<ac;i++){ // to avoid the problem above
				if(av[i][0]==':' && b==-1){
					b = i;
					break; //added 8.8.2008 (keine ahnung welche auswirkungen das wirklich hat, wegen obigem SJOIN geadded)
				}
				else if(av[i][0]=='#' && a==-1)
					a = i+1;
			
				else if(a!=-1 && i>=a){
					straddns(modebuf,av[i],sizeof(modebuf));
					straddnc(modebuf,' ',sizeof(modebuf));
				}
			}
			strclean(modebuf);
		
			on_mode(thread,O->nick,av[3],(time_t)stol(av[2]),modebuf);
		}else
			b = 5;
			
		if(b>-1)
			buf = _dstrjoin(' ',av,b);
	}else
		buf = _dstrjoin(' ',av,4);
	
    if(buf!=NULL)
		sjoin(thread,av[3],(time_t)stol(av[2]),buf[0] == ':' ? &buf[1] : buf,0);
    return MOD_CONT;
}

/* Events */

MOD_BOOL orion_event_ping(Threads *thread, size_t ac, char **av)
{
    if (ac < 2)
        return MOD_CONT;
        
    orion_cmd_pong(thread,m->servername, av[1]);
    return MOD_CONT;
}

MOD_BOOL orion_event_pong(Threads *thread, size_t ac, char **av)
{
    if (ac < 2)
        return MOD_CONT;
        
    if(streqnc(av[1],linkserver)){
		on_linkdone(thread);
	}
    return MOD_CONT;
}

MOD_BOOL orion_event_topic_0(Threads *thread, size_t ac, char **av)
{
//> TOPIC #services shock 1194899530 :ROFL (serverlink topic)
//> :shock TOPIC #p2pworld shock 1194899600 :bamm (normal)
    /* :source FTOPIC channel ts setby :topic */
    
    if (ac < 5)
        return MOD_CONT;
        
    topic(thread,av[2],av[1],(time_t)stol(av[3]),&_dstrjoin(' ',av,4)[1]);
    return MOD_CONT;
}

//> TOPIC #services shock 1194899530 :ROFL (serverlink topic)
//> :shock TOPIC #p2pworld shock 1194899600 :bamm (normal)

/* :setter TOPIC #channel :topic */
MOD_BOOL orion_event_topic_1(Threads *thread, size_t ac, char **av)
{
    if (ac < 4)
        return MOD_CONT;
        
    on_topic(thread,&av[0][1],av[2],(time_t)stol(av[4]),&_dstrjoin(' ',av,5)[1]);
    return MOD_CONT;
}

MOD_BOOL orion_event_squit(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
        
	on_squit(thread,av[1]);
    return MOD_CONT;
}

MOD_BOOL orion_event_quit(Threads *thread, size_t ac, char **av)
{
    if (ac < 2)
        return MOD_CONT;
        
    //printf("unreal ircd running\n");
    //printf("quit nick = %s\n",&av[0][1]);
    on_quit(thread,&av[0][1]);
    //printf("quit nick = %s\n",&av[0][1]);
    //printf("unreal ircd running =DDD\n");
    return MOD_CONT;
}

//> :razorblade.hub.eu.deepirc.net KILL fuck :services!too many invalid passwords
MOD_BOOL orion_event_kill(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
    
    if(stoi(mysql_sel_var(thread,"SELECT `faketype` FROM `orion_core_condata` WHERE `nick`='%s'",mysql_quote(thread,av[2])))>0){
		dlog(thread,0,"ERROR On Nick Connection (Connecting \"%s\" failed) - check your config files\n");
		dlog(thread,0,"  If you can't get it working try: \"make clean\" and \"make update\", then rerun the services!\n");
		quit(thread,av[2]);
		orion_exit(thread);
	}else
   		quit(thread,av[2]);
    return MOD_CONT;
}

//:shock KICK #channel user :msg
MOD_BOOL orion_event_kick(Threads *thread, size_t ac, char **av)
{
    if (ac < 4)
        return MOD_CONT;
        
    on_kick(thread,&av[0][1],av[3],av[2]);
    return MOD_CONT;
}

//:shock JOIN #lol
//:shock JOIN 0 - parts all channels!!! (make sure you recognized that!!
MOD_BOOL orion_event_join(Threads *thread, size_t ac, char **av)
{
	if (ac < 3)
        return MOD_CONT;
    
    if(stoi(av[2])==0)
		partall(thread,&av[0][1]);
		
    else
    	_join(thread,0,&av[0][1],av[2],time(NULL));
    	
    return MOD_CONT;
}

MOD_BOOL orion_event_setident(Threads *thread, size_t ac, char **av)
{
	if(ac < 3)
		return MOD_CONT;
		
	on_setident(thread,&av[0][1],av[2]);
    return MOD_CONT;
}

MOD_BOOL orion_event_chgident(Threads *thread, size_t ac, char **av)
{
	if(ac < 4)
		return MOD_CONT;
		
 	on_chgident(thread,av[2],av[3][0]==':' ? &av[3][1] : av[3]);
    return MOD_CONT;
}

MOD_BOOL orion_event_sethost(Threads *thread, size_t ac, char **av)
{
	if(ac < 2)
		return MOD_CONT;
		
	on_sethost(thread,&av[0][1],av[2][0]==':' ? &av[2][1] : av[2]);
    return MOD_CONT;
}

MOD_BOOL orion_event_chghost(Threads *thread, size_t ac, char **av)
{
	if(ac < 4)
		return MOD_CONT;
		
 	on_chghost(thread,av[2],av[3][0]==':' ? &av[3][1] : av[3]);
    return MOD_CONT;
}

//Inspire :test.chatspike.net NICK 1133992412 Brain synapse.brainbox.winbot.co.uk netadmin.chatspike.net ~brain +xwsioS 10.0.0.2 :Craig Edwards
//Unreal  :NICK tunix25 3 1089515018 atte python.netsplit.de mafia.de.eu.deepirc.net 0 +x pA2A90BE0.dialin.deepirc.net :Your IRC network is currently listed at irc.netspl
MOD_BOOL orion_event_nick(Threads *thread, size_t ac, char **av)
{
	
	//> NICK DeepQuote 1 1227737593 DeepQuote deepirc.info milkyway.hub.eu.deepirc.net 0 +iowrxBHt Quotes.DeepIRC.net W1qYew== :DeepIRC Quote Bot
	if(ac > 5){
		struct in_addr ia;
        ia.s_addr = decode_ip(av[10]);
        char realname[MAX_REALNAME_LEN];
        int update_done = 0;
        bool webcode	= 0;
		
		bzero(&realname,sizeof(realname));
		dynncpy(realname,&_dstrjoin(' ',av,11)[1],sizeof(realname));
		
		if(strcin(IRC.RegMode,av[8])){
			domysql(thread,
				"UPDATE `orion_ns_aliases` SET "
				"`active`=			'1',"
				"`last_login`=		'%ld',"
				"`last_used`=		'%ld',"
				"`last_realname`=	'%s',"
				"`last_usermask`=	'%s' WHERE `nick`='%s' AND `linked`='0'"
				,time(NULL),time(NULL),mysql_quote(thread,realname),mysql_quote(thread,av[9]),mysql_quote(thread,av[1])
			);
			
			/* This part us used as a Trick - so we don't have to change so much in the cnick() code */
			/* If we updated a user and autoreidentifed him we set "thread->Webform" which avoids that the user
				gets noticed with - "use /msg NickServ identify ... to identify for that nick, since we already
				reidentifed him (we tell cnick() about what happend with that method
			*/
			update_done 		= mysql_affected_rows(MYSQL_CONNECTION_HANDLE);
			if(update_done>0){
				/* Nickserv's sends the user a message that he identified */
				send_lang(thread,languages[m->language],0,ns->nick,av[1],"LOGIN_SUCCESSFUL",m->network,ns->nick,"@",ns->host);	// 0 is defined as NOTICE
				send_lang(thread,languages[m->language],0,Q->nick,av[1],"AUTHED_SUCCESSFUL");										// 0 is defined as NOTICE
				do_logchan(thread,ns->nick,av[1],av[4],av[5],"identified for nick %s",av[1]);
				
				webcode		 	= thread->Webform;
				thread->Webform = 1;
			}
		}

		cnick(thread,
				av[1],	// nick
				NULL,	// vident
				av[4],	// ident
				av[8],  // modes
				(time_t)stol(av[3]),  // time
				av[5],	// realhost
				av[9],	// virtual host
				av[6],  // server
				realname,	// realname
				inet_ntoa(ia), // ip
				0 // fake
			);
			
		if(update_done>0){
			Users *user 	= GetUser(av[1]);
			user->rights 	= stoi(mysql_sel_var(thread,"SELECT `orion_ns_core`.`rights` FROM `orion_ns_core` LEFT OUTER JOIN `orion_ns_aliases` ON (`orion_ns_core`.`authnick`=`orion_ns_aliases`.`display`) WHERE `orion_ns_aliases`.`nick`='%s'",mysql_quote(thread,av[1])));
			/* Update our struct for Ignoring IRCOps > ServicesOperator */
			
			thread->Webform = webcode;
		}
		
    } else {
        chgnick(thread,&av[0][1],av[2]);
    }
    return MOD_CONT;
}

MOD_BOOL orion_event_privmsg(Threads *thread, size_t ac, char **av)
{
    if (ac < 4)
        return MOD_CONT;
        
    on_privmsg(thread,av);
    return MOD_CONT;
}

MOD_BOOL orion_event_part(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
        
    on_part(thread,&av[0][1],av[2]);
    return MOD_CONT;
}

PROTOCOL_BOOL orion_cmd_add_account(Threads *thread, char *uid, char *authnick)
{
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_del_account(Threads *thread, char *uid)
{
    return CMD_NOT_SUPPORTED;
}

//> :shock TOPIC #p2pworld shock 1194899600 :bamm (normal)
PROTOCOL_BOOL orion_cmd_topic(Threads *thread, char *whosets, char *chan, char *whosetit,
                      char *topic, time_t when)
{
    send_cmd(thread,whosets ? whosets : m->servername, "TOPIC %s %s %ld :%s", chan, whosetit, when, topic);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_gline(Threads *thread, char *user, char *host, char *who, time_t when, time_t expires, char *reason)
{
	char rident[MAX_IDENT_LEN];
	char rhost[MAX_HOST_LEN];
	
	bzero(&rident,sizeof(rident));
	bzero(&rhost,sizeof(rhost));
	
	sscanf(host,"%[^@]@%s",rident,rhost);
	
    send_cmd(thread,user ? user : m->servername, "TKL + G %s %s %s %ld %ld :%s", rident, rhost, who ? who : _dynadd("%s!%s@%s",m->nick,m->ident,m->vhost), expires, when, reason);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_ungline(Threads *thread, char *unsetter, char *host, char *user, time_t time_in)
{
	char rident[MAX_IDENT_LEN];
	char rhost[MAX_HOST_LEN];
	
	bzero(&rident,sizeof(rident));
	bzero(&rhost,sizeof(rhost));
	
	sscanf(host,"%[^@]@%s",rident,rhost);
	
    send_cmd(thread,unsetter ? unsetter : m->servername, "TKL - G %s %s %s", rident, rhost, user);
    return CMD_SUPPORTED;
}

/* SQLINE */
PROTOCOL_BOOL orion_cmd_sqline(Threads *thread, char *sender, char *mask, time_t time_in, time_t expire, char *reason)
{
    send_cmd(thread,m->servername, "TKL + Q * %s %s %ld %ld :%s", mask, sender ? sender : _dynadd("%s!%s@%s",m->nick,m->ident,m->vhost), expire, time_in, reason);
    return CMD_SUPPORTED;
}

/* UNSQLINE */
PROTOCOL_BOOL orion_cmd_unsqline(Threads *thread, char *servername, char *nick, char *snick)
{
    if (!nick) {
        return CMD_ERROR;
    }
    send_cmd(thread,servername ? servername : m->servername, "TKL - Q * %s %s", nick, snick ? snick : _dynadd("%s!%s@%s",m->nick,m->ident,m->vhost));
    return CMD_SUPPORTED;
}

/* SZLINE */
PROTOCOL_BOOL orion_cmd_szline(Threads *thread, char *servername, char *mask, char *snick, time_t when, time_t expire, char *reason)
{
    send_cmd(thread,servername ? servername : m->servername, "TKL + Z * %s %s %ld %ld :%s", mask, snick ? snick : _dynadd("%s!%s@%s",m->nick,m->ident,m->vhost), expire, when, reason);
    return CMD_SUPPORTED;
}

/* UNSZLINE */
PROTOCOL_BOOL orion_cmd_unszline(Threads *thread, char *servername, char *mask, char *snick, time_t in_time)
{
    send_cmd(thread,servername ? servername : m->servername, "TKL - Z * %s %s", mask , snick ? snick : _dynadd("%s!%s@%s",m->nick,m->ident,m->vhost));
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_svskill(Threads *thread, char *source, char *user, char *buf)
{
    if (!buf) {
    	return CMD_ERROR;
    }

    if (!source || !user) {
        return CMD_ERROR;
    }

    send_cmd(thread,source, "SVSKILL %s :%s", user, buf);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_kill(Threads *thread, char *source, char *user, char *buf)
{
    if (!buf) {
        return CMD_ERROR;
    }

    if (!source || !user) {
        return CMD_ERROR;
    }

    send_cmd(thread,source, "KILL %s :%s", user, buf);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_rawsvsmode(Threads *thread, char *source, char *nick, char *modes)
{
    send_cmd(thread,source, "SVSMODE %s %s", nick, modes);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_svsmode(Threads *thread, char *source, char *nick, char *modes)
{	
    send_cmd(thread,source, "SVSMODE %s %s", nick, modes);
	orion_svsmode(thread,source,nick,modes);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_svs2mode(Threads *thread, char *source, char *nick, char *modes)
{
	send_cmd(thread,source, "SVS2MODE %s %s", nick, modes);
	orion_svsmode(thread,source,nick,modes);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_nick(Threads *thread, char *nick, char *ident, char *host, char *name, char *modes)
{
	//NICK tunix25 3 1089515018 atte python.netsplit.de mafia.de.eu.deepirc.net 0 +x pA2A90BE0.dialin.deepirc.net :Your IRC network is currently listed at irc.netspl
	
	/*unsigned char ipstr[17];
	bzero(&ipstr,sizeof(ipstr));
	
	dynncpy((char*)ipstr,"127.0.0.1",sizeof(ipstr));*/
	
	//,encode_ip(ipstr)
	
	//-> /////w== == 127.0.0.1 Base 64 encoded (we use it plain - that's faster :>)
    send_cmd(thread,NULL, "NICK %s 1 %ld %s %s %s 0 %s %s /////w== :%s",nick,time(NULL),ident,host,m->servername,modes,host,name);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_mode(Threads *thread, char *source, char *dest, time_t time_in, char *buf)
{
    send_cmd(thread,source, "MODE %s %s %ld", dest, buf, time_in);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_rawmode(Threads *thread, char *source, char *dest, time_t time_in, char *buf)
{
    if (!buf) {
    	return CMD_ERROR;
    }
    
    send_cmd(thread,source ? source : O->nick, "MODE %s %s %ld", dest, buf, time_in);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_kick(Threads *thread, char *source, char *chan, char *user, char *buf)
{
    if (buf) {
        send_cmd(thread,source, "KICK %s %s :%s", chan, user, buf);
    } else {
        send_cmd(thread,source, "KICK %s %s :%s", chan, user, user);
    }
    return CMD_SUPPORTED;
}


PROTOCOL_BOOL orion_cmd_notice(Threads *thread, char *source, char *dest, char *buf)
{
    if (!buf) {
    	return CMD_ERROR;
    }

    send_cmd(thread,source, "NOTICE %s :%s", dest, buf);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_privmsg(Threads *thread, char *source, char *dest, char *buf)
{
    if (!buf) {
        return CMD_ERROR;
    }

    send_cmd(thread,source, "PRIVMSG %s :%s", dest, buf);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_serv_notice(Threads *thread, char *source, char *dest, char *msg)
{
    send_cmd(thread,source, "NOTICE $%s :%s", dest, msg);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_serv_privmsg(Threads *thread, char *source, char *dest, char *msg)
{
    send_cmd(thread,source, "PRIVMSG $%s :%s", dest, msg);
    return CMD_SUPPORTED;
}

/* QUIT */
PROTOCOL_BOOL orion_cmd_quit(Threads *thread, char *source, char *buf)
{
    if (buf) {
        send_cmd(thread,source, "QUIT :%s", buf);
    } else {
        send_cmd(thread,source, "QUIT :Exiting");
    }
    return CMD_SUPPORTED;
}

/* PROTOCTL */

PROTOCOL_BOOL orion_cmd_protoctl(Threads *thread)
{
	return CMD_NOT_SUPPORTED;
}

/* PASS */
PROTOCOL_BOOL orion_cmd_pass(Threads *thread, char *pass)
{
    send_cmd(thread,NULL, "PASS %s", pass);
    return CMD_SUPPORTED;
}

/* SERVER services-dev.chatspike.net password 0 :Description here */
PROTOCOL_BOOL orion_cmd_server(Threads *thread, char *servname, int hop, char *descript)
{
    send_cmd(thread,NULL, "SERVER %s %d :%s", servname, hop, descript);
    return CMD_SUPPORTED;
}

/* PONG */
PROTOCOL_BOOL orion_cmd_pong(Threads *thread, char *servname, char *who)
{
    send_cmd(thread,NULL, "PONG %s", who);
    return CMD_SUPPORTED;
}

/* PING */
PROTOCOL_BOOL orion_cmd_ping(Threads *thread, char *servname, char *who)
{
    send_cmd(thread,NULL, "PING %s", who);
    return CMD_SUPPORTED;
}

/* JOIN */
/* Althought inspircd 3.2 does not need the timestamp others do so
   we get it in the common function call */
PROTOCOL_BOOL orion_cmd_services_join(Threads *thread, char *user, char *channel, time_t chantime, int fake)
{
	char output[BUFSIZE];
	bzero(&output,sizeof(output));
	
	/* Updates the Threads Channel Management */
	UpdateChannelInformation3(thread,channel,NULL);

	if(IRC.HasOwner) 			straddnc(output,IRC.SjoinOwnerPrefix,sizeof(output));
	if(IRC.HasProtect)			straddnc(output,IRC.SjoinProtectPrefix,sizeof(output));
								straddnc(output,IRC.SjoinOpPrefix,sizeof(output));
	if(IRC.SjoinHasIgnoreChar)	straddnc(output,IRC.SjoinIgnoredChar,sizeof(output));
								straddns(output,user,sizeof(output));

    sjoin(thread,channel,thread->c.chan.datecreate != 0 ? thread->c.chan.datecreate : chantime,output,fake);
	main_raw(thread,":%s SJOIN %ld %s :%s",m->servername,thread->c.chan.datecreate != 0 ? thread->c.chan.datecreate : chantime,channel,output);
    return CMD_SUPPORTED;
}
PROTOCOL_BOOL orion_cmd_join(Threads *thread, char *user, char *channel, time_t chantime)
{
	if (!user || !channel) 
    	return CMD_ERROR;
		
    send_cmd(thread,user, "JOIN %s", channel);
    return CMD_SUPPORTED;
}

/* CHGHOST */
PROTOCOL_BOOL orion_cmd_chghost(Threads *thread, char *source, char *nick, char *vhost)
{
    if (!nick || !vhost) {
    	return CMD_ERROR;
 
    }
    send_cmd(thread,source ? source : m->servername,"CHGHOST %s %s", nick, vhost);
    return CMD_SUPPORTED;
}

/* CHGIDENT */
PROTOCOL_BOOL orion_cmd_chgident(Threads *thread, char *source, char *nick, char *vIdent)
{
    if (!nick || !vIdent) {
   		return CMD_ERROR;
    }
    send_cmd(thread,source ? source : m->servername, "CHGIDENT %s %s", nick, vIdent);
    return CMD_SUPPORTED;
}

/* INVITE */
PROTOCOL_BOOL orion_cmd_invite(Threads *thread, char *source, char *chan, char *nick)
{
    if (!source || !chan || !nick) {
   		return CMD_ERROR;
    }

    send_cmd(thread,source, "INVITE %s %s", nick, chan);
    return CMD_SUPPORTED;
}

/* PART */
PROTOCOL_BOOL orion_cmd_part(Threads *thread, char *nick, char *chan, char *buf)
{
    if (!nick || !chan) {
   		return CMD_ERROR;
    }

    if (buf) {
        send_cmd(thread,nick, "PART %s :%s", chan, buf);
    } else {
        send_cmd(thread,nick, "PART %s :Leaving", chan);
    }
    return CMD_SUPPORTED;
}

/* GLOBOPS */
PROTOCOL_BOOL orion_cmd_global(Threads *thread, char *source, char *buf)
{
    if (!buf) {
   		return CMD_ERROR;
    }

	send_cmd(thread,source ? source : m->servername, "GLOBOPS :%s", buf);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_wallops(Threads *thread, char *source, char *fmt)
{
    send_cmd(thread,source ? source : O->nick, "WALLOPS :%s", fmt);
    return CMD_SUPPORTED;
}

/* SQUIT */
PROTOCOL_BOOL orion_cmd_squit(Threads *thread, char *servname, char *message)
{
    if (!servname || !message) {
   		return CMD_ERROR;
    }

    send_cmd(thread,m->servername, "SQUIT %s :%s", servname, message);
    return CMD_SUPPORTED;
}

/* SVSO */
PROTOCOL_BOOL orion_cmd_svso(Threads *thread, char *source, char *nick, char *flag)
{
    return CMD_NOT_SUPPORTED;
}

/* NICK <newnick>  */
PROTOCOL_BOOL orion_cmd_chg_nick(Threads *thread, char *oldnick, char *newnick)
{
    if (!oldnick || !newnick) {
   		return CMD_ERROR;
    }

    send_cmd(thread,oldnick, "NICK %s", newnick);
    return CMD_SUPPORTED;
}

/* SVSNICK */
PROTOCOL_BOOL orion_cmd_svsnick(Threads *thread, char *source, char *oldnick, char *newnick, time_t when)
{
    if (!oldnick || !newnick) {
   		return CMD_ERROR;
    }
    send_cmd(thread,source ? source : m->servername, "SVSNICK %s %s %ld", oldnick, newnick, when);
    return CMD_SUPPORTED;
}

/* SVSHOLD - set */
PROTOCOL_BOOL orion_cmd_svshold(Threads *thread, char *nick)
{
    /* Not supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

/* SVSHOLD - release */
PROTOCOL_BOOL orion_cmd_release_svshold(Threads *thread, char *nick)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

/* UNSGLINE */
PROTOCOL_BOOL orion_cmd_unsgline(Threads *thread, char *mask)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

/* SGLINE */
PROTOCOL_BOOL orion_cmd_sgline(Threads *thread, char *mask, char *reason)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_unban(Threads *thread, char *name, char *nick)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

/* SVSMODE channel modes */
PROTOCOL_BOOL orion_cmd_svsmode_chan(Threads *thread, char *name, char *mode, char *nick)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_svsjoin(Threads *thread, char *source, char *nick, char *chan, time_t date)
{
    send_cmd(thread,source ,"SVSJOIN %s %s", nick, chan);
	return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_svspart(Threads *thread, char *source, char *nick, char *chan)
{
    send_cmd(thread,source, "SVSPART %s %s", nick, chan);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_sajoin(Threads *thread, char *source, char *nick, char *chan, time_t date)
{
    send_cmd(thread,source ,"SVSJOIN %s %s", nick, chan);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_sapart(Threads *thread, char *source, char *nick, char *chan)
{
    send_cmd(thread,source, "SVSPART %s %s", nick, chan);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_swhois(Threads *thread, char *source, char *who, char *mask)
{
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_eob(Threads *thread)
{
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_jupe(Threads *thread, char *jserver, char *who, char *reason)
{
    char rbuf[256];

    snprintf(rbuf, sizeof(rbuf), "Juped by %s%s%s", who,
             reason ? ": " : "", reason ? reason : "");

    send_cmd(thread,m->servername, "SERVER %s * %d :%s",jserver, 0/*serverid*/, rbuf);
	new_server(thread, me_server, jserver, rbuf, SERVER_JUPED, NULL);
    return CMD_SUPPORTED;
}

/* GLOBOPS - to handle old WALLOPS */
PROTOCOL_BOOL orion_cmd_global_legacy(Threads *thread, char *source, char *fmt)
{
    send_cmd(thread,source ? source : O->nick, "GLOBOPS :%s", fmt);
    return CMD_SUPPORTED;
}

/*  ************************************************************************************************  */
/**
 * Here are all functions which tell us and all modules that a serverlink occured
 * And it tells us if the server is currently linking or finished linking (important for sockscontrol)
 **/

/* This adds Servers to the server tree */
MOD_BOOL orion_event_server(Threads *thread, size_t ac, char **av)
{
//> SERVER klebestreifen.dev.hub.deepirc.net 1 :U2308-FhiXeOoE-2 DeepIRC Network Developement HUB
//> :klebestreifen.dev.hub.deepirc.net SERVER services.deepirc.net 1 :Flood Server
	if(ac < 4)
		return MOD_CONT;

	int occurance = 0;
    if (streq(av[2], "1")) {
		if(uplink[0]=='\0')
			dynncpy(uplink,av[1],sizeof(uplink));
			
		occurance = getoccurance_x(av[3],'-',2);
		if(occurance == -2)
        	addServer(thread, NULL, av[1], av[2], _dstrjoin(' ',av,4), NULL);
        else{
			occurance++;
        	addServer(thread, NULL, av[1], av[2], _dstrjoin(' ',av,4), &av[3][occurance]);
		}
        //on_link(thread,m->servername,av[1]);
    } else {
        addServer(thread, &av[0][1], av[2], av[3], &_dstrjoin(' ',av,4)[1], NULL);
    }

    return MOD_CONT;
}

/* Called from serviceslink */
PROTOCOL_BOOL orion_cmd_initialize(Threads *thread){
	if(!connected){		/* Proof if we are already connected, we aren't so... */
		/* We start connecting to the target UnrealIRCd, providing the Event we are doing */
		orion_cmd_connect(thread,1);
		
		/* Tell all modules we STARTED linking and they are able to perform their settings now */
		on_serviceslink(thread);
		
		/* We identify ourselves as connected to the target */
		connected = true;
	}
    return CMD_SUPPORTED;
}

/* Called from orion_cmd_initialize */
PROTOCOL_BOOL orion_cmd_connect(Threads *thread, bool how)
{
	/* We send the connect password */
	orion_cmd_pass(thread,m->linkpass);
		
	/* We tell UnrealIRCd what types of it's protocol we are able to understand */
	main_raw(thread,"PROTOCTL NICKv2 SJOIN SJOIN2 SJ3 NOQUIT VHP NICKIP");
		
	/* We introduce our server */
	orion_cmd_server(thread,m->servername, 0, m->description);

	/* We do the primary connect, means connecting all services clients and BotServ Bots */
	if(how == true)
		_ExecuteEvent(NULL,NULL,"on_connect_primary");

    return CMD_SUPPORTED;
}

/* ON PROTOCTL - we should be already connected (connected = true) */
MOD_BOOL orion_event_started(Threads *thread, size_t ac, char **av)
{
	if(ac < 2)
		return MOD_CONT;
		
	if(!connected){
		/* We connect to target UnrealIRCd and provide an Event for our modules, that we started communication */
		orion_cmd_connect(thread,1);
		
		/* Tell all modules we STARTED linking and they are able to perform their settings now */
		on_serviceslink(thread);
		
		/* We identify ourselves as connected to the target */
		connected = true;
	}
	
	/* We tell our uplink we finished synchronising */
	main_raw(thread,":%s EOS",m->servername);
	return MOD_CONT;
}

/* This means to us that a server is currently linking to the network */
MOD_BOOL orion_event_linking(Threads *thread, size_t ac, char **av)
{
	if(ac < 8)
		return MOD_CONT;
	
//> :razorblade.dev.hub.deepirc.net SMO o :(link) Link razorblade.dev.hub.deepirc.net -> CServe.deepirc.net[@127.0.0.1.21828] established
//> :razorblade.dev.hub.deepirc.net SMO o :(sync) Link CServe.deepirc.net -> razorblade.dev.hub.deepirc.net is now synced [secs: 1 recv: 3.764 sent: 2.482]
	
	char linkserver_tmp[MAX_SERVER_LEN+20];
	char *server[2];
	
	/* A server is linking to the network */
	if(streqnc(&av[3][1],"(link)")){
		dynncpy(linkserver_tmp,av[7],sizeof(linkserver_tmp));
		strsplit(linkserver_tmp,server,'[');
		
		/* We tell all modules a server (linkserver_tmp) started linking (linkserver = linkserver_tmp) */
		on_link(thread,linkserver_tmp,NULL);
	}
	
	/* A server successfully finished linking and synced it's link */
	else if(streqnc(&av[3][1],"(sync)")){
		if(DuringServerlink) MySQL_FlushInserts(thread,SQL_BUFFER_CONDATA,SQL_BUFFER_SJOIN); // Flush our Cache
		
		/* We provide the information, that linking (for "linkserver") was done - we event it */
		on_linkdone(thread);
	}
    return MOD_CONT;
}

/* This is provided when services linking finished successfully */
MOD_BOOL orion_event_services_link_done(Threads *thread, size_t ac, char **av)
{
	if(DuringServerlink)
		MySQL_FlushInserts(thread,SQL_BUFFER_CONDATA,SQL_BUFFER_SJOIN); // Flush our Cache
	
	/* We report our NETINFO */						
	main_raw(thread,"NETINFO %d %ld %s",stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_core_condata`")),time(NULL),_dstrjoin(' ',av,3));

	/* We add the services server to the servers tree (we do that when we finished linking) */
	me_server = addServer(thread, serv_uplink->server, m->servername, "1", m->description, NULL);

	/* Event, connect secondary - connects bots, sets mlock etc. */
	_ExecuteEvent(NULL,NULL,"on_connect_secondary");
	
	/* We report that our linking is done now */
	on_serviceslink_done(thread);	/* This means that linking !FOR US! is done */							//- reactivates Sockscontrol
	on_linkdone(thread);			/* This is executed whenever a serverlink completed successfully */		//- reactivates Sockscontrol
	return MOD_CONT;
}

/* On receiving a EndOfSync */
MOD_BOOL orion_event_eos(Threads *thread, size_t ac, char **av)
{
    Servers *s;
    s = findserver(thread, servlist, &av[0][1]);
    /* If we found a server with the given source, that one just
     * finished bursting. If there was no source, then our uplink
     * server finished bursting. -GD
     */
    if (!s && serv_uplink)
        s = serv_uplink;
        
    return MOD_CONT;
}

/*  ************************************************************************************************  */

#define START_OF_RAW 4
/**
 * @brief raw:  To process outgoing raw messages
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param string The outgoing Raw-message
 * @param len The length of the created Raw-message
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL orion_cmd_raw(Threads *thread, char *param[]){
	PROTOCOL_BOOL analyzer;
	
	//:M PRIVMSG #operserv: raw :operserv svsmode M +SjFzq
	//:operserv SVSKILL M :lalala
	//:M MODE  +a
	//Tue Oct 19 22:47:14 2004 > :OperServ GLOBOPS :M used RAW command for svsmode
	//Tue Oct 19 22:47:14 2004 > :Global PRIVMSG #services :M used RAW command for :operserv
	printf("raw handler\n");

	int paramvalue = 0;

	/* SVSMODE VIA RAW */
		//:operserv SVSMODE shock +d 1
		//operserv SVSMODE shock +d 1
	if(streqnc(param[START_OF_RAW+1],"svsmode"))       analyzer = do_svsmode(thread,&param[START_OF_RAW+0][1],param[START_OF_RAW+2],param[START_OF_RAW+3]);
	else if(streqnc(param[START_OF_RAW+0],"svsmode"))  analyzer = do_svsmode(thread,m->servername,param[START_OF_RAW+1],param[START_OF_RAW+2]);

	/* SVS2MODE VIA RAW */
	//
	else if(streqnc(param[START_OF_RAW+1],"svs2mode")) analyzer = do_svs2mode(thread,&param[START_OF_RAW+0][1],param[START_OF_RAW+2],param[START_OF_RAW+3]);
	else if(streqnc(param[START_OF_RAW+0],"svs2mode")) analyzer = do_svs2mode(thread,m->servername,param[START_OF_RAW+1],param[START_OF_RAW+2]);

	/* SVSKILL VIA RAW */
		// > :shock PRIVMSG O :raw :operserv SVSKILL shock :lalala
		// < :CServe.deepirc.net SVSKILL  ::lalala
		// > :shock PRIVMSG ircdos :raw :sockscontrol SVSKILL shock :hey alter was geht bei dir
	else if(streqnc(param[START_OF_RAW+1],"svskill")){
		char *reason = _dstrjoin(' ',param,START_OF_RAW+2);
		analyzer = do_svskill(thread,&param[START_OF_RAW+0][1],param[START_OF_RAW+2],reason[0]==':' ? &reason[1] : reason);
	}
		
		// > :shock PRIVMSG O :raw SVSKILL shock :lalala
	else if(streqnc(param[START_OF_RAW+0],"svskill")){
		printf("found svskill\n");
		char *reason = _dstrjoin(' ',param,START_OF_RAW+2);
		analyzer = do_svskill(thread,m->servername,param[START_OF_RAW+1],reason[0]==':' ? &reason[1] : reason);
	}

	/* SVSNICK VIA RAW */
		// > :shock PRIVMSG O :raw :operserv SVSNICK shock shockzZ
	else if(streqnc(param[START_OF_RAW+1],"svsnick")) analyzer = do_svsnick(thread,&param[START_OF_RAW+0][1],param[START_OF_RAW+2],param[START_OF_RAW+3]);
		// > :shock PRIVMSG O :raw SVSNICK shock shockzZ
	else if(streqnc(param[START_OF_RAW+0],"svsnick")) analyzer = do_svsnick(thread,m->servername,param[START_OF_RAW+1],param[START_OF_RAW+2]);

	/* SVSJOIN VIA RAW */
		//:operserv svsjoin Nick #chan
	else if(streqnc(param[START_OF_RAW+0],"svsjoin")) analyzer = do_svsjoin(thread,0,m->servername,param[START_OF_RAW+1],param[START_OF_RAW+2],time(NULL));
	else if(streqnc(param[START_OF_RAW+1],"svsjoin")) analyzer = do_svsjoin(thread,0,&param[START_OF_RAW+0][1],param[START_OF_RAW+2],param[START_OF_RAW+3],time(NULL));

	/* SVSPART VIA RAW */
	else if(streqnc(param[START_OF_RAW+0],"svspart")) analyzer = do_svspart(thread,m->servername,param[START_OF_RAW+1],param[START_OF_RAW+2]);
	else if(streqnc(param[START_OF_RAW+1],"svspart")) analyzer = do_svspart(thread,&param[START_OF_RAW+0][1],param[2],param[3]);
	
	/* NORMAL IRC (RFC) RAW */
	else{
		while(param[paramvalue])
			paramvalue++;
			
		if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_core_condata` WHERE `nick`='%s'",mysql_quote(thread,&param[START_OF_RAW+0][1])))>0){
				on_protocol(thread,&IRC,paramvalue-START_OF_RAW,param);
				main_raw(thread,_djoin(' ',param));
				return CMD_EXECUTED;
		}	
		else
			return CMD_ERROR;
	}
	return analyzer;
}

/* Copied from Anope */
bool orion_valid_nick(Threads *thread, char *nick)
{
    if (streqnc("ircd", nick)) {
        return false;
    }
    if (streqnc("irc", nick)) {
        return false;
    }
    return true;
}

bool orion_valid_chan(Threads *thread, char *chan){
    if (strchr(chan, ':')) {
      return false;
    }
    return true;
}

/* check if +f mode is valid for the ircd */
/* borrowed part of the new check from channels.c in Unreal */
bool orion_flood_mode_check(Threads *thread, char *value){
    char *dp, *end;
    /* NEW +F */
    char xbuf[256], *p, *p2, *x = xbuf + 1;
    int v;

    if (!value) {
        return false;
    }

    if (*value != ':'
        && (strtoul((*value == '*' ? value + 1 : value), &dp, 10) > 0)
        && (*dp == ':') && (*(++dp) != 0) && (strtoul(dp, &end, 10) > 0)
        && (*end == 0)) {
        return true;
    } else {
        /* '['<number><1 letter>[optional: '#'+1 letter],[next..]']'':'<number> */
        strncpy(xbuf, value, sizeof(xbuf));
        p2 = strchr(xbuf + 1, ']');
        if (!p2) {
            return false;
        }
        *p2 = '\0';
        if (*(p2 + 1) != ':') {
            return false;
        }
        for (x = strtok(xbuf + 1, ","); x; x = strtok(NULL, ",")) {
            /* <number><1 letter>[optional: '#'+1 letter] */
            p = x;
            while (isdigit(*p)) {
                p++;
            }
            if ((*p == '\0')
                || !((*p == 'c') || (*p == 'j') || (*p == 'k')
                     || (*p == 'm') || (*p == 'n') || (*p == 't'))) {
                continue;       /* continue instead of break for forward compatability. */
            }
            *p = '\0';
            v = atoi(x);
            if ((v < 1) || (v > 999)) {
                return false;
            }
            p++;
        }
        return true;
    }
}
