/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 194                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-16 18:33:19 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file qline.c
 *   @brief This is the Module file for QLINE (Server Side) Command.
 *
 *   It contains all functions which are neccessary to process and use 
 *   the QLINE command.
 *
 *   The QLINE File is used to process QLINES sent from Servers
 *   and to Servers. (Protocol-Process-File -> PPF)
 *   (Qline is used to handle Sqlines - to set/remove SQLINES)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"

/**
 * @brief do_sqline:  Sqlines a nick (Outgoing - Send ONLY)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param snick The ServiceNick which sends the SQLINE
 * @param nickmask The Nick to sqline
 * @param in_expiretime The Time when the SQLINE should expire
 * @param in_hubtime The Time when the SQLINE was set
 * @param reason The Reason which we want to provide for the IRCOp's
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_sqline(Threads *thread, char *snick, char *nickmask, time_t in_expiretime, time_t in_hubtime, const char *reason, ...){
	PROTOCOL_BOOL analyzer;
	
	/* Deklarate Variables */
	va_list va;
	char tmp[BUFFER+1];
	char addedby[MAX_NICK_LEN];
	char ident_at_host[MAX_IDENT_LEN+1+MAX_HOST_LEN];
	time_t expire = 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&tmp,sizeof(tmp));
	bzero(&addedby,sizeof(addedby));
	bzero(&ident_at_host,sizeof(ident_at_host));

	/* Join reason string */
	va_start(va, reason);
	vsnprintf(tmp, sizeof(tmp)-1, reason, va);
	va_end(va);
	
	/* Clean the delay in our Duration */
							expire = in_expiretime-in_hubtime;
	if(in_expiretime > 0) 	expire+=time(NULL);
	else 					expire = 0;

	/*
	 * Build response.
	 */
	
	sscanf(snick,"%[^!]!%s",addedby,ident_at_host);
	//addedby = NICK ohne !ident@host.net ....
	
	if(reason){
		/* Analyze Command - is it supported from the IRCd !? */
		analyzer = orion_WRCtranslate_cmd_sqline(thread, addedby, nickmask, in_hubtime, in_expiretime, tmp);
		if(analyzer >= CMD_NOT_SUPPORTED)
			return analyzer;

		/* Command supported .. we go on running it */
		domysql(thread,"INSERT INTO `orion_core_lines` (`type`,`mask`,`addedby`,`duration`,`reason`) VALUES ('Q','%s','%s','%ld','%s') ON DUPLICATE KEY UPDATE "
		"`addedby`=VALUES(`addedby`),"
		"`duration`=VALUES(`duration`),"
		"`reason`=VALUES(`reason`)"
		,mysql_quote(thread,nickmask),mysql_quote(thread,addedby),expire,mysql_quote(thread,tmp));
		
		if(streqnc(addedby,O->nick) || streqnc(addedby,SC->nick) || streqnc(addedby,Q->nick))		
			domysql(thread,"INSERT INTO `orion_os_sqlines` (`qline`,`duration`,`addedby`,`reason`) VALUES ('%s','%ld','%s','%s') ON DUPLICATE KEY UPDATE "
			"`duration`=VALUES(`duration`),"
			"`addedby`=VALUES(`addedby`),"
			"`reason`=VALUES(`reason`)"
			,mysql_quote(thread,nickmask),expire,mysql_quote(thread,addedby),mysql_quote(thread,tmp));
	}else return CMD_ERROR;

	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief on_sqline:  Sqlines a nick (Incoming - Receive ONLY)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param snick The ServiceNick which sent the SQLINE
 * @param nickmask The Nick which was sqlined
 * @param in_expiretime The Time when the SQLINE should expire
 * @param in_hubtime The Time when the SQLINE was set
 * @param in_reason The Reason which for the SQLINE
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_sqline(Threads *thread, char *snick, char *nickmask, time_t in_expiretime, time_t in_hubtime, char *in_reason){
	/* Declare variables */
	char addedby[MAX_NICK_LEN];
	char ident_at_host[MAX_IDENT_LEN+1+MAX_HOST_LEN];
	time_t expire = 0;

	/* Initialize variables to NULL */
	bzero(&addedby,sizeof(addedby));
	bzero(&ident_at_host,sizeof(ident_at_host));

	/* Clean the delay in our Duration */
	expire = in_expiretime == 0 ? 0 : (time(NULL)+(in_expiretime-in_hubtime));

	/* Get the Qlines setter */
	sscanf(snick,"%[^!]!%s",addedby,ident_at_host);

	if(DuringServerlink)
		MySQL_FlushInserts(thread,SQL_BUFFER_SJOIN,SQL_BUFFER_SJOIN); /* Flush Inserts - Since on serverlink or not; the SJOIN
																Event comes before TKL! */
																
	domysql(thread,"INSERT INTO `orion_core_lines` (`type`,`mask`,`addedby`,`duration`,`reason`) VALUES ('Q','%s','%s','%ld','%s') ON DUPLICATE KEY UPDATE "
	"`addedby`=VALUES(`addedby`),"
	"`duration`=VALUES(`duration`),"
	"`reason`=VALUES(`reason`)"
	,mysql_quote(thread,nickmask),mysql_quote(thread,addedby),expire,mysql_quote(thread,in_reason));
	
	if(streqnc(addedby,O->nick) || streqnc(addedby,SC->nick) || streqnc(addedby,Q->nick))
		domysql(thread,"INSERT INTO `orion_os_sqlines` (`qline`,`duration`,`addedby`,`reason`) VALUES ('%s','%ld','%s','%s') ON DUPLICATE KEY UPDATE "
		"`duration`=VALUES(`duration`),"
		"`addedby`=VALUES(`addedby`),"
		"`reason`=VALUES(`reason`)"
		,mysql_quote(thread,nickmask),expire,mysql_quote(thread,addedby),mysql_quote(thread,in_reason));

	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */

/**
 * @brief do_unsqline:  UnSqlines a nick
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param snick The servicenick which should execute the unsqline
 * @param servername The server from which the unsqline should be sent
 * @param nickmask The nick to unsqline
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_unsqline(Threads *thread, char *snick, char* servername, char *nickmask){
	PROTOCOL_BOOL analyzer;
	
	/*
	 * Build response.
	 */
	 
	if(nickmask){
		/* Analyze Command - is it supported from the IRCd !? */
		analyzer = orion_WRCtranslate_cmd_unsqline(thread, servername, nickmask, snick);
		if(analyzer >= CMD_NOT_SUPPORTED)
			return analyzer;
			
		/* Command supported .. we go on running it */
		domysql(thread,
			"DELETE `orion_os_sqlines`,`orion_core_lines` FROM `orion_os_sqlines` "
			"LEFT OUTER JOIN `orion_core_lines` ON (`orion_core_lines`.`mask`=`orion_os_sqlines`.`qline` AND `orion_core_lines`.`type`='Q') "
			"WHERE `orion_os_sqlines`.`qline`='%s'"
			,mysql_quote(thread,nickmask));
	}else return CMD_ERROR;
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief on_unsqline:  Deletes an Sqline
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param snick The nick which used the unsqline
 * @param servername The server to which the unsqline should be sent
 * @param nickmask The unsqlined target nick
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_unsqline(Threads *thread, char *snick, char* servername, char *nickmask){
	/* Declare variables */
	deepSQL getSQL;
	
	/* Initialize variables to NULL */
	bzero(&getSQL,sizeof(getSQL));
    
	if(DuringServerlink)
		MySQL_FlushInserts(thread,SQL_BUFFER_SJOIN,SQL_BUFFER_SJOIN); /* Flush Inserts - Since on serverlink or not; the SJOIN
																Event comes before TKL! */
	
	domysql(thread,"DELETE FROM `orion_core_lines` WHERE `type`='Q' AND `mask`='%s'",mysql_quote(thread,nickmask));
	
	/* MySQL */
	getSQL.res    = doSelectMySQL(thread,"SELECT `qline`,`addedby`,`duration`,`reason` FROM `orion_os_sqlines` WHERE `qline`='%s'",mysql_quote(thread,nickmask));
	getSQL.fields = mysql_num_rows(getSQL.res);
	getSQL.row    = mysql_fetch_row(getSQL.res);
	
	if(streqnc(getSQL.row[1],O->nick))
		do_sqline(thread,_dynadd("%s!%s@%s",O->nick,O->ident,O->host),getSQL.row[0],stol(getSQL.row[2]),time(NULL),getSQL.row[3]);
		
	else if(streqnc(getSQL.row[1],Q->nick))
		do_sqline(thread,_dynadd("%s!%s@%s",Q->nick,Q->ident,Q->host),getSQL.row[0],stol(getSQL.row[2]),time(NULL),getSQL.row[3]);
		
	else if(streqnc(getSQL.row[1],SC->nick))
		do_sqline(thread,_dynadd("%s!%s@%s",SC->nick,SC->ident,SC->host),getSQL.row[0],stol(getSQL.row[2]),time(NULL),getSQL.row[3]);
		
	mysql_free_result_myr(getSQL.res);
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */
